#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "upu3image4xi.py">
#<abstract4xi abstract = "UPU 3D image management"/>
#<refer4xi reference = "image3d4xi.py"/>
#<verify4xi verification = "10" target = "upu3image4xi5test.py"/>
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'UPU3Image4Xi',                        # NACK 5.31  p76
            
            'copy3image4upu',                      # NACK 5.3.3 p88
            'copy3imagetobuffer4upu',              # NACK 5.3.4 p90
            'copy3buffertoimage4upu',              # NACK 5.3.4 p92
            'create3image4upu',                    # ACK 5.31  p78      

            'get3imageinfo4upu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get3supportedimageformats4upu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get3imageinfo4upu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre3copyimage4upu',             # ACK modernized 
            'manoeuvre3copyimagetobuffer4upu',     # ACK modernized        
            'manoeuvre3copybuffertoimage4upu',     # ACK modernized
            'manoeuvre3mapimage4upu',              # NACK modernized                                           
            'manoeuvre3readimage4upu',             # ACK modernized
            'manoeuvre3writeimage4upu',            # ACK modernized            
            'map3image4upu',                       # NACK 5.3.5 p94 
            
            'read3image4upu',                      # ACK 5.3.3 p84  
            'retain3image4upu',                    # ACK 5.4.1 p99
            'release3image4upu',                   # ACK 5.4.1 p99

            'set3imagecallback4upu',               # ACK modernized
            'set3image4upu',                       # ACK modernized nb: with memorymap features you can process 3D images as files!                    

            'trace3image4upu',                     # ACK modernized 
                                    
            'wait3image4upu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write3image4upu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from upu2buffer4xi import UPU2Buffer4Xi
from upu2buffer4xi import create2buffer4upu
from upu2buffer4xi import create2subbuffer4upu
from upu2buffer4xi import get2bufferinfo4upu
from upu2buffer4xi import release2buffer4upu
from upu2buffer4xi import retain2buffer4upu
from upu2buffer4xi import set2buffer4upu
from upu2buffer4xi import trace2buffer4upu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from upu2commandqueue4xi import UPU2CommandQueue4Xi
from upu2commandqueue4xi import create2commandqueue4upu 
from upu2commandqueue4xi import flush2commandqueue4upu
from upu2commandqueue4xi import finish2commandqueue4upu
from upu2commandqueue4xi import get2commandqueueinfo4upu
from upu2commandqueue4xi import release2commandqueue4upu
from upu2commandqueue4xi import retain2commandqueue4upu
from upu2commandqueue4xi import set2commandqueue4upu
from upu2commandqueue4xi import trace2commandqueue4upu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from upu2context4xi import UPU2Context4Xi 
from upu2context4xi import create2context4upu
from upu2context4xi import get2context4upu
from upu2context4xi import release2context4upu
from upu2context4xi import retain2context4upu
from upu2context4xi import set2context4upu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from upu2device4xi import UPU4Xi
from upu2device4xi import init2upu4xi 
from upu2device4xi import set2upu4xi
from upu2device4xi import get2upu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from upu2event4xi import UPU2Event4Xi
from upu2event4xi import create2event4upu
from upu2event4xi import get2eventinfo4upu
from upu2event4xi import release2event4upu
from upu2event4xi import retain2event4upu
from upu2event4xi import set2eventcallback4upu 
from upu2event4xi import set2event4upu
from upu2event4xi import trace2event4upu
from upu2event4xi import wait2event4upu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global upu3image4xi                                                     # 3D-IMAGES FOR UPU
global upu3trace4xi                                                     # TRACE

global upu3buffer4xi                                                    # BUFFER FOR UPU

global control3readimage4upu                                            # COMMAND AND CONTROL TO READ  3D IMAGES WITH UPU  
global state3read4upu                                                   # STATE FOR COMMAND AND CONTROL
global max3readstate4upu                                                # MAX FOR STATE
global ack3readinfo4upu                                                 # ACK FOR IMAGE INFO
global message3readget4upu                                              # QUEUE GET
global message3readput4upu                                              # QUEUE PUT
global message3readreceive4upu                                          # PIPE RECEIVE
global message3readsend4upu                                             # PIPE SEND

global control3writeimage4upu                                           # COMMAND AND CONTROL TO WRITE 3D IMAGES WITH UPU 
global state3write4upu                                                  # STATE FOR COMMAND AND CONTROL
global max3writestate4upu                                               # MAX FOR STATE
global ack3writeinfo4upu                                                # ACK FOR IMAGE INFO
global message3writeget4upu                                             # QUEUE GET
global message3writeput4upu                                             # QUEUE PUT
global message3writereceive4upu                                         # PIPE RECEIVE
global message3writesend4upu                                            # PIPE SEND

global control3copyimage4upu                                            # COMMAND AND CONTROL TO COPY 3D IMAGES WITH UPU 
global state3copyimage4upu                                              # STATE FOR COMMAND AND CONTROL
global max3copyimagestate4upu                                           # MAX FOR STATE
global ack3copyimageinfo4upu                                            # ACK FOR IMAGE INFO
global message3copyimageget4upu                                         # QUEUE GET
global message3copyimageput4upu                                         # QUEUE PUT
global message3copyimagereceive4upu                                     # PIPE RECEIVE
global message3copyimagesend4upu                                        # PIPE SEND

global control3copyimagetobuffer4upu                                    # COMMAND AND CONTROL TO COPY 3D IMAGES TO BUFFER WITH UPU 
global state3copyimagetobuffer4upu                                      # STATE FOR COMMAND AND CONTROL
global max3copyimagetobufferstate4upu                                   # MAX FOR STATE
global ack3copyimagetobufferinfo4upu                                    # ACK FOR IMAGE INFO
global message3copyimagetobufferget4upu                                 # QUEUE GET
global message3copyimagetobufferput4upu                                 # QUEUE PUT
global message3copyimagetobufferreceive4upu                             # PIPE RECEIVE
global message3copyimagetobuffersend4upu                                # PIPE SEND

global control3copybuffertoimage4upu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 3D IMAGE WITH UPU 
global state3copybuffertoimage4upu                                      # STATE FOR COMMAND AND CONTROL
global max3copybuffertoimagestate4upu                                   # MAX FOR STATE
global ack3copybuffertoimageinfo4upu                                    # ACK FOR IMAGE INFO
global message3copybuffertoimageget4upu                                 # QUEUE GET
global message3copybuffertoimageput4upu                                 # QUEUE PUT
global message3copybuffertoimagereceive4upu                             # PIPE RECEIVE
global message3copybuffertoimagesend4upu                                # PIPE SEND                      

global control3mapimage4upu                                            # COMMAND AND CONTROL TO READ  3D IMAGES WITH UPU  
global state3map4upu                                                   # STATE FOR COMMAND AND CONTROL
global max3mapstate4upu                                                # MAX FOR STATE
global ack3mapinfo4upu                                                 # ACK FOR IMAGE INFO
global message3mapget4upu                                              # QUEUE GET
global message3mapput4upu                                              # QUEUE PUT
global message3mapreceive4upu                                          # PIPE RECEIVE
global message3mapsend4upu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace3image4upu" usage = "
#
#   trace3upu = None
#   try:
#       trace3upu = trace3image4upu() 
#       print('(Step 15B/25) OF UPU3IMAGE4XI5TEST - TRACE OF 3D IMAGE FOR UPU: ', trace3upu)  
#   except:
#       print('(Step 15Z1/25) OF UPU3IMAGE4XI5TEST *** FAILED IN TRACE OF 3D IMAGE FOR UPU: ', trace3upu)     
#">
# -------------------------------------------------------------------------------------------------   
def trace3image4upu(task4trace3upu=None):  

    '''
    creates trace of 3d image for upu
    '''
    
    global upu3trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace3image4upu" 
    method2print = "trace3image4upu"

    message2print = "{(Step 1A/33) started" + "; trace3image4upu-started: " + str(trace3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace3image4upu at 0x00000000031B1848> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace3upu-to-trace3image4upu: " + str(task4trace3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code3ack  = ' '
    code3nack = ' '    
    
    upu3trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace3upu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY UPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace3upu = {}                                                                                            
            
        task4trace3upu['CLASS']   = "trace3image4upu"                                                               # UPUIZE ZPU TRACE!
        task4trace3upu['METHOD']  = "trace3image4upu" 
        task4trace3upu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace3image4upu-failed " + str(upu3trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace3upu-set-by-trace3image4upu to: " + str(task4trace3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4upu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}',
        #  'CLASS': 'trace3image4upu',
        #  'METHOD': 'trace3image4upu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}
    
        code3ack = code3ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY UPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace3upu-try-by-trace3image4upu: " + str(task4trace3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    # 

    try:
        upu3trace4xi ['CLASS']  = task4trace3upu['CLASS']                                                      # UPUIZE UPU TRACE!
        upu3trace4xi ['METHOD']  = task4trace3upu['METHOD']
        upu3trace4xi ['MESSAGE'] = task4trace3upu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; upu3trace4xi-set-by-trace3image4upu to: " + str(upu3trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4upu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}',
        #  'CLASS': 'trace3image4upu',
        #  'METHOD': 'trace3image4upu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}


        code3ack = code3ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; upu-trace-failed-by-trace3image4upu: " + str(task4trace3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code3nack = code3nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE UPU TRACE JATKA
        # =================================
        class2print   = task4trace3upu['CLASS']                                                                # UPUIZE UPU TRACE!
        method2print  = task4trace3upu['METHOD']
        
        message2print = task4trace3upu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # UPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace3image4upu : trace3image4upu ; (Step 2Z0/33) failed; trace3image4upu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}
        
        code3ack = code3ack + '3C-'         
    except:
        class2print = "trace3image4upu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace3image4upu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-upu-trace-failed-by-trace3image4upu: " + str(task4trace3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code3nack = code3nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace3image4upu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace3image4upu"                
        message2print = "{(Step 3D/33) performed" + "; upu-trace-performed-by-trace3image4upu: " + str(task4trace3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4upu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
        #   'CLASS': 'trace3image4upu', 
        #   'METHOD': 'trace3image4upu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}
        
        code3ack = code3ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; upu-trace-failed-by-trace3image4upu: " + str(task4trace3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code3nack = code3nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; upu3trace4xi-returning-by-trace3image4upu: " + str(upu3trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4upu-failed {}; time: 06:24:23 10/10/11 Pacific Daylight Time}',
    #  'CLASS': 'trace3image4upu',
    #  'METHOD': 'trace3image4upu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}

    
    return upu3trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set3imagecallback4upu" usage = "
#
#   print('(Step 14A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SET 3D IMAGE CALLBACK OF UPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback3image = set3imagecallback4upu()                                                                                                 
#       print('(Step 14B/700) OF UPU3IMAGE4XI5TEST - SET 3D IMAGE CALLBACK OF UPU DEVICE TO: ', response4setcallback3image)   
#       # 
#   except:
#       print('(Step 14Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SET 3D IMAGE CALLBACK OF UPU DEVICE ', time4xi())  
# 
#">
# ------------------------------------------------------------------------------------------------- 
 
def set3imagecallback4upu(task4set3imagecallback=None):

    '''
    registers callback function to upu3image4xi
    '''
    global upu3image4xi
    
    class2print = "set3imagecallback4upu" 
    method2print = "set3imagecallback4upu"
    
    message2print = "{(Step 1A/70) started" + "; set3imagecallback4upu-started: " + str(set3imagecallback4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function trace3image4upu at 0x00000000031B1848>
    
    try:
    
        upu3image4xi ['CALLBACK'] = trace3image4upu
        
        message2print = "{(Step 1B/70) set callback3upu" + "; imagecallback4upu-set-by-set3imagecallback4upu: " + str(upu3image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-upu3image4xi-returning-by-set3imagecallback4upu: " + str(upu3image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace3image4upu at 0x00000000031B1848> 
        
        return upu3image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback3image-failed-by-set3imagecallback4upu: " + str(callback3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace3image4upu-returning-by-set3imagecallback4upu: " + str(trace3image4upu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace3image4upu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create3image4upu" usage = "
#     
#   print('=' *33)          
#   print('(Step 2A/700) OF UPU3IMAGE4XI5TEST - TRYING TO CREATE 3D IMAGE4UPU ', time4xi())      
#   print('=' *33)        
#
#   try:
#       response4create3Dimage = create3image4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES   
#       print('(Step 2B/700) OF UPU3IMAGE4XI5TEST - CREATED 3D IMAGE4UPU ', time4xi())
#       keys3response = response4create3Dimage.keys()
#       for key3response in sorted(keys3response):
#           print(key3response, ' : ', response4create3Dimage[key3response])
            # 
#   except:
#       print('(Step 2Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO CREATE 3D IMAGE4UPU *** ', time4xi())
#
#">
# -------------------------------------------------------------------------------------------------   
def create3image4upu(task4imagecreate3upu=None):

    '''
    creates 3D image for UPU
    '''
    
    global upu3image4xi                                                                                              # 3D image of UPU  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create3image4upu" 
    method2print = "create3image4upu"

    message2print = "{(Step 1A/70) started" + "; create3image4upu-started: " + str(create3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create3image4upu at 0x0000000004874648> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate3upu-to-create3image4upu: " + str(task4imagecreate3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code3ack  = ' '
    code3nack = ' '    
    
    upu3image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate3upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO UPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate3upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate3upu-created-by-create3image4upu to: " + str(task4imagecreate3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function3callback4upu = trace3image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1D/70) initialized" + "; function3callback4upu-created-by-create3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace3image4upu at 0x000000000486C848>
            
            code3ack  = code3ack + '1D-'
           
            args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "create3image4upu"                                                           # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "create3image4upu"                                                           # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args3callback-created-by-create3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create3image4upu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}',
            #  'CLASS': 'create3image4upu',
            #  'METHOD': 'create3image4upu'}; time: 04:45:25 10/10/11 Pacific Daylight Time}

            code3ack  = code3ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate3upu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate3upu['FUNCTION3CALLBACK'] = function3callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code3ack  = code3ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate3upu-created-by-create3image4upu to: " + str(task4imagecreate3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4upu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}',
            #  'CLASS': 'create3image4upu',
            #  'METHOD': 'create3image4upu'},
            #  'FUNCTION3CALLBACK': <function trace3image4upu at 0x000000000486C848>}; time: 04:45:25 10/10/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-create3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-create3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 04:45:25 10/10/11 Pacific Daylight Time}
                 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-create3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagecreate3upu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 3A/70) trying" + "; context3upu-trying-by-create3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3upu-got-by-create3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}', ...
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3upu-failed-by-create3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagecreate3upu ['CONTEXT'] = context3upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3upu-trying-by-create3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3upu-got-by-create3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3upu-failed-by-create3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
                
            task4imagecreate3upu ['DEVICE'] = device3upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate3upu ['CODE3ERROR'] = 0             
            code3ack  = code3ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3upu-trying-by-create3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3upu-got-by-create3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000482FD30> 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3upu-failed-by-create3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imagecreate3upu ['QUEUE'] = queue3upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3upu-trying-by-create3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3upu-got-by-create3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>) 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3upu-failed-by-create3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagecreate3upu ['PIPE'] = pipe3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3upu-trying-by-create3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3upu-got-by-create3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 04:45:25 10/10/11 Pa ...
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3upu-failed-by-create3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imagecreate3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3upu-trying-by-create3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3upu-got-by-create3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 04:45:25 10/10/11 Pacific  ...
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3upu-failed-by-create3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagecreate3upu ['BUFFER'] = bufferinfo3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3upu-trying-by-create3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo3upu-got-by-create3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 04:45:25 10/10/11 Pacific Daylight Time}', 'CLASS ...
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3upu-failed-by-create3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagecreate3upu ['EVENT'] = eventinfo3upu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags3upu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags3upu-trying-by-create3image4upu: " + str(flags3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags3upu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags3upu-set-by-create3image4upu to: " + str(flags3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags3upu-failed-by-create3image4upu: " + str(flags3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagecreate3upu ['FLAGS'] = flags3upu             
            
            # ---------------------------------------------------------------------------------------------
            # (12) DEPTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            depth3upu = 0

            quantity3core = 1                                                                      # ORIGIN 
            quantity3skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity3layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity3frame = 1                                                                     # FRAME
            quantity3extra = 4                                                                     # CONTROLS FOR GUI ETC...
            depth3layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 12A/70) trying" + "; depth3upu-trying-by-create3image4upu: " + str(depth3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                depth3upu = int(depth3layer * (quantity3core + quantity3skeleton + quantity3layers + quantity3frame + quantity3extra))
                depth3upu = 2 * depth3upu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 12B/70) set" + "; depth3upu-set-by-create3image4upu to: " + str(depth3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800
                 
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; depth3upu-failed-by-create3image4upu: " + str(depth3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagecreate3upu ['DEPTH'] = depth3upu
                        
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width3upu = 0

            quantity3core = 1                                                                      # ORIGIN 
            quantity3skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity3layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity3frame = 1                                                                     # FRAME
            quantity3extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width3layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width3upu-trying-by-create3image4upu: " + str(width3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width3upu = int(width3layer * (quantity3core + quantity3skeleton + quantity3layers + quantity3frame + quantity3extra))
                width3upu = 2 * width3upu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width3upu-set-by-create3image4upu to: " + str(width3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width3upu-failed-by-create3image4upu: " + str(width3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagecreate3upu ['WIDTH'] = width3upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height3upu = 0
            quantity3core = 1                                                                      # ORIGIN 
            quantity3skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity3layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity3frame = 1                                                                     # FRAME
            quantity3extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity3total = quantity3core + quantity3skeleton + quantity3layers + quantity3frame + quantity3extra
            height3layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height3upu-trying-by-create3image4upu: " + str(height3upu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height3upu = height3layer * quantity3total
                
                height3upu = 2 * height3upu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height3upu-set-by-create3image4upu to: " + str(height3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height3upu-failed-by-create3image4upu: " + str(height3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '15Z-'
             
                        
            task4imagecreate3upu ['HEIGHT'] = height3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch3upu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3upu-trying-by-create3image4upu: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3upu-set-by-create3image4upu to: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3upu-failed-by-create3image4upu: " + str(rowpitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imagecreate3upu ['ROWPITCH'] = rowpitch3upu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) SLICEPITCH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # the size in bytes of each 2D slice in the 3D image

            slicepitch3upu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 17A/70) trying" + "; slicepitch3upu-trying-by-create3image4upu: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                slicepitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)           # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 17B/70) set" + "; slicepitch3upu-set-by-create3image4upu to: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '17B-' 
            except:
                message2print = "{(Step 17Z/70) failed" + "; slicepitch3upu-failed-by-create3image4upu: " + str(slicepitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '17Z-'
        
            task4imagecreate3upu ['SLICEPITCH'] = slicepitch3upu     
                        
            # ---------------------------------------------------------------------------------------------
            # (19) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer3upu = None
            size3upu = int(width3upu * height3upu)                                                                  # CALCULATED MEMORY
            if size3upu == 0:
                size3upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 19A/70) trying" + "; pointer3upu-trying-by-create3image4upu: " + str(pointer3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3upu = Arena4Xi(size3upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 19B1/70) got" + "; arena3upu-got-by-create3image4upu: " + str(arena3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000042A2E48> 
                
                size3arena = arena3upu.size                                                                         # 
                              
                message2print = "{(Step 19B2/70) got" + "; size3arena-got-by-create2buffer4upu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer3arena = arena3upu.buffer
                
                message2print = "{(Step 19B3/70) got" + "; buffer3arena-got-by-create2buffer4upu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000004295348>
                
                code3ack  = code3ack + '19B-' 
                
                if size3arena < size3upu:
                    size3upu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 19C/70) set" + "; size3upu-set-by-create2buffer4upu to: " + str(size3upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '19C-'
                else:
                    pass
                    
                pointer3upu = buffer3arena
                message2print = "{(Step 19D/70) set" + "; pointer3upu-set-by-create2buffer4upu to: " + str(pointer3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000004295348>
                
                code3ack  = code3ack + '19D-'       
            except:
                message2print = "{(Step 19Z/70) failed" + "; pointer3upu-failed-by-create3image4upu: " + str(pointer3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '19Z-'
        
            task4imagecreate3upu ['POINTER'] = pointer3upu                            
            task4imagecreate3upu ['SIZE']    = size3upu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (20) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format3upu = {}
           
            message2print = "{(Step 20A/70) trying" + "; format3upu-trying-by-create3image4upu: " + str(format3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format3upu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 20B/70) set" + "; format3upu-set-by-create3image4upu to: " + str(format3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code3ack  = code3ack + '20B-'
                
                format3upu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 20C/70) set" + "; format3upu-set-by-create3image4upu to: " + str(format3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
                
                code3ack  = code3ack + '20C-'                
                 
            except:
                message2print = "{(Step 20Z/70) failed" + "; format3upu-failed-by-create3image4upu: " + str(format3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '20Z-'
        
            task4imagecreate3upu ['FORMAT'] = format3upu                  

                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate3upu-yielded-by-create3image4upu: " + str(task4imagecreate3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys3task = task4imagecreate3upu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys3task-using-by-create3image4upu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE3ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA3CALLBACK',
    #  'DEPTH',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION3CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'SLICEPITCH',
    #  'WIDTH']; time: 05:07:09 10/10/11 Pacific Daylight Time}

  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 3D IMAGE FOR UPU! 
    # ---------------------------------------------------------------------------------------------    
    upu3image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; upu3image4xi-initialized-by-create3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    message2print = "{(Step 21A0/70) try" + "; data3callback-try-by-create3image4upu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA3CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['DATA3CALLBACK'] = task4imagecreate3upu['DATA3CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data3callback-created-by-create3image4upu to: " + str(upu3image4xi['DATA3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create3image4upu-failed ; time: 05:07:09 10/10/11 Pacific Daylight Time}',
        #  'CLASS': 'create3image4upu',
        #  'METHOD': 'create3image4upu'}; time: 05:07:09 10/10/11 Pacific Daylight Time}

        code3ack  = code3ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data3callback-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function3callback-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION3CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['FUNCTION3CALLBACK'] = task4imagecreate3upu['FUNCTION3CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function3callback-created-by-create3image4upu to: " + str(upu3image4xi['FUNCTION3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace3image4upu at 0x0000000004876848>  
        code3ack  = code3ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function3callback-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['PLATFORM'] = task4imagecreate3upu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create3image4upu to: " + str(upu3image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:07:09 10/10/11 Pacific Daylight Time}
  
        code3ack  = code3ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context3upu-try-by-create3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['CONTEXT'] = task4imagecreate3upu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create3image4upu to: " + str(upu3image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:07:09 10/10/11 Pacific Daylight Time}', ' ...
        
        
        code3ack  = code3ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code3nack  = code3nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['DEVICE'] = task4imagecreate3upu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create3image4upu to: " + str(upu3image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
        
        code3ack  = code3ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code3error-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['CODE3ERROR'] = task4imagecreate3upu['CODE3ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code3error-created-by-create3image4upu to: " + str(upu3image4xi['CODE3ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        
        code3ack  = code3ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code3error-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['QUEUE'] = task4imagecreate3upu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create3image4upu to: " + str(upu3image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x00000000048330B8>
         
        code3ack  = code3ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['PIPE'] = task4imagecreate3upu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create3image4upu to: " + str(upu3image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 384>, <read-write PipeConnection, handle 388>) 
        
        code3ack  = code3ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-upu3image4xi-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['COMMANDQUEUE']  = task4imagecreate3upu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create3image4upu to: " + str(upu3image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 05:07:09 10/10/11 Pa ...
        
        code3ack  = code3ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code3nack  = code3nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['BUFFER']  = task4imagecreate3upu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create3image4upu to: " + str(upu3image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 05:07:09 10/10/11 Pacific  ...
        
        code3ack  = code3ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['EVENT']  = task4imagecreate3upu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create3image4upu to: " + str(upu3image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 05:07:09 10/10/11 Pacific Daylight Time}', 'CLASS ...
        
        code3ack  = code3ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['FLAGS']  = task4imagecreate3upu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create3image4upu to: " + str(upu3image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0 
        
        code3ack  = code3ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '31Z1-'     

    message2print = "{(Step 32A/70) try" + "; depth-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DEPTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu3image4xi['DEPTH']  = task4imagecreate3upu['DEPTH']  
 
        message2print = "{(Step 32H/70) initialized" + "; depth-created-by-create3image4upu to: " + str(upu3image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '32H-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; depth-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '32Z1-'   
        
    message2print = "{(Step 34A/70) try" + "; width-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu3image4xi['WIDTH']  = task4imagecreate3upu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create3image4upu to: " + str(upu3image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu3image4xi['HEIGHT']  = task4imagecreate3upu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create3image4upu to: " + str(upu3image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['ROWPITCH']  = task4imagecreate3upu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create3image4upu to: " + str(upu3image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        
        code3ack  = code3ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; slicepitch-try-by-create3image4upu: None" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) SLICEPITCH - REQUIRED JATKA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['SLICEPITCH']  = task4imagecreate3upu['SLICEPITCH']  
 
        message2print = "{(Step 37B/70) initialized" + "; slicepitch-created-by-create3image4upu to: " + str(upu3image4xi['SLICEPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        
        code3ack  = code3ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; slicepitch-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '37Z1-'
        
    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create3image4upu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['POINTER']  = task4imagecreate3upu['POINTER']  
 
        message2print = "{(Step 38B/70) initialized" + "; pointer-created-by-create3image4upu to: " + str(upu3image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x000000000429E348> 
        
         
        code3ack  = code3ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; pointer-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; format-try-by-create3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['FORMAT']  = task4imagecreate3upu['FORMAT']  
 
        message2print = "{(Step 39B/70) initialized" + "; format-created-by-create3image4upu to: " + str(upu3image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        code3ack  = code3ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; format-yielded-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '39Z1-'


                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    upu3image4xi ['ACK2RETURN']  = code3ack
    upu3image4xi ['NACK2RETURN'] = code3nack  
    
    message2print = "{(Step 70/70) returning" + "; upu3image4xi-returning-by-create3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # {'PIPE': (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>),
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-17B-19B-19D-20B-20C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-32H-34H-35H-36B-37B-38B-39B-',
    #  'ROWPITCH': 800,
    #  'CODE3ERROR': 0,
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 05:18:11 10/10/ ...
    
    
    return upu3image4xi
            
#</function4xi> 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "set3image4upu" usage = "
#    
#   print('(Step 4B/33) OF UPU3IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform3xi = {}
#   try:
#       platform3xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4UPU5TEST - GOT PLATFORMIDS4XI', platform3xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4UPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#    
#   print('(Step 2A/33) OF DEVICE4UPU5TEST - TRYING TO INITIALIZE DEVICE4UPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device3upu = init2upu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4UPU5TEST - INITIALIZED DEVICE4UPU TO: ', device3upu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4UPU5TEST *** FAILED TO INITIALIZE DEVICE4UPU', time4xi()) 
#
#   print('(Step 3A/33) OF UPU3IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4UPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context3upu = create2context4upu()                                                                               
#       print('(Step 3B/33) OF UPU3IMAGE4XI5TEST - CREATED CONTEXT4UPU: ', context3upu)                                  
#   except:
#       print('(Step 3Z/33) OF UPU3IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4UPU', time4xi()) 
#          
#   task4set3image = {}
#   task4set3image  ['CONTEXT'] = context3upu  
# 
#   ... other data for set operation ...
#    
#   
# ---------------------------------------------------------------------------------------------
# (11) SET 3D IMAGE                         
# ---------------------------------------------------------------------------------------------          
#   response4set3Dimage = {} 
#   print('(Step 11A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SET 3D IMAGE4UPU ', time4xi())   
#   try:                               
#       response4set3Dimage = set3image4upu(task4set3image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF UPU3IMAGE4XI5TEST - SET 3D IMAGE4UPU ', time4xi())
#       keys3response = response4set3Dimage.keys()
#       for key3response in sorted(keys3response):
#           print(key3response, ' : ', response4set3Dimage[key3response])
#           # 
#   except:
#       print('(Step 11Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SET 3D IMAGE4UPU *** ', time4xi()) 
#        
#
#">
# -------------------------------------------------------------------------------------------------   
def set3image4upu(task4imageset3upu=None):

    '''
    sets 3D images for UPU
    '''
    global upu3image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set3image4upu"
    method2print = "set3image4upu"

    message2print = "{(Step 1A/33) started" + "; set3image4upu-started: " + str(set3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set3image4upu at 0x00000000032196C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset3upu-to-set3image4upu: " + str(task4imageset3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    #1
    # {'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>),
    #  'CODE3ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004431B38>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:35:43 10/10/11 Pacific Dayl ...
    
    #2
    # {'HEIGHT': 1000,
    #  'WIDTH': 1100,
    #  'DEPTH': 900,
    #  'ROWPITCH': 901,
    #  'SLICEPITCH': 902,
    #  'DIMENSION': 3}; time: 06:03:06 10/10/11 Pacific Daylight Time}
    
            
    code3ack  = ' '
    code3nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) set" + "; upu3image4xi-set-by-set3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4upu-skipped-by-set3image4upu: " + str(create3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create3image4upu at 0x0000000003219648>                                
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 1E/70) set" + "; upu3image4xi-set-by-set3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data3callback-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA3CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['DATA3CALLBACK'] = task4imageset3upu['DATA3CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data3callback-set-by-set3image4upu to: " + str(upu3image4xi['DATA3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code3ack  = code3ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data3callback-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function3callback-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION3CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['FUNCTION3CALLBACK'] = task4imageset3upu['FUNCTION3CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function3callback-set-by-set3image4upu to: " + str(upu3image4xi['FUNCTION3CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code3ack  = code3ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function3callback-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['PLATFORM'] = task4imageset3upu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set3image4upu to: " + str(upu3image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #   
        code3ack  = code3ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code3nack  = code3nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context3upu-try-by-set3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['CONTEXT'] = task4imageset3upu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set3image4upu to: " + str(upu3image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:35:43 10/10/11 Pacific Daylight Time}', ...
        
        code3ack  = code3ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code3nack  = code3nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['DEVICE'] = task4imageset3upu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set3image4upu to: " + str(upu3image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
        
        code3ack  = code3ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code3error-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['CODE3ERROR'] = task4imageset3upu['CODE3ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code3error-set-by-set3image4upu to: " + str(upu3image4xi['CODE3ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
         
        code3ack  = code3ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code3error-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['QUEUE'] = task4imageset3upu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set3image4upu to: " + str(upu3image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000004431B38>
        code3ack  = code3ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['PIPE'] = task4imageset3upu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set3image4upu to: " + str(upu3image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>)
        
        code3ack  = code3ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-upu3image4xi-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['COMMANDQUEUE']  = task4imageset3upu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set3image4upu to: " + str(upu3image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 05:35:43 10/10/11 Pa ...
        
        code3ack  = code3ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['BUFFER']  = task4imageset3upu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set3image4upu to: " + str(upu3image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
         
        code3ack  = code3ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set3image4upu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['EVENT']  = task4imageset3upu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set3image4upu to: " + str(upu3image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded 
        code3ack  = code3ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['FLAGS']  = task4imageset3upu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set3image4upu to: " + str(upu3image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded
        code3ack  = code3ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '31Z1-'     

    message2print = "{(Step 32A/70) try" + "; depth-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (32) DEPTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu3image4xi['DEPTH']  = task4imageset3upu['DEPTH']  
 
        message2print = "{(Step 32H/70) set" + "; depth-set-by-set3image4upu to: " + str(upu3image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded
        
        code3ack  = code3ack + '32H-'
    except: 
        message2print = "{(Step 32Z1/70) yielded" + "; depth-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '32Z1-'   

    message2print = "{(Step 34A/70) try" + "; width-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu3image4xi['WIDTH']  = task4imageset3upu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set3image4upu to: " + str(upu3image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded 
        
        code3ack  = code3ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code3nack  = code3nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set3image4upu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        upu3image4xi['HEIGHT']  = task4imageset3upu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set3image4upu to: " + str(upu3image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded  
        code3ack  = code3ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['ROWPITCH']  = task4imageset3upu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set3image4upu to: " + str(upu3image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code3nack  = code3nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; slicepitch-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) SLICEPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['SLICEPITCH']  = task4imageset3upu['SLICEPITCH']  
 
        message2print = "{(Step 37B/70) set" + "; slicepitch-set-by-set3image4upu to: " + str(upu3image4xi['SLICEPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; slicepitch-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code3nack  = code3nack + '37Z1-'
        
    message2print = "{(Step 38A/70) try" + "; pointer-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['POINTER']  = task4imageset3upu['POINTER']  
 
        message2print = "{(Step 38B/70) set" + "; pointer-set-by-set3image4upu to: " + str(upu3image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; pointer-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code3nack  = code3nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; format-try-by-set3image4upu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        upu3image4xi['FORMAT']  = task4imageset3upu['FORMAT']  
 
        message2print = "{(Step 39B/70) set" + "; format-set-by-set3image4upu to: " + str(upu3image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # yielded
        
        code3ack  = code3ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; format-yielded-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        code3nack  = code3nack + '39Z1-'
        
         
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    upu3image4xi ['ACK2RETURN']  = code3ack
    upu3image4xi ['NACK2RETURN'] = code3nack 
    
    message2print = "{(Step 70/70) returning" + "; upu3image4xi-returning-by-set3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1:
    # ACK2RETURN  :   23B-24B-25B-26B-27B-28B-
    # BUFFER  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 05:35:43 10/10/11 ...
    # CODE3ERROR  :  0
    # COMMANDQUEUE  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 05: ...
    # CONTEXT  :  {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 05:35:43 10/10/11 Pacific Dayl ...
    # DATA3CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create3image4upu-failed ; time: 05:35:43 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4 ...
    # DEPTH  :  800
    # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHL ...
    # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 05:35:43 10/10/11 Pacific Daylight Time ...
    # FLAGS  :  0
    # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # FUNCTION3CALLBACK  :  <function trace3image4upu at 0x0000000003213848>
    # HEIGHT  :  800
    # NACK2RETURN  :   21Z1-22Z2-22Z3-29Z1-30Z1-31Z1-32Z1-34Z1-35Z1-36Z1-37Z1-38Z1-
    # PIPE  :  (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>)
    # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
    # POINTER  :  <mmap.mmap object at 0x000000000427B3F8>
    # QUEUE  :  <queue4xi.Queue4Xi object at 0x0000000004431B38>
    # ROWPITCH  :  800
    # SLICEPITCH  :  800
    # WIDTH  :  800
    
    #2:
    # {'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 'ACK2RETURN': ' 32H-34H-35H-36B-37B-', 'ROWPITCH': 901, 'CODE3ERROR': 0, 'WIDTH': 1100,
    # 'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 06:24:23 10/10/1 ...
    #  'DEPTH': 900, 'FLAGS': 0, 
    # 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 06:24:23 10/10/11 Pacific Dayli ...
    
    
    return upu3image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy3image4upu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY3IMAGE4UPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGE4UPU ', time4xi())    
#   response4copy3image = {} 
#   task4copy3image = None
#   try:                               
#       response4copy3image = copy3image4upu(task4copy3image)                                                                                                          
#       print('(Step 16B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY3IMAGE4UPU : ', response4copy3image)
#       queue3copy4upu      = response4copy3image['QUEUE']
#       channel3copypublish4upu = response4copy3image['PUBLISHERCHANNEL']
#       channel3copyconsume4upu = response4copy3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGE4UPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3IMAGE4UPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4UPU ', time4xi())     
#   try:                               
#       message3copyimagesend4upu = "<copy4xi>copy3copy4upu</copy4xi>"
#       channel3copypublish4upu = response4copy3image['PUBLISHERCHANNEL']
#       channel3copypublish4upu.send(message3copyimagesend4upu)
#       print("(Step 17B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3copypublish4upu, " MESSAGE3SEND4UPU: " , message3copyimagesend4upu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE3IMAGE4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3copyimage4upu(queue3copy4upu, channel3copyconsume4upu, channel3copypublish4upu)
#       print("(Step 18D/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3copyimage4upu)
#   except:
#       print("(Step 18Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4UPU ', time4xi())     
#   try:
#       channel3copyconsume4upu = response4copy3image['CONSUMERCHANNEL']                              
#       message3copyimagereceive4upu = channel3copyconsume4upu.recv()
#       print("(Step 19B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copyconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                  
#       # <roger4xi>roger3copy4upu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGE4UPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3IMAGE4UPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4UPU ', time4xi())     
#   try:                               
#       message3copyimagesend4upu = "<copy4xi>copy3copy4upu</copy4xi>"
#       channel3copypublish4upu = response4copy3image['PUBLISHERCHANNEL']
#       channel3copypublish4upu.send(message3copyimagesend4upu)
#       print("(Step 20B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3copypublish4upu, " MESSAGE3SEND4UPU: " , message3copyimagesend4upu)                                                                                                
#       # <copy4xi>copy3copy4upu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE3IMAGE4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3copyimage4upu(queue3copy4upu, channel3copyconsume4upu, channel3copypublish4upu)
#       print("(Step 21B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3copyimage4upu)
#   except:
#       print("(Step 21Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGE4UPU ', time4xi())     
#   try:
#       channel3copyconsume4upu = response4copy3image['CONSUMERCHANNEL']                              
#       message3copyimagereceive4upu = channel3copyconsume4upu.recv()
#       print("(Step 22B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copyconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                  
#       # <copy4xi>copy3copy4upu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGE4UPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3copy4upu.put(path3upu)             
#       print("(Step 24B/700) OF UPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3upu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE3IMAGE4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3copyimage4upu(queue3copy4upu, channel3copyconsume4upu, channel3copypublish4upu)
#       print("(Step 25B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3copyimage4upu)
#   except:
#       print("(Step 25Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3copyimagereceive4upu = channel3copyconsume4upu.recv()
#       print("(Step 26D/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copyconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                   
#       # <ack4xi>ack3imageinfo4upu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE3IMAGE4UPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3copyimage4upu(queue3copy4upu, channel3copyconsume4upu, channel3copypublish4upu)
#       print("(Step 27B/700) OF UPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4UPU ", response3manoeuvre4upu)
#   except:
#       print("(Step 27Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def copy3image4upu(task4imagecopy3upu=None):

    '''
    copies 3D images for UPU
    '''
    global upu3image4xi                                                                                             # 3D-IMAGES FOR UPU  
    global control3copyimage4upu                                                                                    # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy3image4upu"
    method2print = "copy3image4upu"

    message2print = "{(Step 0A/33) started" + "; copy3image4upu-started: " + str(copy3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy3image4upu at 0x000000000321A748>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy3upu-to-copy3image4upu: " + str(task4imagecopy3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                       # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 0C/70) created" + "; upu3image4xi-created-by-copy3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create3image4upu-skipped-by-copy3image4upu: " + str(copy3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  <function copy3image4upu at 0x000000000321A748>                               
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 0E/70) set" + "; upu3image4xi-set-by-copy3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; upu3image4xi-yielded-by-copy3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy3upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy3upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy3upu-created-by-copy3image4upu to: " + str(task4imagecopy3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4upu-created-by-copy3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace3image4upu at 0x0000000003214848> 
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "copy3image4upu"                                                            # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "copy3image4upu"                                                            # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-copy3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy3image4upu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}', 
            #   'CLASS': 'copy3image4upu', 
            #   'METHOD': 'copy3image4upu'}; time: 07:14:55 10/10/11 Pacific Daylight Time}
            

            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy3upu['DATA3CALLBACK'] = args3callback                                                    # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy3upu['FUNCTION3CALLBACK'] = function3callback4upu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy3upu-created-by-copy3image4upu to: " + str(task4imagecopy3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-copy3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-copy3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-copy3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagecopy3upu['PLATFORM'] = platform3xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 3A/70) trying" + "; context3upu-trying-by-copy3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3upu-got-by-copy3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3upu-failed-by-copy3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagecopy3upu ['CONTEXT'] = context3upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3upu-trying-by-copy3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3upu-got-by-copy3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3upu-failed-by-copy3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imagecopy3upu ['DEVICE'] = device3upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo3upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo3upu-trying-by-copy3image4upu: " + str(imagesourceinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo3upu-got-by-copy3image4upu: " + str(imagesourceinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo3upu-failed-by-copy3image4upu: " + str(imagesourceinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imagecopy3upu ['IMAGESOURCE'] = imagesourceinfo3upu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo3upu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo3upu-trying-by-copy3image4upu: " + str(imagedestinationinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo3upu-got-by-copy3image4upu: " + str(imagedestinationinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                # note: source and destination will be the same !             
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo3upu-failed-by-copy3image4upu: " + str(imagedestinationinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'          
                      
            task4imagecopy3upu ['IMAGEDESTINATION'] = imagedestinationinfo3upu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue3upu-trying-by-copy3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue3upu-got-by-copy3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue3upu-failed-by-copy3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagecopy3upu ['QUEUE'] = queue3upu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe3upu-trying-by-copy3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe3upu-got-by-copy3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe3upu-failed-by-copy3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
                
            task4imagecopy3upu ['PIPE'] = pipe3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo3upu-trying-by-copy3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo3upu-got-by-copy3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo3upu-failed-by-copy3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagecopy3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo3upu-trying-by-copy3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                               # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo3upu-got-by-copy3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo3upu-failed-by-copy3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagecopy3upu ['BUFFER'] = bufferinfo3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo3upu-trying-by-copy3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo3upu-got-by-copy3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo3upu-failed-by-copy3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagecopy3upu ['EVENT'] = eventinfo3upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist3upu-trying-by-copy3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist3upu-set-by-copy3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist3upu-failed-by-copy3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagecopy3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity3upu-trying-by-copy3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity3upu-set-by-copy3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-copy3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imagecopy3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source3upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source3upu-trying-by-copy3image4upu: " + str(origin4source3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source3upu['X'] = 0 
                origin4source3upu['Y'] = 0  
                origin4source3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source3upu-set-by-copy3image4upu to: " + str(origin4source3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source3upu-failed-by-copy3image4upu: " + str(origin4source3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagecopy3upu ['SOURCEORIGIN'] = origin4source3upu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination3upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination3upu-trying-by-copy3image4upu: " + str(origin4destination3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination3upu['X'] = 100 
                origin4destination3upu['Y'] = 100  
                origin4destination3upu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination3upu-set-by-copy3image4upu to: " + str(origin4destination3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination3upu-failed-by-copy3image4upu: " + str(origin4destination3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imagecopy3upu ['DESTINATIONORIGIN'] = origin4destination3upu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy3upu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy3upu-trying-by-copy3image4upu: " + str(region4copy3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy3upu['X'] = 10 
                region4copy3upu['Y'] = 10  
                region4copy3upu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy3upu-set-by-copy3image4upu to: " + str(region4copy3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code3ack  = code3ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy3upu-failed-by-copy3image4upu: " + str(region4copy3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '16Z-'
            
            task4imagecopy3upu ['REGION'] = region4copy3upu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagecopy3upu.keys()
        message2print = "{(Step 20/70) using" + "; keys3task-using-by-copy3image4upu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
        
 
        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                            # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4upu-created-by-copy3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "copy3image4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "copy3image4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-copy3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code3ack  = code3ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy3upu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy3upu['FUNCTION3CALLBACK'] = function3callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy3upu-created-by-copy3image4upu to: " + str(task4imagecopy3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-copy3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-copy3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-copy3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4imagecopy3upu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 23A/70) trying" + "; context3upu-trying-by-copy3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3upu-got-by-copy3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3upu-failed-by-copy3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imagecopy3upu ['CONTEXT'] = context3upu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3upu-trying-by-copy3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3upu-got-by-copy3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3upu-failed-by-copy3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imagecopy3upu ['DEVICE'] = device3upu
        
        if 'IMAGESOURCE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo3upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo3upu-trying-by-copy3image4upu: " + str(sourceimageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo3upu-got-by-copy3image4upu: " + str(sourceimageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo3upu-failed-by-copy3image4upu: " + str(sourceimageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imagecopy3upu ['IMAGESOURCE'] = sourceimageinfo3upu         
        
        if 'IMAGEDESTINATION' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo3upu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo3upu-trying-by-copy3image4upu: " + str(destinationimageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo3upu-got-by-copy3image4upu: " + str(destinationimageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo3upu-failed-by-copy3image4upu: " + str(destinationimageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'          
                      
            task4imagecopy3upu ['IMAGEDESTINATION'] = destinationimageinfo3upu    
                                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue3upu-trying-by-copy3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue3upu-got-by-copy3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue3upu-failed-by-copy3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imagecopy3upu ['QUEUE'] = queue3upu

        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe3upu-trying-by-copy3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe3upu-got-by-copy3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe3upu-failed-by-copy3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
                
            task4imagecopy3upu ['PIPE'] = pipe3upu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo3upu-trying-by-copy3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo3upu-got-by-copy3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo3upu-failed-by-copy3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imagecopy3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
        
        
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo3upu-trying-by-copy3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                           # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo3upu-got-by-copy3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo3upu-failed-by-copy3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imagecopy3upu ['BUFFER'] = bufferinfo3upu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo3upu-trying-by-copy3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo3upu-got-by-copy3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo3upu-failed-by-copy3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imagecopy3upu ['EVENT'] = eventinfo3upu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist3upu-trying-by-copy3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist3upu-set-by-copy3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist3upu-failed-by-copy3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imagecopy3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity3upu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity3upu-trying-by-copy3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity3upu-set-by-copy3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-copy3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
                    
            task4imagecopy3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                             
                    
        if 'SOURCEORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin3upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin3upu-trying-by-copy3image4upu: " + str(source4origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin3upu['X'] = 0 
                source4origin3upu['Y'] = 0  
                source4origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin3upu-set-by-copy3image4upu to: " + str(source4origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin3upu-failed-by-copy3image4upu: " + str(source4origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imagecopy3upu ['SOURCEORIGIN'] = source4origin3upu
             
        if 'DESTINATIONORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin3upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin3upu-trying-by-copy3image4upu: " + str(destination4origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin3upu['X'] = 100 
                destination4origin3upu['Y'] = 100  
                destination4origin3upu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin3upu-set-by-copy3image4upu to: " + str(destination4origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin3upu-failed-by-copy3image4upu: " + str(destination4origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imagecopy3upu ['DESTINATIONSOURCEORIGIN'] = destination4origin3upu                
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region3upu-trying-by-copy3image4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
                region3upu['Y'] = 330                                                                               # HEIGHT IN PIXELS TO WRITE
                region3upu['Z'] = 330                                                                               # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region3upu-set-by-copy3image4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region3upu-failed-by-copy3image4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '36Z-'
        
            task4imagecopy3upu ['REGION'] = region3upu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagecopy3upu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-copy3image4upu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy3upu-yielded-by-copy3image4upu: " + str(task4imagecopy3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3copyimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3upu = False
    channels3upu = ()
    message2print = "{(Step 40A/70) trying" + "channels3upu-trying-by-copy3image4upu: " + str(channels3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3upu, publisherchannel3upu = task4imagecopy3upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3upu-got-by-copy3image4upu: " + str(consumerchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3upu-got-by-copy3image4upu: " + str(publisherchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                              
        ack4channels3upu = True
        
        control3copyimage4upu['CONSUMERCHANNEL']  = consumerchannel3upu
        control3copyimage4upu['PUBLISHERCHANNEL'] = publisherchannel3upu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3upu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3upu = False
    queue3upu = None
    message2print = "{(Step 41A/70) trying" + "queue3upu-trying-by-copy3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3upu = task4imagecopy3upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3upu-got-by-copy3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue3upu = True
        control3copyimage4upu['QUEUE']  = queue3upu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3upu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4upu-trying-by-copy3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4upu = DummyProcess4Xi(target=manoeuvre3copyimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3upu, publisherchannel3upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4upu-created-by-copy3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-copy3image4upu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-copy3image4upu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control3copyimage4upu-returning-by-copy3image4upu: " + str(control3copyimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000430E710>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 684>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 680>}; time: 07:14:56 10/10/11 Pacific Daylight Time}


    return control3copyimage4upu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy3imagetobuffer4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4UPU ', time4xi())    
#   response4copy3imagetobuffer = {} 
#   task4copy3imagetobuffer = None
#   try:                               
#       response4copy3imagetobuffer = copy3imagetobuffer4upu(task4copy3imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4UPU : ', response4copy3imagetobuffer)
#       queue3copyimagetobuffer4upu      = response4copy3imagetobuffer['QUEUE']
#       channel3copyimagetobufferpublish4upu = response4copy3imagetobuffer['PUBLISHERCHANNEL']
#       channel3copyimagetobufferconsume4upu = response4copy3imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY3IMAGETOBUFFER4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4UPU ', time4xi())     
#   try:                               
#       message3copyimagetobuffersend4upu = "<copy4xi>copy3copyimagetobuffer4upu</copy4xi>"
#       channel3copyimagetobufferpublish4upu = response4copy3imagetobuffer['PUBLISHERCHANNEL']
#       channel3copyimagetobufferpublish4upu.send(message3copyimagetobuffersend4upu)
#       print("(Step 61B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3copyimagetobufferpublish4upu, " MESSAGE3SEND4UPU: " , message3copyimagetobuffersend4upu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copyimagetobuffer4upu(queue3copyimagetobuffer4upu, channel3copyimagetobufferconsume4upu, channel3copyimagetobufferpublish4upu)
#       print("(Step 62B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", manoeuvre3copyimagetobuffer4upu)
#   except:
#       print("(Step 62Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4UPU ', time4xi())     
#   try:
#       channel3copyimagetobufferconsume4upu = response4copy3imagetobuffer['CONSUMERCHANNEL']                              
#       message3copyimagereceive4upu = channel3copyimagetobufferconsume4upu.recv()
#       print("(Step 63B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copyimagetobufferconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGETOBUFFER4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4UPU ', time4xi())     
#   try:                               
#       message3copyimagetobuffersend4upu = "<copy4xi>copy3copyimagetobuffer4upu</copy4xi>"
#       channel3copyimagetobufferpublish4upu = response4copy3imagetobuffer['PUBLISHERCHANNEL']
#       channel3copyimagetobufferpublish4upu.send(message3copyimagetobuffersend4upu)
#       print("(Step 64B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3copyimagetobufferpublish4upu, " MESSAGE3SEND4UPU: " , message3copyimagetobuffersend4upu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copyimagetobuffer4upu(queue3copyimagetobuffer4upu, channel3copyimagetobufferconsume4upu, channel3copyimagetobufferpublish4upu)
#       print("(Step 65B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", manoeuvre3copyimagetobuffer4upu)
#   except:
#       print("(Step 65Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY3IMAGETOBUFFER4UPU ', time4xi())     
#   try:
#       channel3copyimagetobufferconsume4upu = response4copy3imagetobuffer['CONSUMERCHANNEL']                              
#       message3copyimagereceive4upu = channel3copyimagetobufferconsume4upu.recv()
#       print("(Step 66B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copyimagetobufferconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY3IMAGETOBUFFER4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3copyimagetobuffer4upu.put(path3upu)             
#       print("(Step 67B/700) OF UPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3upu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copyimagetobuffer4upu(queue3copyimagetobuffer4upu, channel3copyimagetobufferconsume4upu, channel3copyimagetobufferpublish4upu)
#       print("(Step 68B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", manoeuvre3copyimagetobuffer4upu)
#   except:
#       print("(Step 68Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3copyimagereceive4upu = channel3copyimagetobufferconsume4upu.recv()
#       print("(Step 69D/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copyimagetobufferconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copyimagetobuffer4upu(queue3copyimagetobuffer4upu, channel3copyimagetobufferconsume4upu, channel3copyimagetobufferpublish4upu)
#       print("(Step 70B/700) OF UPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4UPU ", response3manoeuvreimagecopytobuffer4upu)
#   except:
#       print("(Step 70Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def copy3imagetobuffer4upu(task4copyimagetobuffer3upu=None):

    '''
    copies 3D images to buffer for UPU
    '''
    global upu3image4xi                                                                                             # 3D-IMAGES FOR UPU  
    global upu3buffer4xi                                                                                            # BUFFER FOR UPU
    
    global control3copyimagetobuffer4upu                                                                            # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy3imagetobuffer4upu"
    method2print = "copy3imagetobuffer4upu"

    message2print = "{(Step 1A/33) started" + "; copy3imagetobuffer4upu-started: " + str(copy3imagetobuffer4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer3upu-to-copy3imagetobuffer4upu: " + str(task4copyimagetobuffer3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu3image4xi-created-by-copy3imagetobuffer4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4upu-skipped-by-copy3imagetobuffer4upu: " + str(copy3imagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                          
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 1E/70) set" + "; upu3image4xi-set-by-copy3imagetobuffer4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; upu3image4xi-yielded-by-copy3imagetobuffer4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3buffer4xi is None:
            upu3buffer4xi = create2buffer4upu()                                                                     # INITIALIZE BUFFER FOR UPU 
            message2print = "{(Step 1F/70) created" + "; upu3buffer4xi-created-by-copy3imagetobuffer4upu to: " + str(upu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4upu-skipped-by-copy3imagetobuffer4upu: " + str(copy3imagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            upu3buffer4xi = create2buffer4upu()
            message2print = "{(Step 1H/70) set" + "; upu3buffer4xi-set-by-copy3imagetobuffer4upu to: " + str(upu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        except:
            upu3buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; upu3buffer4xi-yielded-by-copy3imagetobuffer4upu: " + str(upu3buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer3upu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer3upu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer3upu-created-by-copy3imagetobuffer4upu to: " + str(task4copyimagetobuffer3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function3callback4upu = trace3image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function3callback4upu-created-by-copy3imagetobuffer4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code3ack  = code3ack + '1B1-'
           
        args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args3callback['CLASS']   = "copy3imagetobuffer4upu"                                                            # MODULE OF UPUIZED OPENCL MODULE
        args3callback['METHOD']  = "copy3imagetobuffer4upu"                                                            # FUNCTION OF UPUIZED OPENCL MODULE
        args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3imagetobuffer4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-copy3imagetobuffer4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        

        code3ack  = code3ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer3upu['DATA3CALLBACK'] = args3callback                                                    # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer3upu['FUNCTION3CALLBACK'] = function3callback4upu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code3ack  = code3ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer3upu-created-by-copy3imagetobuffer4upu to: " + str(task4copyimagetobuffer3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform3xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-copy3imagetobuffer4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-copy3imagetobuffer4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
              
            code3ack  = code3ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-copy3imagetobuffer4upu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code3nack  = code3nack + '2Z-'
                 
        task4copyimagetobuffer3upu['PLATFORM'] = platform3xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context3upu = {}
        message2print = "{(Step 3A/70) trying" + "; context3upu-trying-by-copy3imagetobuffer4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context3upu-got-by-copy3imagetobuffer4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
             
            code3ack  = code3ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context3upu-failed-by-copy3imagetobuffer4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z-'
            
        task4copyimagetobuffer3upu ['CONTEXT'] = context3upu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device3upu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device3upu-trying-by-copy3imagetobuffer4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device3upu-got-by-copy3imagetobuffer4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device3upu-failed-by-copy3imagetobuffer4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z-'
        
        task4copyimagetobuffer3upu ['DEVICE'] = device3upu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo3upu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(imagesourceinfo3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo3upu = get3imageinfo4upu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo3upu-got-by-copy3imagetobuffer4upu: " + str(imagesourceinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(imagesourceinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '5Z-'          
                  
        task4copyimagetobuffer3upu ['IMAGESOURCE'] = imagesourceinfo3upu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo3upu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(bufferdestinationinfo3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo3upu = get2bufferinfo4upu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo3upu-got-by-copy3imagetobuffer4upu: " + str(bufferdestinationinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(bufferdestinationinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '6Z-'          
                  
        task4copyimagetobuffer3upu ['BUFFERDESTINATION'] = bufferdestinationinfo3upu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY UPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue3copyimagetobuffer4upu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue3copyimagetobuffer4upu-trying-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue3copyimagetobuffer4upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue3copyimagetobuffer4upu-got-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue3copyimagetobuffer4upu-failed-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '7Z-'
            
        task4copyimagetobuffer3upu ['QUEUE'] = queue3copyimagetobuffer4upu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY UPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe3upu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe3upu-trying-by-copy3imagetobuffer4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe3upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe3upu-got-by-copy3imagetobuffer4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe3upu-failed-by-copy3imagetobuffer4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '8Z-'
            
        task4copyimagetobuffer3upu ['PIPE'] = pipe3upu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo3upu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo3upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo3upu-got-by-copy3imagetobuffer4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '9Z-'
    
        task4copyimagetobuffer3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo3upu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo3upu = get2eventinfo4upu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo3upu-got-by-copy3imagetobuffer4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '11Z-'
    
        task4copyimagetobuffer3upu ['EVENT'] = eventinfo3upu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist3upu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist3upu-trying-by-copy3imagetobuffer4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist3upu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist3upu-set-by-copy3imagetobuffer4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist3upu-failed-by-copy3imagetobuffer4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '12Z-'
    
        task4copyimagetobuffer3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity3upu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity3upu-trying-by-copy3imagetobuffer4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys3event = eventwaitlist3upu.keys()
            for key3event in keys3event:
                eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity3upu-set-by-copy3imagetobuffer4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-copy3imagetobuffer4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '13Z-'
    
        task4copyimagetobuffer3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source3upu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source3upu-trying-by-copy3imagetobuffer4upu: " + str(origin4source3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source3upu['X'] = 0 
            origin4source3upu['Y'] = 0  
            origin4source3upu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source3upu-set-by-copy3imagetobuffer4upu to: " + str(origin4source3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source3upu-failed-by-copy3imagetobuffer4upu: " + str(origin4source3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '14Z-'
    
        task4copyimagetobuffer3upu ['SOURCEORIGIN'] = origin4source3upu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy3upu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy3upu-trying-by-copy3imagetobuffer4upu: " + str(region4copy3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy3upu['X'] = 10 
            region4copy3upu['Y'] = 10  
            region4copy3upu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy3upu-set-by-copy3imagetobuffer4upu to: " + str(region4copy3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy3upu-failed-by-copy3imagetobuffer4upu: " + str(region4copy3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '16Z-'
        
        task4copyimagetobuffer3upu ['REGION'] = region4copy3upu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy3upu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy3upu-set-by-copy3imagetobuffer4upu to: " + str(offset4copy3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code3ack  = code3ack + '17A-'                
        task4copyimagetobuffer3upu ['OFFSET'] = offset4copy3upu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys3task = task4copyimagetobuffer3upu.keys()
    message2print = "{(Step 20/70) got" + "; keys3task-got-by-copy3imagetobuffer4upu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # 


    if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function3callback4upu = trace3image4upu                                                             # CALLBACK FUNCTION FOR UPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function3callback4upu-created-by-copy3imagetobuffer4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code3ack  = code3ack + '21B1-'
           
        args3callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args3callback['CLASS']   = "copy3imagetobuffer4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
        args3callback['METHOD']  = "copy3imagetobuffer4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
        args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3imagetobuffer4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-copy3imagetobuffer4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code3ack  = code3ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer3upu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer3upu['FUNCTION3CALLBACK'] = function3callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code3ack  = code3ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer3upu-created-by-copy3imagetobuffer4upu to: " + str(task4copyimagetobuffer3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys3task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform3xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-copy3imagetobuffer4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-copy3imagetobuffer4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code3ack  = code3ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-copy3imagetobuffer4upu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code3nack  = code3nack + '22Z-'
                 
        task4copyimagetobuffer3upu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys3task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context3upu = {}
        message2print = "{(Step 23A/70) trying" + "; context3upu-trying-by-copy3imagetobuffer4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context3upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context3upu-got-by-copy3imagetobuffer4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context3upu-failed-by-copy3imagetobuffer4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '23Z-'
            
        task4copyimagetobuffer3upu ['CONTEXT'] = context3upu           
    
    if 'DEVICE' in keys3task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device3upu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device3upu-trying-by-copy3imagetobuffer4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device3upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device3upu-got-by-copy3imagetobuffer4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code3ack  = code3ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device3upu-failed-by-copy3imagetobuffer4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '24Z-'
        
        task4copyimagetobuffer3upu ['DEVICE'] = device3upu
    
    if 'IMAGESOURCE' in keys3task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo3upu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(sourceimageinfo3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo3upu = get3imageinfo4upu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo3upu-got-by-copy3imagetobuffer4upu: " + str(sourceimageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code3ack  = code3ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(sourceimageinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '25Z-'          
                  
        task4copyimagetobuffer3upu ['IMAGESOURCE'] = sourceimageinfo3upu         
    
    if 'BUFFERDESTINATION' in keys3task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo3upu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(destinationbufferinfo3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo3upu = get2bufferinfo4upu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo3upu-got-by-copy3imagetobuffer4upu: " + str(destinationbufferinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code3ack  = code3ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(destinationbufferinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '26Z-'          
                  
        task4copyimagetobuffer3upu ['BUFFERDESTINATION'] = destinationbufferinfo3upu    
                                
    if 'QUEUE' in keys3task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY UPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue3copyimagetobuffer4upu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue3copyimagetobuffer4upu-trying-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue3copyimagetobuffer4upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue3copyimagetobuffer4upu-got-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue3copyimagetobuffer4upu-failed-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '27Z-'
            
        task4copyimagetobuffer3upu ['QUEUE'] = queue3copyimagetobuffer4upu
        
    if 'PIPE' in keys3task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY UPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe3upu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe3upu-trying-by-copy3imagetobuffer4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe3upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe3upu-got-by-copy3imagetobuffer4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe3upu-failed-by-copy3imagetobuffer4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '28Z-'
            
        task4copyimagetobuffer3upu ['PIPE'] = pipe3upu 
        
    if 'COMMANDQUEUE' in keys3task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo3upu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo3upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo3upu-got-by-copy3imagetobuffer4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '29Z-'
    
        task4copyimagetobuffer3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
    
         
    if 'EVENT' in keys3task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo3upu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo3upu-trying-by-copy3imagetobuffer4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo3upu = get2eventinfo4upu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo3upu-got-by-copy3imagetobuffer4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo3upu-failed-by-copy3imagetobuffer4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '31Z-'
    
        task4copyimagetobuffer3upu ['EVENT'] = eventinfo3upu 
    
         
    if 'EVENTWAITLIST' in keys3task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist3upu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist3upu-trying-by-copy3imagetobuffer4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist3upu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist3upu-set-by-copy3imagetobuffer4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist3upu-failed-by-copy3imagetobuffer4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '32Z-'
    
        task4copyimagetobuffer3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys3task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity3upu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity3upu-trying-by-copy3imagetobuffer4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys3event = eventwaitlist3upu.keys()
            for key3event in keys3event:
                eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity3upu-set-by-copy3imagetobuffer4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-copy3imagetobuffer4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '33Z-'
    
        task4copyimagetobuffer3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                         
                
    if 'SOURCEORIGIN' in keys3task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin3upu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin3upu-trying-by-copy3imagetobuffer4upu: " + str(source4origin3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin3upu['X'] = 0 
            source4origin3upu['Y'] = 0  
            source4origin3upu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin3upu-set-by-copy3imagetobuffer4upu to: " + str(source4origin3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code3ack  = code3ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin3upu-failed-by-copy3imagetobuffer4upu: " + str(source4origin3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '34Z-'
    
        task4copyimagetobuffer3upu ['SOURCEORIGIN'] = source4origin3upu
             
   
             
    if 'REGION' in keys3task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region3upu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region3upu-trying-by-copy3imagetobuffer4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
            region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO WRITE
            region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region3upu-set-by-copy3imagetobuffer4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code3ack  = code3ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region3upu-failed-by-copy3imagetobuffer4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '36Z-'
    
        task4copyimagetobuffer3upu ['REGION'] = region3upu
    
         
    if 'OFFSET' in keys3task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2upu = 0
        message2print = "{(Step 37A/70) set" + "; offset2upu-set-by-copy3imagetobuffer4upu to: " + str(offset2upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer3upu ['OFFSET'] = offset2upu        
        code3ack  = code3ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys3task = task4copyimagetobuffer3upu.keys()
    message2print = "{(Step 39/70) set" + "; keys3task-set-by-copy3imagetobuffer4upu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # 
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3copyimagetobuffer4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3upu = False
    channels3upu = ()
    message2print = "{(Step 40A/70) trying" + "channels3upu-trying-by-copy3imagetobuffer4upu: " + str(channels3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3copyimagetobuffer4upu, publisherchannel3copyimagetobuffer4upu = task4copyimagetobuffer3upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3copyimagetobuffer4upu-got-by-copy3imagetobuffer4upu: " + str(consumerchannel3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3copyimagetobuffer4upu-got-by-copy3imagetobuffer4upu: " + str(publisherchannel3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                   
        ack4channels3upu = True
        
        control3copyimagetobuffer4upu['CONSUMERCHANNEL']  = consumerchannel3copyimagetobuffer4upu
        
        
        control3copyimagetobuffer4upu['PUBLISHERCHANNEL'] = publisherchannel3copyimagetobuffer4upu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3upu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3copyimagetobuffer4upu = False
    queue3copyimagetobuffer4upu = None
    message2print = "{(Step 41A/70) trying" + "queue3copyimagetobuffer4upu-trying-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3copyimagetobuffer4upu = task4copyimagetobuffer3upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3copyimagetobuffer4upu-got-by-copy3imagetobuffer4upu: " + str(queue3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
              
        ack4queue3copyimagetobuffer4upu = True
        control3copyimagetobuffer4upu['QUEUE']  = queue3copyimagetobuffer4upu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3copyimagetobuffer4upu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4upu-trying-by-copy3imagetobuffer4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4upu = DummyProcess4Xi(target=manoeuvre3copyimagetobuffer4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3copyimagetobuffer4upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3copyimagetobuffer4upu, publisherchannel3copyimagetobuffer4upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4upu-created-by-copy3imagetobuffer4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-copy3imagetobuffer4upu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-copy3imagetobuffer4upu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control3copyimagetobuffer4upu-returning-by-copy3imagetobuffer4upu: " + str(control3copyimagetobuffer4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
        
    return control3copyimagetobuffer4upu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy3buffertoimage4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU ', time4xi())    
#   response4copy3buffertoimage = {} 
#   task4copy3buffertoimage = None
#   try:                               
#       response4copy3buffertoimage = copy3buffertoimage4upu(task4copy3buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU : ', response4copy3buffertoimage)
#       queue3copybuffertoimage4upu      = response4copy3buffertoimage['QUEUE']
#       channel3copybuffertoimagepublish4upu = response4copy3buffertoimage['PUBLISHERCHANNEL']
#       channel3copybuffertoimageconsume4upu = response4copy3buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:                               
#       message3copybuffertoimagesend4upu = "<copy4xi>copy3copybuffertoimage4upu</copy4xi>"
#       channel3copybuffertoimagepublish4upu = response4copy3buffertoimage['PUBLISHERCHANNEL']
#       channel3copybuffertoimagepublish4upu.send(message3copybuffertoimagesend4upu)
#       print("(Step 81B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3copybuffertoimagepublish4upu, " MESSAGE3SEND4UPU: " , message3copybuffertoimagesend4upu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copybuffertoimage4upu(queue3copybuffertoimage4upu, channel3copybuffertoimageconsume4upu, channel3copybuffertoimagepublish4upu)
#       print("(Step 83D/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", manoeuvre3copybuffertoimage4upu)
#   except:
#       print("(Step 82Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:
#       channel3copybuffertoimageconsume4upu = response4copy3buffertoimage['CONSUMERCHANNEL']                              
#       message3copyimagereceive4upu = channel3copybuffertoimageconsume4upu.recv()
#       print("(Step 83B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copybuffertoimageconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:                               
#       message3copybuffertoimagesend4upu = "<copy4xi>copy3copybuffertoimage4upu</copy4xi>"
#       channel3copybuffertoimagepublish4upu = response4copy3buffertoimage['PUBLISHERCHANNEL']
#       channel3copybuffertoimagepublish4upu.send(message3copybuffertoimagesend4upu)
#       print("(Step 84B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3copybuffertoimagepublish4upu, " MESSAGE3SEND4UPU: " , message3copybuffertoimagesend4upu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copybuffertoimage4upu(queue3copybuffertoimage4upu, channel3copybuffertoimageconsume4upu, channel3copybuffertoimagepublish4upu)
#       print("(Step 85B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", manoeuvre3copybuffertoimage4upu)
#   except:
#       print("(Step 85Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4UPU ', time4xi())     
#   try:
#       channel3copybuffertoimageconsume4upu = response4copy3buffertoimage['CONSUMERCHANNEL']                              
#       message3copyimagereceive4upu = channel3copybuffertoimageconsume4upu.recv()
#       print("(Step 86B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copybuffertoimageconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3copybuffertoimage4upu.put(path3upu)             
#       print("(Step 87B/700) OF UPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3upu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copybuffertoimage4upu(queue3copybuffertoimage4upu, channel3copybuffertoimageconsume4upu, channel3copybuffertoimagepublish4upu)
#       print("(Step 88B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", manoeuvre3copybuffertoimage4upu)
#   except:
#       print("(Step 88Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3copyimagereceive4upu = channel3copybuffertoimageconsume4upu.recv()
#       print("(Step 89D/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3copybuffertoimageconsume4upu, " CONTROL & COMMAND: ", message3copyimagereceive4upu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE3COPYIMAGETOBUFFER4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi()) 
#   try:        
#       response3manoeuvreimagecopytobuffer4upu = manoeuvre3copybuffertoimage4upu(queue3copybuffertoimage4upu, channel3copybuffertoimageconsume4upu, channel3copybuffertoimagepublish4upu)
#       print("(Step 90B/700) OF UPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4UPU ", response3manoeuvreimagecopytobuffer4upu)
#   except:
#       print("(Step 90Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGETOBUFFER4UPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def copy3buffertoimage4upu(task4copybuffertoimage3upu=None):

    '''
    copies buffer to image for UPU
    '''
    global upu3image4xi                                                                                             # 3D-IMAGES FOR UPU  
    global upu3buffer4xi                                                                                            # BUFFER FOR UPU
    
    global control3copybuffertoimage4upu                                                                            # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy3buffertoimage4upu"
    method2print = "copy3buffertoimage4upu"

    message2print = "{(Step 1A/33) started" + "; copy3buffertoimage4upu-started: " + str(copy3buffertoimage4upu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage3upu-to-copy3buffertoimage4upu: " + str(task4copybuffertoimage3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu3image4xi-created-by-copy3buffertoimage4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4upu-skipped-by-copy3buffertoimage4upu: " + str(copy3buffertoimage4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                 
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 1E/70) set" + "; upu3image4xi-set-by-copy3buffertoimage4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; upu3image4xi-yielded-by-copy3buffertoimage4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3buffer4xi is None:
            upu3buffer4xi = create2buffer4upu()                                                                     # INITIALIZE BUFFER FOR UPU 
            message2print = "{(Step 1F/70) created" + "; upu3buffer4xi-created-by-copy3buffertoimage4upu to: " + str(upu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4upu-skipped-by-copy3buffertoimage4upu: " + str(copy3buffertoimage4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            upu3buffer4xi = create2buffer4upu()
            message2print = "{(Step 1H/70) set" + "; upu3buffer4xi-set-by-copy3buffertoimage4upu to: " + str(upu3buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; upu3buffer4xi-yielded-by-copy3buffertoimage4upu: " + str(upu3buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage3upu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage3upu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage3upu-created-by-copy3buffertoimage4upu to: " + str(task4copybuffertoimage3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function3callback4upu = trace3image4upu                                                                 # CALLBACK FUNCTION FOR UPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function3callback4upu-created-by-copy3buffertoimage4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
        code3ack  = code3ack + '1B1-'
       
        args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args3callback['CLASS']   = "copy3buffertoimage4upu"                                                     # MODULE OF UPUIZED OPENCL MODULE
        args3callback['METHOD']  = "copy3buffertoimage4upu"                                                     # FUNCTION OF UPUIZED OPENCL MODULE
        args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3buffertoimage4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-copy3buffertoimage4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        code3ack  = code3ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage3upu['DATA3CALLBACK'] = args3callback                                             # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage3upu['FUNCTION3CALLBACK'] = function3callback4upu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code3ack  = code3ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage3upu-created-by-copy3buffertoimage4upu to: " + str(task4copybuffertoimage3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform3xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-copy3buffertoimage4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-copy3buffertoimage4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code3ack  = code3ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-copy3buffertoimage4upu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code3nack  = code3nack + '2Z-'
                 
        task4copybuffertoimage3upu['PLATFORM'] = platform3xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context3upu = {}
        message2print = "{(Step 3A/70) trying" + "; context3upu-trying-by-copy3buffertoimage4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context3upu-got-by-copy3buffertoimage4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
             
            code3ack  = code3ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context3upu-failed-by-copy3buffertoimage4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z-'
                
        task4copybuffertoimage3upu ['CONTEXT'] = context3upu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device3upu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device3upu-trying-by-copy3buffertoimage4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device3upu-got-by-copy3buffertoimage4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
             
            code3ack  = code3ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device3upu-failed-by-copy3buffertoimage4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z-'
            
        task4copybuffertoimage3upu ['DEVICE'] = device3upu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo3upu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo3upu-trying-by-copy3buffertoimage4upu: " + str(imagedestinationinfo3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo3upu = get3imageinfo4upu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo3upu-got-by-copy3buffertoimage4upu: " + str(imagedestinationinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo3upu-failed-by-copy3buffertoimage4upu: " + str(imagedestinationinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '5Z-'          
                  
        task4copybuffertoimage3upu ['IMAGEDESTINATION'] = imagedestinationinfo3upu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo3upu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo3upu-trying-by-copy3buffertoimage4upu: " + str(buffersourceinfo3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo3upu = get2bufferinfo4upu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo3upu-got-by-copy3buffertoimage4upu: " + str(buffersourceinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo3upu-failed-by-copy3buffertoimage4upu: " + str(buffersourceinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '6Z-'          
                  
        task4copybuffertoimage3upu ['BUFFERSOURCE'] = buffersourceinfo3upu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY UPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue3copybuffertoimage4upu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue3copybuffertoimage4upu-trying-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue3copybuffertoimage4upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue3copybuffertoimage4upu-got-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue3copybuffertoimage4upu-failed-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '7Z-'
            
        task4copybuffertoimage3upu ['QUEUE'] = queue3copybuffertoimage4upu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY UPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe3upu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe3upu-trying-by-copy3buffertoimage4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe3upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe3upu-got-by-copy3buffertoimage4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code3ack  = code3ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe3upu-failed-by-copy3buffertoimage4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '8Z-'
            
        task4copybuffertoimage3upu ['PIPE'] = pipe3upu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo3upu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo3upu-trying-by-copy3buffertoimage4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo3upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo3upu-got-by-copy3buffertoimage4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo3upu-failed-by-copy3buffertoimage4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '9Z-'
    
        task4copybuffertoimage3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo3upu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo3upu-trying-by-copy3buffertoimage4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo3upu = get2eventinfo4upu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo3upu-got-by-copy3buffertoimage4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo3upu-failed-by-copy3buffertoimage4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '11Z-'
    
        task4copybuffertoimage3upu ['EVENT'] = eventinfo3upu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist3upu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist3upu-trying-by-copy3buffertoimage4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist3upu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist3upu-set-by-copy3buffertoimage4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist3upu-failed-by-copy3buffertoimage4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '12Z-'
    
        task4copybuffertoimage3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity3upu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity3upu-trying-by-copy3buffertoimage4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys3event = eventwaitlist3upu.keys()
            for key3event in keys3event:
                eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity3upu-set-by-copy3buffertoimage4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-copy3buffertoimage4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '13Z-'
    
        task4copybuffertoimage3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination3upu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination3upu-trying-by-copy3buffertoimage4upu: " + str(origin4destination3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination3upu['X'] = 100 
            origin4destination3upu['Y'] = 100  
            origin4destination3upu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination3upu-set-by-copy3buffertoimage4upu to: " + str(origin4destination3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination3upu-failed-by-copy3buffertoimage4upu: " + str(origin4destination3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '15Z-'
    
        task4copybuffertoimage3upu ['DESTINATIONORIGIN'] = origin4destination3upu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy3upu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy3upu-trying-by-copy3buffertoimage4upu: " + str(region4copy3upu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy3upu['X'] = 10 
            region4copy3upu['Y'] = 10  
            region4copy3upu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy3upu-set-by-copy3buffertoimage4upu to: " + str(region4copy3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code3ack  = code3ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy3upu-failed-by-copy3buffertoimage4upu: " + str(region4copy3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '16Z-'
        
        task4copybuffertoimage3upu ['REGION'] = region4copy3upu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy3upu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy3upu-set-by-copy3buffertoimage4upu to: " + str(offset4copy3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
                
        task4copybuffertoimage3upu ['OFFSET'] = offset4copy3upu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4copybuffertoimage3upu.keys()
        message2print = "{(Step 20/70) got" + "; keys3task-got-by-copy3buffertoimage4upu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
 
        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                             # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4upu-created-by-copy3buffertoimage4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "copy3buffertoimage4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "copy3buffertoimage4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy3buffertoimage4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-copy3buffertoimage4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage3upu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage3upu['FUNCTION3CALLBACK'] = function3callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage3upu-created-by-copy3buffertoimage4upu to: " + str(task4copybuffertoimage3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-copy3buffertoimage4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-copy3buffertoimage4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-copy3buffertoimage4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4copybuffertoimage3upu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 23A/70) trying" + "; context3upu-trying-by-copy3buffertoimage4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3upu-got-by-copy3buffertoimage4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3upu-failed-by-copy3buffertoimage4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4copybuffertoimage3upu ['CONTEXT'] = context3upu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3upu-trying-by-copy3buffertoimage4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3upu-got-by-copy3buffertoimage4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3upu-failed-by-copy3buffertoimage4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4copybuffertoimage3upu ['DEVICE'] = device3upu
        
        if 'IMAGEDESTINATION' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo3upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo3upu-trying-by-copy3buffertoimage4upu: " + str(destinationimageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo3upu-got-by-copy3buffertoimage4upu: " + str(destinationimageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo3upu-failed-by-copy3buffertoimage4upu: " + str(destinationimageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4copybuffertoimage3upu ['IMAGEDESTINATION'] = destinationimageinfo3upu         
        
        if 'BUFFERSOURCE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo3upu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo3upu-trying-by-copy3buffertoimage4upu: " + str(sourcebufferinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo3upu = get2bufferinfo4upu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo3upu-got-by-copy3buffertoimage4upu: " + str(sourcebufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo3upu-failed-by-copy3buffertoimage4upu: " + str(sourcebufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'          
                      
            task4copybuffertoimage3upu ['BUFFERSOURCE'] = sourcebufferinfo3upu    
                                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3copybuffertoimage4upu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue3copybuffertoimage4upu-trying-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3copybuffertoimage4upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue3copybuffertoimage4upu-got-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue3copybuffertoimage4upu-failed-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4copybuffertoimage3upu ['QUEUE'] = queue3copybuffertoimage4upu
            
        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe3upu-trying-by-copy3buffertoimage4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe3upu-got-by-copy3buffertoimage4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe3upu-failed-by-copy3buffertoimage4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
                
            task4copybuffertoimage3upu ['PIPE'] = pipe3upu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo3upu-trying-by-copy3buffertoimage4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo3upu-got-by-copy3buffertoimage4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo3upu-failed-by-copy3buffertoimage4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4copybuffertoimage3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
        
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo3upu-trying-by-copy3buffertoimage4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo3upu-got-by-copy3buffertoimage4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo3upu-failed-by-copy3buffertoimage4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4copybuffertoimage3upu ['EVENT'] = eventinfo3upu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist3upu-trying-by-copy3buffertoimage4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist3upu-set-by-copy3buffertoimage4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist3upu-failed-by-copy3buffertoimage4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4copybuffertoimage3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity3upu-trying-by-copy3buffertoimage4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity3upu-set-by-copy3buffertoimage4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-copy3buffertoimage4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4copybuffertoimage3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin3upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin3upu-trying-by-copy3buffertoimage4upu: " + str(destination4origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin3upu['X'] = 0 
                destination4origin3upu['Y'] = 0  
                destination4origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin3upu-set-by-copy3buffertoimage4upu to: " + str(destination4origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin3upu-failed-by-copy3buffertoimage4upu: " + str(destination4origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4copybuffertoimage3upu ['DESTINATIONORIGIN'] = destination4origin3upu
             
     
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region3upu-trying-by-copy3buffertoimage4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
                region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO WRITE
                region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region3upu-set-by-copy3buffertoimage4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region3upu-failed-by-copy3buffertoimage4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '36Z-'
        
            task4copybuffertoimage3upu ['REGION'] = region3upu
        
             
        if 'OFFSET' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2upu = 0
            message2print = "{(Step 37A/70) set" + "; offset2upu-set-by-copy3buffertoimage4upu to: " + str(offset2upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage3upu ['OFFSET'] = offset2upu        
            code3ack  = code3ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys3task = task4copybuffertoimage3upu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-copy3buffertoimage4upu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3copybuffertoimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3upu = False
    channels3upu = ()
    message2print = "{(Step 40A/70) trying" + "channels3upu-trying-by-copy3buffertoimage4upu: " + str(channels3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3copybuffertoimage4upu, publisherchannel3copybuffertoimage4upu = task4copybuffertoimage3upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3copybuffertoimage4upu-got-by-copy3buffertoimage4upu: " + str(consumerchannel3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3copybuffertoimage4upu-got-by-copy3buffertoimage4upu: " + str(publisherchannel3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                  
        ack4channels3upu = True
        
        control3copybuffertoimage4upu['CONSUMERCHANNEL']  = consumerchannel3copybuffertoimage4upu
        
        
        control3copybuffertoimage4upu['PUBLISHERCHANNEL'] = publisherchannel3copybuffertoimage4upu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3upu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3copybuffertoimage4upu = False
    queue3copybuffertoimage4upu = None
    message2print = "{(Step 41A/70) trying" + "queue3copybuffertoimage4upu-trying-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3copybuffertoimage4upu = task4copybuffertoimage3upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3copybuffertoimage4upu-got-by-copy3buffertoimage4upu: " + str(queue3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue3copybuffertoimage4upu = True
        control3copybuffertoimage4upu['QUEUE']  = queue3copybuffertoimage4upu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3copybuffertoimage4upu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4upu-trying-by-copy3buffertoimage4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4upu = DummyProcess4Xi(target=manoeuvre3copybuffertoimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3copybuffertoimage4upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3copybuffertoimage4upu, publisherchannel3copybuffertoimage4upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4upu-created-by-copy3buffertoimage4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-copy3buffertoimage4upu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-copy3buffertoimage4upu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control3copybuffertoimage4upu-returning-by-copy3buffertoimage4upu: " + str(control3copybuffertoimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
    

        
    return control3copybuffertoimage4upu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read3image4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4UPU ', time4xi())    
#   respon4read3image = {} 
#   task4read3image = None
#   try:                               
#       response4read3image = read3image4upu(task4read3image)                                                                                                          
#       print('(Step 16B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ3IMAGE4UPU : ', response4read3image)
#       queue3read4upu      = response4read3image['QUEUE']
#       channel3readpublish4upu = response4read3image['PUBLISHERCHANNEL']
#       channel3readconsume4upu = response4read3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3IMAGE4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:                               
#       message3readsend4upu = "<copy4xi>copy3read4upu</copy4xi>"
#       channel3readpublish4upu = response4read3image['PUBLISHERCHANNEL']
#       channel3readpublish4upu.send(message3readsend4upu)
#       print("(Step 17B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3readpublish4upu, " MESSAGE3SEND4UPU: " , message3readsend4upu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE3IMAGE4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3read4upu, channel3readconsume4upu, channel3readpublish4upu)
#       print("(Step 18D/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3readimage4upu)
#   except:
#       print("(Step 18Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:
#       channel3readconsume4upu = response4read3image['CONSUMERCHANNEL']                              
#       message3readreceive4upu = channel3readconsume4upu.recv()
#       print("(Step 19B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3readconsume4upu, " CONTROL & COMMAND: ", message3readreceive4upu)                                                                                                  
#       # <roger4xi>roger3read4upu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3IMAGE4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:                               
#       message3readsend4upu = "<ready4xi>ready3read4upu</ready4xi>"
#       channel3readpublish4upu = response4read3image['PUBLISHERCHANNEL']
#       channel3readpublish4upu.send(message3readsend4upu)
#       print("(Step 20B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3readpublish4upu, " MESSAGE3SEND4UPU: " , message3readsend4upu)                                                                                                
#       # <ready4xi>ready3read4upu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE3IMAGE4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3read4upu, channel3readconsume4upu, channel3readpublish4upu)
#       print("(Step 21B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3readimage4upu)
#   except:
#       print("(Step 21Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:
#       channel3readconsume4upu = response4read3image['CONSUMERCHANNEL']                              
#       message3readreceive4upu = channel3readconsume4upu.recv()
#       print("(Step 22B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3readconsume4upu, " CONTROL & COMMAND: ", message3readreceive4upu)                                                                                                  
#       # <ready4xi>ready3read4upu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3read4upu.put(path3upu)             
#       print("(Step 24B/700) OF UPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3upu) 
#       #  
#   except:
#       print("(Step 24Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE3IMAGE4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3read4upu, channel3readconsume4upu, channel3readpublish4upu)
#       print("(Step 25B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3readimage4upu)
#   except:
#       print("(Step 25Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3readreceive4upu = channel3readconsume4upu.recv()
#       print("(Step 26D/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3readconsume4upu, " CONTROL & COMMAND: ", message3readreceive4upu)                                                                                                   
#       # <ack4xi>ack3imageinfo4upu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE3IMAGE4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3read4upu, channel3readconsume4upu, channel3readpublish4upu)
#       print("(Step 27B/700) OF UPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4UPU ", response3manoeuvre4upu)
#   except:
#       print("(Step 27Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())   
#">
# -------------------------------------------------------------------------------------------------   
def read3image4upu(task4imageread3upu=None):

    '''
    reads 3D images for UPU
    '''
    global upu3image4xi                                                     # 3D-IMAGES FOR UPU  
    global control3readimage4upu                                              # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read3image4upu"
    method2print = "read3image4upu"

    message2print = "{(Step 1A/33) started" + "; read3image4upu-started: " + str(read3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read3image4upu at 0x00000000031B78C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread3upu-to-read3image4upu: " + str(task4imageread3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu3image4xi-created-by-read3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4upu-skipped-by-read3image4upu: " + str(read3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function read3image4upu at 0x00000000031B78C8>                               
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 1E/70) set" + "; upu3image4xi-set-by-read3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu3image4xi-yielded-by-read3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread3upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread3upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread3upu-created-by-read3image4upu to: " + str(task4imageread3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4upu-created-by-read3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace3image4upu at 0x00000000031B1848> 
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "read3image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "read3image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-read3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read3image4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
            #   'CLASS': 'read3image4upu', 
            #   'METHOD': 'read3image4upu'}; time: 06:24:23 10/10/11 Pacific Daylight Time}

            
            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread3upu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread3upu['FUNCTION3CALLBACK'] = function3callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread3upu-created-by-read3image4upu to: " + str(task4imageread3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read3image4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
            #   'CLASS': 'read3image4upu', 
            #   'METHOD': 'read3image4upu'}, 
            #   'FUNCTION3CALLBACK': <function trace3image4upu at 0x00000000031B1848>}; time: 06:24:23 10/10/11 Pacific Daylight Time}
            
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-read3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-read3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 06:24:23 10/10/11 Pacific Daylight Time}
                 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-read3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imageread3upu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 3A/70) trying" + "; context3upu-trying-by-read3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3upu-got-by-read3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 
                #   'CLASS': 'create2context4upu', 
                #   'METHOD': 'create2context4upu'}, 
                #   'FUNCTION2CALLBACK': <function trace2context4upu at 0x000000000313E0C8>, 
                #   'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #   'INFO2PRIVATE': None, 
                #   'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 06:24:23 10/10/11 Pacific Daylight Time}

                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3upu-failed-by-read3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imageread3upu ['CONTEXT'] = context3upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3upu-trying-by-read3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3upu-got-by-read3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3upu-failed-by-read3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imageread3upu ['DEVICE'] = device3upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo3upu-trying-by-read3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo3upu-got-by-read3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #   'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #   'ORDER': 'CL_RGBA'}, 
                #   'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 
                #   'WIDTH': 1100, 
                #   'DEPTH': 900, 
                #   'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 06:24:23 10/10/11 Pacific Day ...
                
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo3upu-failed-by-read3image4upu: " + str(imageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imageread3upu ['IMAGE'] = imageinfo3upu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3upu-trying-by-read3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3upu-got-by-read3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004422FD0> 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3upu-failed-by-read3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imageread3upu ['QUEUE'] = queue3upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3upu-trying-by-read3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3upu-got-by-read3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 588>, <read-write PipeConnection, handle 592>) 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3upu-failed-by-read3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imageread3upu ['PIPE'] = pipe3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3upu-trying-by-read3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3upu-got-by-read3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 06:24:23 10/10/11 Pa ...
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3upu-failed-by-read3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imageread3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3upu-trying-by-read3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3upu-got-by-read3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 06:24:23 10/10/11 Pacific  ...
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3upu-failed-by-read3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imageread3upu ['BUFFER'] = bufferinfo3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3upu-trying-by-read3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo3upu-got-by-read3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS ...
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3upu-failed-by-read3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imageread3upu ['EVENT'] = eventinfo3upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist3upu-trying-by-read3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist3upu-set-by-read3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist3upu-failed-by-read3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imageread3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity3upu-trying-by-read3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity3upu-set-by-read3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-read3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imageread3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3upu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking3upu-trying-by-read3image4upu: " + str(blocking3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3upu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking3upu-set-by-read3image4upu to: " + str(blocking3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking3upu-failed-by-read3image4upu: " + str(blocking3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imageread3upu ['BLOCKING'] = blocking3upu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin3upu-trying-by-read3image4upu: " + str(origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3upu['X'] = 0 
                origin3upu['Y'] = 0  
                origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin3upu-set-by-read3image4upu to: " + str(origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin3upu-failed-by-read3image4upu: " + str(origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imageread3upu ['ORIGIN'] = origin3upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region3upu-trying-by-read3image4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO READ 
                region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO READ 
                region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region3upu-set-by-read3image4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 330, 'X': 330, 'Z': 330} 
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region3upu-failed-by-read3image4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imageread3upu ['REGION'] = region3upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch3upu = 0
            size3element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 8                                                                                  # ONE DIRECTIONS IN 3D
            width3upu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3upu-trying-by-read3image4upu: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3upu-set-by-read3image4upu to: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 8000                                                                                        
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3upu-failed-by-read3image4upu: " + str(rowpitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imageread3upu ['ROWPITCH'] = rowpitch3upu     
            
            # JATKA --------------------------------------------------------------------------------------------------------
            # (17) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3upu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
            height3upu = 800
            message2print = "{(Step 17A/70) trying" + "; slicepitch3upu-trying-by-read3image4upu: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3upu = int(rowpitch3upu * height3upu)                                                           
                message2print = "{(Step 17B/70) set" + "; slicepitch3upu-set-by-read3image4upu to: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 6400000

                code3ack  = code3ack + '17B-'                
                 
            except:
                message2print = "{(Step 17Z/70) failed" + "; slicepitch3upu-failed-by-read3image4upu: " + str(slicepitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '17Z-'
            
            task4imageread3upu ['SLICEPITCH'] = slicepitch3upu           
                        
            # ----------------------------------------------------------------------------------------------------
            # (18) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3upu = None
            height3upu = 800
            size3upu = int(width3upu * height3upu)                                                                  # CALCULATED MEMORY
            if size3upu == 0:
                size3upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 18A/70) trying" + "; pointer3upu-trying-by-read3image4upu: " + str(pointer3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3upu = Arena4Xi(size3upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 18B/70) got" + "; arena3upu-got-by-read3image4upu: " + str(arena3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000048859B0>
                
                size3arena = arena3upu.size                                                                         # 
                              
                message2print = "{(Step 18C/70) got" + "; size3arena-got-by-read2buffer4upu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer3arena = arena3upu.buffer
                
                message2print = "{(Step 18D/70) got" + "; buffer3arena-got-by-read2buffer4upu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000435B4A8>
                
                code3ack  = code3ack + '18D-' 
                
                if size3arena < size3upu:
                    size3upu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 18E/70) set" + "; size3upu-set-by-read2buffer4upu to: " + str(size3upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '18E-'
                else:
                    pass
                    
                pointer3upu = buffer3arena
                message2print = "{(Step 18F/70) set" + "; pointer3upu-set-by-read2buffer4upu to: " + str(pointer3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000435B4A8>
                
                code3ack  = code3ack + '18F-'       
            except:
                message2print = "{(Step 18Z/70) failed" + "; pointer3upu-failed-by-read3image4upu: " + str(pointer3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '18Z-'
        
            task4imageread3upu ['POINTER'] = pointer3upu                            
            task4imageread3upu ['SIZE']    = size3upu                                                               # AUTOMATIC BY UPUXI
            
       
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imageread3upu.keys()
        message2print = "{(Step 20/70) got" + "; keys3task-got-by-read3image4upu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:04:35 10/10/11 Pacific Daylight Time}


        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4upu-created-by-read3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "read3image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "read3image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-read3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread3upu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread3upu['FUNCTION3CALLBACK'] = function3callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread3upu-created-by-read3image4upu to: " + str(task4imageread3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-read3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-read3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-read3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4imageread3upu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 23A/70) trying" + "; context3upu-trying-by-read3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3upu-got-by-read3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3upu-failed-by-read3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imageread3upu ['CONTEXT'] = context3upu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3upu-trying-by-read3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3upu-got-by-read3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3upu-failed-by-read3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imageread3upu ['DEVICE'] = device3upu
        
        if 'IMAGE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo3upu-trying-by-read3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo3upu-got-by-read3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo3upu-failed-by-read3image4upu: " + str(imageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imageread3upu ['IMAGE'] = imageinfo3upu         
                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue3upu-trying-by-read3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue3upu-got-by-read3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue3upu-failed-by-read3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'
                
            task4imageread3upu ['QUEUE'] = queue3upu
        
        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe3upu-trying-by-read3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe3upu-got-by-read3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe3upu-failed-by-read3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imageread3upu ['PIPE'] = pipe3upu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo3upu-trying-by-read3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo3upu-got-by-read3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo3upu-failed-by-read3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
        
            task4imageread3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
        
        
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo3upu-trying-by-read3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo3upu-got-by-read3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo3upu-failed-by-read3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imageread3upu ['BUFFER'] = bufferinfo3upu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo3upu-trying-by-read3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo3upu-got-by-read3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo3upu-failed-by-read3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imageread3upu ['EVENT'] = eventinfo3upu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist3upu-trying-by-read3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist3upu-set-by-read3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist3upu-failed-by-read3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imageread3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity3upu-trying-by-read3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity3upu-set-by-read3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-read3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imageread3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                                 
                 
        if 'BLOCKING' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3upu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking3upu-trying-by-read3image4upu: " + str(blocking3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3upu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking3upu-set-by-read3image4upu to: " + str(blocking3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking3upu-failed-by-read3image4upu: " + str(blocking3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4imageread3upu ['BLOCKING'] = blocking3upu             
             
        if 'ORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin3upu-trying-by-read3image4upu: " + str(origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3upu['X'] = 0 
                origin3upu['Y'] = 0  
                origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin3upu-set-by-read3image4upu to: " + str(origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin3upu-failed-by-read3image4upu: " + str(origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imageread3upu ['ORIGIN'] = origin3upu
            
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region3upu-trying-by-read3image4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO READ 
                region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO READ 
                region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region3upu-set-by-read3image4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region3upu-failed-by-read3image4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imageread3upu ['REGION'] = region3upu
        
            
             
        if 'ROWPITCH' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch3upu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch3upu-trying-by-read3image4upu: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch3upu-set-by-read3image4upu to: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch3upu-failed-by-read3image4upu: " + str(rowpitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '36Z-'
        
            task4imageread3upu ['ROWPITCH'] = rowpitch3upu     
             
        if 'POINTER' in keys3task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3upu = None
            size3upu = int(width3upu * height3upu)                                                                  # CALCULATED MEMORY
            if size3upu == 0:
                size3upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer3upu-trying-by-read3image4upu: " + str(pointer3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3upu = Arena4Xi(size3upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena3upu-got-by-read3image4upu: " + str(arena3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size3arena = arena3upu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size3arena-got-by-read2buffer4upu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer3arena = arena3upu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer3arena-got-by-read2buffer4upu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code3ack  = code3ack + '37D-' 
                
                if size3arena < size3upu:
                    size3upu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size3upu-set-by-read2buffer4upu to: " + str(size3upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '37E-'
                else:
                    pass
                    
                pointer3upu = buffer3arena
                message2print = "{(Step 37F/70) set" + "; pointer3upu-set-by-read2buffer4upu to: " + str(pointer3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code3ack  = code3ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer3upu-failed-by-read3image4upu: " + str(pointer3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '37Z-'
        
            task4imageread3upu ['POINTER'] = pointer3upu                            
            task4imageread3upu ['SIZE']    = size3upu                                                               # AUTOMATIC BY UPUXI
        
             
        if 'SLICEPITCH' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3upu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch3upu-trying-by-read3image4upu: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3upu = int(rowpitch3upu * height3upu)                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch3upu-set-by-read3image4upu to: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch3upu-failed-by-read3image4upu: " + str(slicepitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '38Z-'
            
            task4imageread3upu ['SLICEPITCH'] = slicepitch3upu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imageread3upu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-read3image4upu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #[ 'BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:04:35 10/10/11 Pacific Daylight Time}

                    
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread3upu-yielded-by-read3image4upu: " + str(task4imageread3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3readimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3upu = False
    channels3upu = ()
    message2print = "{(Step 40A/70) trying" + "channels3upu-trying-by-read3image4upu: " + str(channels3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        consumerchannel3upu, publisherchannel3upu = task4imageread3upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3upu-got-by-read3image4upu: " + str(consumerchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 584>

        message2print = "{(Step 40C/70) got" + "; publisherchannel3upu-got-by-read3image4upu: " + str(publisherchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 588> 
                             
        ack4channels3upu = True
        
        control3readimage4upu['CONSUMERCHANNEL']  = consumerchannel3upu
        control3readimage4upu['PUBLISHERCHANNEL'] = publisherchannel3upu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3upu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3upu = False
    queue3upu = None
    message2print = "{(Step 41A/70) trying" + "; queue3upu-trying-by-read3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3upu = task4imageread3upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3upu-got-by-read3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x000000000459DFD0>
              
        ack4queue3upu = True
        control3readimage4upu['QUEUE']  = queue3upu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3upu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4upu-trying-by-read3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4upu = DummyProcess4Xi(target=manoeuvre3readimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3upu, publisherchannel3upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4upu-created-by-read3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
        # { DummyProcess4Xi : __init__ ; (Step 1/7a) started; init-of-DummyProcess4Xi-started: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7b) passed; group-passed-to-init-of-DummyProcess4Xi: None; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7c) passed; target-passed-to-init-of-DummyProcess4Xi: <function manoeuvre3readimage4upu at 0x0000000003F01C48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7d) passed; name-passed-to-init-of-DummyProcess4Xi: None; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7e) passed; args-passed-to-init-of-DummyProcess4Xi: (<queue4xi.Queue4Xi object at 0x0000000003FB3208>, <read-write PipeConnection, handle 412>, <read-write PipeConnection, handle 416>); time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7f) passed; kwargs-passed-to-init-of-DummyProcess4Xi: {}; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7g) borning; dummyprocess2xi-of-DummyProcess4Xi-borning: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7h) inheriting; threading-Thread-inheriting-by-dummyprocess2xi-of-DummyProcess4Xi: <class 'threading.Thread'>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7i) sharing; is-alive-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function is_alive at 0x0000000002891AC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7j) sharing; get-name-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function getName at 0x0000000002891D48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7k) sharing; set-name-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function setName at 0x0000000002891DC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7l) sharing; is-daemon-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function isDaemon at 0x0000000002891C48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 1/7m) sharing; set_daemon-sharing-by-dummyprocess2xi-of-DummyProcess4Xi: <function setDaemon at 0x0000000002891CC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 2/7a) calling; Thread-init-calling-by-init-of-DummyProcess4Xi: <function __init__ at 0x0000000002891348>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 2/7b) performed; Thread-init-performed-by-init-of-DummyProcess4Xi: <function __init__ at 0x0000000002891348>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; set; ack2thread-set-by-init-of-DummyProcess4Xi to: True; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 3/7) set; dummyprocess2xi-pid-set-by-init-of-DummyProcess4Xi to: None; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 4/7) set; dummyprocess2xi-children-set-by-init-of-DummyProcess4Xi to: <WeakKeyDictionary at 69165912>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 5/7) set; dummyprocess2xi-start-called-set-by-init-of-DummyProcess4Xi to: False; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 6/7a) calling; current_dummy_process: <function current_thread at 0x0000000002894248>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 6/7b) set; dummyprocess2xi-parent-set-by-init-of-DummyProcess4Xi to: <_MainThread(MainThread, started 2424)>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7a) inherited; threading-Thread-inherited-by-dummyprocess2xi-of-DummyProcess4Xi: <class 'threading.Thread'>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7b) activated; dummyprocess2xi-of-DummyProcess4Xi-activated: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7c) shared; is-alive-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function is_alive at 0x0000000002891AC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7d) shared; get-name-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function getName at 0x0000000002891D48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7e) shared; set-name-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function setName at 0x0000000002891DC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7f) shared; is-daemon-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function isDaemon at 0x0000000002891C48>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7g) shared; set-daemon-shared-by-dummyprocess2xi-of-DummyProcess4Xi: <function setDaemon at 0x0000000002891CC8>; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { DummyProcess4Xi : __init__ ; (Step 7/7h) returning; time: 04:49:42 09/28/11 Pacific Daylight Time}
        # { read3image4upu : read3image4upu ; (Step 69B/70) createddummy3image4upu-created-by-read3image4upu: <DummyProcess4Xi(Thread-1, initial)>; time: 04:49:42 09/28/11 Pacific Daylight Time}
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-read3image4upu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18D-18F-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-read3image4upu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # ;  
        
    message2print = "{(Step 70C/70) returning" + "; control3readimage4upu-returning-by-read3image4upu: " + str(control3readimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000459DFD0>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 588>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 584>}; time: 07:04:35 10/10/11 Pacific Daylight Time}

        
    return control3readimage4upu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map3image4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   #                     
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP3IMAGE4UPU ', time4xi())           
#                    
#   response4map3image = {} 
#   task4map3image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map3image = map3image4upu(task4map3image)                                                                                                          
#       print('(Step 92B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP3IMAGE4UPU : ', response4map3image)
#    
#       queue4map3image4upu      = response4map3image['QUEUE']
#       channel4map3imagepublish4upu = response4map3image['PUBLISHERCHANNEL']
#       channel4map3imageconsume4upu = response4map3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP3IMAGE4UPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3COPYIMAGE4UPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4UPU ', time4xi())      
#   try:                               
#       message3send4upu = "<copy4xi>copy3mapimage4upu</copy4xi>"
#       channel4map3imagepublish4upu.send(message3send4upu)
#       print("(Step 93B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel4map3imagepublish4upu, " MESSAGE3SEND4UPU: " , message3send4upu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4UPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4UPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack3simulate4upu is True:
#       print("(Step 94A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi())   
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4upu(queue4map3image4upu, channel4map3imageconsume4upu, channel4map3imagepublish4upu)
#           print("(Step 94B/700) OF UPU3IMAGE4XI5TEST - PERFORMED MANOEUVRE3COPYIMAGE4UPU, RESPONSE3MANOEUVRE4COPY: ", response3manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4UPU ', time4xi())      
#   try:
#       message3receive4upu = channel4map3imageconsume4upu.recv()
#       print("(Step 95B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel4map3imageconsume4upu, " CONTROL & COMMAND: ", message3receive4upu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP3IMAGE4UPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3COPYIMAGE4UPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4UPU ', time4xi())     
#   try:                               
#       message3send4upu = "<ready4xi>ready3mapimage4upu</ready4xi>"
#       channel4map3imagepublish4upu.send(message3send4upu)
#       print("(Step 96B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel4map3imagepublish4upu, " MESSAGE3SEND4UPU: " , message3send4upu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4UPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4UPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack3simulate4upu is True:
#       print("(Step 97A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi())  
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4upu(queue4map3image4upu, channel4map3imageconsume4upu, channel4map3imagepublish4upu)
#           print("(Step 97B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", manoeuvre3copyimage4upu)
#       except:
#           print("(Step 97Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP3IMAGE4UPU ', time4xi())     
#   try:
#       message3receive4upu = channel4map3imageconsume4upu.recv()
#       print("(Step 98B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel4map3imageconsume4upu, " CONTROL & COMMAND: ", message3receive4upu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP3IMAGE4UPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE UPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF UPU3IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE UPUXI ', time4xi())       
#   try:        
#       path3dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file3upu = open(path3upu)
#       print("(Step 99B/700) OF UPU3IMAGE4XI5TEST - FILENO: ", file3upu.fileno( ))  
#       print("(Step 99C/700) OF UPU3IMAGE4XI5TEST - MODE: ", file3upu.mode)
#       print("(Step 99D/700) OF UPU3IMAGE4XI5TEST - NAME: ", file3upu.name)
#       offset3upu = 10
#       file3upu.seek(offset3upu)
#       print("(Step 99E/700) OF UPU3IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset3upu)
#       file3upu.close()
#       print("(Step 99F/700) OF UPU3IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path3upu)  
#   except:
#       print("(Step 87Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path3upu)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map3image4upu.put(path3upu)             
#       print("(Step 100B/700) OF UPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3upu) 
#   except:
#       print("(Step 88Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4UPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack3simulate4upu is True:
#       print("(Step 101A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi()) 
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4upu(queue4map3image4upu, channel4map3imageconsume4upu, channel4map3imagepublish4upu)
#           print("(Step 101B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", manoeuvre3copyimage4upu)
#       except:
#           print("(Step 101Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3receive4upu = channel4map3imageconsume4upu.recv()
#       print("(Step 102B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel4map3imageconsume4upu, " CONTROL & COMMAND: ", message3receive4upu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE3COPYIMAGE4UPU  - LET MANOEUVRE3COPYIMAGE4UPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack3simulate4upu is True:
#       print("(Step 103A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi())  
#       try:        
#           response3manoeuvre4copy = manoeuvre3mapimage4upu(queue4map3image4upu, channel4map3imageconsume4upu, channel4map3imagepublish4upu)
#           print("(Step 103B/700) OF UPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4UPU ", response3manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3COPYIMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map3image4upu(task4imagemap3upu=None):

    '''
    maps 3D images for UPU
    '''
    global upu3image4xi                                                                                             # 3D-IMAGES FOR UPU  
    global control3mapimage4upu                                                                                     # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map3image4upu"
    method2print = "map3image4upu"

    message2print = "{(Step 1A/33) started" + "; map3image4upu-started: " + str(map3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap3upu-to-map3image4upu: " + str(task4imagemap3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu3image4xi-created-by-map3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4upu-skipped-by-map3image4upu: " + str(map3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 1E/70) set" + "; upu3image4xi-set-by-map3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu3image4xi-yielded-by-map3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap3upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap3upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap3upu-created-by-map3image4upu to: " + str(task4imagemap3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4upu-created-by-map3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "map3image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "map3image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-map3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap3upu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap3upu['FUNCTION3CALLBACK'] = function3callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap3upu-created-by-map3image4upu to: " + str(task4imagemap3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-map3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-map3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-map3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagemap3upu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 3A/70) trying" + "; context3upu-trying-by-map3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3upu-got-by-map3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3upu-failed-by-map3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagemap3upu ['CONTEXT'] = context3upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3upu-trying-by-map3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3upu-got-by-map3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3upu-failed-by-map3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imagemap3upu ['DEVICE'] = device3upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo3upu-trying-by-map3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo3upu-got-by-map3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo3upu-failed-by-map3image4upu: " + str(imageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imagemap3upu ['IMAGE'] = imageinfo3upu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3upu-trying-by-map3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3upu-got-by-map3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3upu-failed-by-map3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imagemap3upu ['QUEUE'] = queue3upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3upu-trying-by-map3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3upu-got-by-map3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3upu-failed-by-map3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagemap3upu ['PIPE'] = pipe3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3upu-trying-by-map3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3upu-got-by-map3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3upu-failed-by-map3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imagemap3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3upu-trying-by-map3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3upu-got-by-map3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3upu-failed-by-map3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagemap3upu ['BUFFER'] = bufferinfo3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3upu-trying-by-map3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo3upu-got-by-map3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3upu-failed-by-map3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagemap3upu ['EVENT'] = eventinfo3upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist3upu-trying-by-map3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist3upu-set-by-map3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist3upu-failed-by-map3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagemap3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity3upu-trying-by-map3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity3upu-set-by-map3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-map3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagemap3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3upu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking3upu-trying-by-map3image4upu: " + str(blocking3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3upu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking3upu-set-by-map3image4upu to: " + str(blocking3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking3upu-failed-by-map3image4upu: " + str(blocking3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imagemap3upu ['BLOCKING'] = blocking3upu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin3upu-trying-by-map3image4upu: " + str(origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3upu['X'] = 0 
                origin3upu['Y'] = 0  
                origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin3upu-set-by-map3image4upu to: " + str(origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin3upu-failed-by-map3image4upu: " + str(origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagemap3upu ['ORIGIN'] = origin3upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region3upu-trying-by-map3image4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO MAP 
                region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO MAP 
                region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region3upu-set-by-map3image4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region3upu-failed-by-map3image4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imagemap3upu ['REGION'] = region3upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch3upu = 0
            size3element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 8                                                                                  # ONE DIRECTIONS IN 3D
            width3upu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3upu-trying-by-map3image4upu: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3upu-set-by-map3image4upu to: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3upu-failed-by-map3image4upu: " + str(rowpitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imagemap3upu ['ROWPITCH'] = rowpitch3upu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3upu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch3upu-trying-by-map3image4upu: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3upu = int(rowpitch3upu * height3upu)                                                       
                message2print = "{(Step 18B/70) set" + "; slicepitch3upu-set-by-map3image4upu to: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code3ack  = code3ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch3upu-failed-by-map3image4upu: " + str(slicepitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '18Z-'
            
            task4imagemap3upu ['SLICEPITCH'] = slicepitch3upu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags3upu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags3upu-trying-by-map3image4upu: " + str(flags3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags3upu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags3upu-set-by-map3image4upu to: " + str(flags3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags3upu-failed-by-map3image4upu: " + str(flags3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '19Z-'
        
            task4imagemap3upu ['FLAGS'] = flags3upu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagemap3upu.keys()
        message2print = "{(Step 20/70) got" + "; keys3task-got-by-map3image4upu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
 

        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                              # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4upu-created-by-map3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "map3image4upu"                                                          # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "map3image4upu"                                                          # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-map3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap3upu['DATA3CALLBACK'] = args3callback                                                   # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap3upu['FUNCTION3CALLBACK'] = function3callback4upu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap3upu-created-by-map3image4upu to: " + str(task4imagemap3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-map3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-map3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-map3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                 
            task4imagemap3upu['PLATFORM'] = platform3xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 23A/70) trying" + "; context3upu-trying-by-map3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3upu-got-by-map3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3upu-failed-by-map3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imagemap3upu ['CONTEXT'] = context3upu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3upu-trying-by-map3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3upu-got-by-map3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3upu-failed-by-map3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imagemap3upu ['DEVICE'] = device3upu
        
        if 'IMAGE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo3upu-trying-by-map3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo3upu-got-by-map3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo3upu-failed-by-map3image4upu: " + str(imageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imagemap3upu ['IMAGE'] = imageinfo3upu         
                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue3upu-trying-by-map3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue3upu-got-by-map3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue3upu-failed-by-map3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'
                
            task4imagemap3upu ['QUEUE'] = queue3upu
        
        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe3upu-trying-by-map3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe3upu-got-by-map3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe3upu-failed-by-map3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imagemap3upu ['PIPE'] = pipe3upu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo3upu-trying-by-map3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo3upu-got-by-map3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo3upu-failed-by-map3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
        
            task4imagemap3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
        
        
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo3upu-trying-by-map3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                                   # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo3upu-got-by-map3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo3upu-failed-by-map3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imagemap3upu ['BUFFER'] = bufferinfo3upu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo3upu-trying-by-map3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo3upu-got-by-map3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo3upu-failed-by-map3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imagemap3upu ['EVENT'] = eventinfo3upu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist3upu-trying-by-map3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist3upu-set-by-map3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist3upu-failed-by-map3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imagemap3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity3upu-trying-by-map3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity3upu-set-by-map3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-map3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imagemap3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                                 
                 
        if 'BLOCKING' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3upu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking3upu-trying-by-map3image4upu: " + str(blocking3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3upu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking3upu-set-by-map3image4upu to: " + str(blocking3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking3upu-failed-by-map3image4upu: " + str(blocking3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4imagemap3upu ['BLOCKING'] = blocking3upu             
             
        if 'ORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin3upu-trying-by-map3image4upu: " + str(origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3upu['X'] = 0 
                origin3upu['Y'] = 0  
                origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin3upu-set-by-map3image4upu to: " + str(origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin3upu-failed-by-map3image4upu: " + str(origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imagemap3upu ['ORIGIN'] = origin3upu
            
             
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region3upu-trying-by-map3image4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO MAP 
                region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO MAP 
                region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region3upu-set-by-map3image4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region3upu-failed-by-map3image4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imagemap3upu ['REGION'] = region3upu
        
            
             
        if 'ROWPITCH' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch3upu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch3upu-trying-by-map3image4upu: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch3upu-set-by-map3image4upu to: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch3upu-failed-by-map3image4upu: " + str(rowpitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '36Z-'
        
            task4imagemap3upu ['ROWPITCH'] = rowpitch3upu     
   
        if 'SLICEPITCH' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3upu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch3upu-trying-by-map3image4upu: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3upu = int(rowpitch3upu * height3upu)                                                       
                message2print = "{(Step 38B/70) set" + "; slicepitch3upu-set-by-map3image4upu to: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch3upu-failed-by-map3image4upu: " + str(slicepitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '38Z-'
            
            task4imagemap3upu ['SLICEPITCH'] = slicepitch3upu
   
        if 'FLAGS' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags3upu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags3upu-trying-by-map3image4upu: " + str(flags3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags3upu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags3upu-set-by-map3image4upu to: " + str(flags3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags3upu-failed-by-map3image4upu: " + str(flags3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '39Z-'
            
            task4imagemap3upu ['FLAGS'] = flags3upu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagemap3upu.keys()
        message2print = "{(Step 40A/70) set" + "; keys3task-set-by-map3image4upu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap3upu-yielded-by-map3image4upu: " + str(task4imagemap3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3mapimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3upu = False
    channels3upu = ()
    message2print = "{(Step 40A/70) trying" + "channels3upu-trying-by-map3image4upu: " + str(channels3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3upu, publisherchannel3upu = task4imagemap3upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3upu-got-by-map3image4upu: " + str(consumerchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3upu-got-by-map3image4upu: " + str(publisherchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
                    
        ack4channels3upu = True
        
        control3mapimage4upu['CONSUMERCHANNEL']  = consumerchannel3upu
        
        
        control3mapimage4upu['PUBLISHERCHANNEL'] = publisherchannel3upu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3upu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3upu = False
    queue3upu = None
    message2print = "{(Step 41A/70) trying" + "queue3upu-trying-by-map3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3upu = task4imagemap3upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3upu-got-by-map3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
        
              
        ack4queue3upu = True
        control3mapimage4upu['QUEUE']  = queue3upu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3upu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4upu-trying-by-map3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4upu = DummyProcess4Xi(target=manoeuvre3mapimage4upu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue3upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3upu, publisherchannel3upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4upu-created-by-map3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-map3image4upu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-map3image4upu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control3mapimage4upu-returning-by-map3image4upu: " + str(control3mapimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
    

        
    return control3mapimage4upu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3copyimage4upu" usage = "control function for copy3image4upu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3copyimage4upu(queue3copymanoeuvre4upu, consumerchannel3copymanoeuvre4upu, publisherchannel3copymanoeuvre4upu):
    '''
    manoeuvres copy control and command
    '''
    global state3copyimage4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3copyimagestate4upu                                                                                        # MAX FOR STATE
    global ack3copyimageinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message3copyimageget4upu                                 
    global message3copyimageput4upu
    global message3copyimagereceive4upu
    global message3copyimagesend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3copyimage4upu" 
    method2print = "manoeuvre3copyimage4upu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3copyimage4upu-started: " + str(manoeuvre3copyimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue3copymanoeuvre4upu-to-manoeuvre3copyimage4upu: " + str(queue3copymanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3copymanoeuvre4upu-to-manoeuvre3copyimage4upu: " + str(consumerchannel3copymanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3copymanoeuvre4upu-to-manoeuvre3copyimage4upu: " + str(publisherchannel3copymanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3copyimage4upu-try-by-manoeuvre3copyimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3copyimage4upu is None:
            state3copyimage4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3copyimage4upu-initialized-by-manoeuvre3copyimage4upu to: " + str(state3copyimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3copyimage4upu-used-by-manoeuvre3copyimage4upu: " + str(state3copyimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
    except:
        state3copyimage4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3copyimage4upu-initialized-by-manoeuvre3copyimage4upu to: " + str(state3copyimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimageinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3copyimageinfo4upu-initialized-by-manoeuvre3copyimage4upu to: " + str(ack3copyimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimageget4upu     = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3copyimageget4upu-initialized-by-manoeuvre3copyimage4upu to: " + str(message3copyimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimageput4upu     = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3copyimageput4upu-initialized-by-manoeuvre3copyimage4upu to: " + str(message3copyimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagereceive4upu = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3copyimagereceive4upu-initialized-by-manoeuvre3copyimage4upu to: " + str(message3copyimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagesend4upu    = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3copyimagesend4upu-initialized-by-manoeuvre3copyimage4upu to: " + str(message3copyimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagestate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3copyimagestate4upu-initialized-by-manoeuvre3copyimage4upu to: " + str( max3copyimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3copyimage4upu = state3copyimage4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3copyimage4upu-added-by-manoeuvre3copyimage4upu: " + str(state3copyimage4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3copyimage4upu > max3copyimagestate4upu:
        state3copyimage4upu = 0
        message2print = "{(Step 0I/33) set" + "; state3copyimage4upu-set-by-manoeuvre3copyimage4upu to: " + str(state3copyimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimageinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack3copyimageinfo4upu-set-by-manoeuvre3copyimage4upu to: " + str(ack3copyimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimageget4upu     = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3copyimageget4upu-set-by-manoeuvre3copyimage4upu to: " + str(message3copyimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimageput4upu     = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3copyimageput4upu-set-by-manoeuvre3copyimage4upu to: " + str(message3copyimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagereceive4upu = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3copyimagereceive4upu-set-by-manoeuvre3copyimage4upu to: " + str(message3copyimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagesend4upu    = '<default4xi>default3copy4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3copyimagesend4upu-set-by-manoeuvre3copyimage4upu to: " + str(message3copyimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagestate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3copyimagestate4upu-set-by-manoeuvre3copyimage4upu to: " + str( max3copyimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimage4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3copyimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagereceive4upu = consumerchannel3copymanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message3copyimagereceive4upu-received-by-manoeuvre3copyimage4upu: " + str(message3copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagereceive4upu == '<copy4xi>copy3copy4upu</copy4xi>':
                message3copyimagesend4upu = '<roger4xi>roger3copy4upu</roger4xi>'
            else:
                message3copyimagesend4upu = '<negative4xi><negative3copy4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copymanoeuvre4upu.send(message3copyimagesend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message3copyimagesend4upu-sent-by-manoeuvre3copyimage4upu: " + str(message3copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3copyimagereceive4upu-failed-by-manoeuvre3copyimage4upu: " + str(message3copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimage4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3copyimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagereceive4upu = consumerchannel3copymanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message3copyimagereceive4upu-received-by-manoeuvre3copyimage4upu: " + str(message3copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagereceive4upu == '<ready4xi>ready3copy4upu</ready4xi>':
                message3copyimagesend4upu = '<ready4xi>ready3copy4upu</ready4xi>'
            else:
                message3copyimagesend4upu = '<negative4xi><negative3copy4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copymanoeuvre4upu.send(message3copyimagesend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message3copyimagesend4upu-sent-by-manoeuvre3copyimage4upu: " + str(message3copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3copyimagereceive4upu-failed-by-manoeuvre3copyimage4upu: " + str(message3copyimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimage4upu == 3:    
        info3image4upu = None
        message2print = "{(Step 3A/70) try" + "; info3image4upu-try-by-manoeuvre3copyimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4upu = queue3copymanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4upu-got-by-manoeuvre3copyimage4upu: " + str(info3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4upu is None:
                code3nack  = code3nack + '3Z1-'
                ack3copyimageinfo4upu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3copyimageinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3copymanoeuvre4upu-failed-by-manoeuvre3copyimage4upu: " + str(queue3copymanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3copyimagesend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3copyimageinfo4upu is True:
                message3copyimagesend4upu = '<ack4xi>ack3imageinfo4upu</ack4xi>'
            else:
                message3copyimagesend4upu = '<nack4xi>nack3imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3copyimagesend4upu-set-by-manoeuvre3copyimage4upu to: " + str(message3copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagesend4upu-failed-by-manoeuvre3copyimage4upu: " + str(message3copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3copymanoeuvre4upu-calling-by-manoeuvre3copyimage4upu: " + str(publisherchannel3copymanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3copyimagesend4upu-passing-to-send-by-manoeuvre3copyimage4upu: " + str(message3copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagesend4upu-failed-by-manoeuvre3copyimage4upu: " + str(message3copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3copymanoeuvre4upu.send(message3copyimagesend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message3copyimagesend4upu-sent-by-manoeuvre3copyimage4upu: " + str(message3copyimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #         
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3copyimagesend4upu-failed-by-manoeuvre3copyimage4upu: " + str(message3copyimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4copy['ACK3INFO4UPU']        = ack3copyimageinfo4upu
    response3manouvre4copy['CODE3ACK4UPU']        = code3ack
    response3manouvre4copy['CODE3NACK4UPU']       = code3nack
    response3manouvre4copy['MESSAGE3GET4UPU']     = message3copyimageget4upu
    response3manouvre4copy['MESSAGE3PUT4UPU']     = message3copyimageput4upu
    response3manouvre4copy['MESSAGE3RECEIVE4UPU'] = message3copyimagereceive4upu
    response3manouvre4copy['MESSAGE3SEND4UPU']    = message3copyimagesend4upu
    response3manouvre4copy['STATE3MANOEUVRE4UPU'] = state3copyimage4upu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4copy-returning-by-manoeuvre3copyimage4upu: " + str(response3manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 

 
    #2 

   
    #3 

    
    #4 

    return response3manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3copyimagetobuffer4upu" usage = "control function for copy3imagetobuffer4upu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3copyimagetobuffer4upu(queue3copyimagetobuffermanoeuvre4upu, consumerchannel3copyimagetobuffermanoeuvre4upu, publisherchannel3copyimagetobuffermanoeuvre4upu):
    '''
    manoeuvres copy control and command for copy3imagetobuffer4upu
    '''
    global state3copyimagetobuffer4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3copyimagetobufferstate4upu                                                                                        # MAX FOR STATE
    global ack3copyimagetobufferinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message3copyimagetobufferget4upu                                 
    global message3copyimagetobufferput4upu
    global message3copyimagetobufferreceive4upu
    global message3copyimagetobuffersend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3copyimagetobuffer4upu" 
    method2print = "manoeuvre3copyimagetobuffer4upu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3copyimagetobuffer4upu-started: " + str(manoeuvre3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue3copyimagetobuffermanoeuvre4upu-to-manoeuvre3copyimagetobuffer4upu: " + str(queue3copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3copyimagetobuffermanoeuvre4upu-to-manoeuvre3copyimagetobuffer4upu: " + str(consumerchannel3copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3copyimagetobuffermanoeuvre4upu-to-manoeuvre3copyimagetobuffer4upu: " + str(publisherchannel3copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3copyimagetobuffer4upu-try-by-manoeuvre3copyimagetobuffer4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3copyimagetobuffer4upu is None:
            state3copyimagetobuffer4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3copyimagetobuffer4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str(state3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3copyimagetobuffer4upu-used-by-manoeuvre3copyimagetobuffer4upu: " + str(state3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
    except:
        state3copyimagetobuffer4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3copyimagetobuffer4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str(state3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimagetobufferinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3copyimagetobufferinfo4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str(ack3copyimagetobufferinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimagetobufferget4upu     = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3copyimagetobufferget4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobufferget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimagetobufferput4upu     = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3copyimagetobufferput4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobufferput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobufferreceive4upu = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3copyimagetobufferreceive4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobufferreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobuffersend4upu    = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3copyimagetobuffersend4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagetobufferstate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3copyimagetobufferstate4upu-initialized-by-manoeuvre3copyimagetobuffer4upu to: " + str( max3copyimagetobufferstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3copyimagetobuffer4upu = state3copyimagetobuffer4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3copyimagetobuffer4upu-added-by-manoeuvre3copyimagetobuffer4upu: " + str(state3copyimagetobuffer4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
     
    if state3copyimagetobuffer4upu > max3copyimagetobufferstate4upu:
        state3copyimagetobuffer4upu = 0
        message2print = "{(Step 0I/33) set" + "; state3copyimagetobuffer4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str(state3copyimagetobuffer4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copyimagetobufferinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack3copyimagetobufferinfo4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str(ack3copyimagetobufferinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copyimagetobufferget4upu     = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3copyimagetobufferget4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobufferget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copyimagetobufferput4upu     = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3copyimagetobufferput4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobufferput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobufferreceive4upu = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3copyimagetobufferreceive4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobufferreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copyimagetobuffersend4upu    = '<default4xi>default3copyimagetobuffer4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3copyimagetobuffersend4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copyimagetobufferstate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3copyimagetobufferstate4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str( max3copyimagetobufferstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimagetobuffer4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3copyimagetobuffer4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagetobufferreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagetobufferreceive4upu = consumerchannel3copyimagetobuffermanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message3copyimagetobufferreceive4upu-received-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagetobufferreceive4upu == '<copy4xi>copy3copyimagetobuffer4upu</copy4xi>': 
                message3copyimagetobuffersend4upu = '<roger4xi>roger3copyimagetobuffer4upu</roger4xi>'
            else:
                message3copyimagetobuffersend4upu = '<negative4xi><negative3copyimagetobuffer4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copyimagetobuffermanoeuvre4upu.send(message3copyimagetobuffersend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message3copyimagetobuffersend4upu-sent-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3copyimagetobufferreceive4upu-failed-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimagetobuffer4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3copyimagetobuffer4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copyimagetobufferreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copyimagetobufferreceive4upu = consumerchannel3copyimagetobuffermanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message3copyimagetobufferreceive4upu-received-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copyimagetobufferreceive4upu == '<ready4xi>ready3copyimagetobuffer4upu</ready4xi>':
                message3copyimagetobuffersend4upu = '<ready4xi>ready3copyimagetobuffer4upu</ready4xi>'
            else:
                message3copyimagetobuffersend4upu = '<negative4xi><negative3copyimagetobuffer4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copyimagetobuffermanoeuvre4upu.send(message3copyimagetobuffersend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message3copyimagetobuffersend4upu-sent-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3copyimagetobufferreceive4upu-failed-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobufferreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copyimagetobuffer4upu == 3:    
        info3image4upu = None
        message2print = "{(Step 3A/70) try" + "; info3image4upu-try-by-manoeuvre3copyimagetobuffer4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4upu = queue3copyimagetobuffermanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4upu-got-by-manoeuvre3copyimagetobuffer4upu: " + str(info3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4upu is None:
                code3nack  = code3nack + '3Z1-'
                ack3copyimagetobufferinfo4upu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3copyimagetobufferinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3copyimagetobuffermanoeuvre4upu-failed-by-manoeuvre3copyimagetobuffer4upu: " + str(queue3copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3copyimagetobuffersend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3copyimagetobufferinfo4upu is True:
                message3copyimagetobuffersend4upu = '<ack4xi>ack3imageinfo4upu</ack4xi>'
            else:
                message3copyimagetobuffersend4upu = '<nack4xi>nack3imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3copyimagetobuffersend4upu-set-by-manoeuvre3copyimagetobuffer4upu to: " + str(message3copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagetobuffersend4upu-failed-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3copyimagetobuffermanoeuvre4upu-calling-by-manoeuvre3copyimagetobuffer4upu: " + str(publisherchannel3copyimagetobuffermanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3copyimagetobuffersend4upu-passing-to-send-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copyimagetobuffersend4upu-failed-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3copyimagetobuffermanoeuvre4upu.send(message3copyimagetobuffersend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message3copyimagetobuffersend4upu-sent-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobuffersend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #       
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3copyimagetobuffersend4upu-failed-by-manoeuvre3copyimagetobuffer4upu: " + str(message3copyimagetobuffersend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4copyimagetobuffer['ACK3INFO4UPU']        = ack3copyimagetobufferinfo4upu
    response3manouvre4copyimagetobuffer['CODE3ACK4UPU']        = code3ack
    response3manouvre4copyimagetobuffer['CODE3NACK4UPU']       = code3nack
    response3manouvre4copyimagetobuffer['MESSAGE3GET4UPU']     = message3copyimagetobufferget4upu
    response3manouvre4copyimagetobuffer['MESSAGE3PUT4UPU']     = message3copyimagetobufferput4upu
    response3manouvre4copyimagetobuffer['MESSAGE3RECEIVE4UPU'] = message3copyimagetobufferreceive4upu
    response3manouvre4copyimagetobuffer['MESSAGE3SEND4UPU']    = message3copyimagetobuffersend4upu
    response3manouvre4copyimagetobuffer['STATE3MANOEUVRE4UPU'] = state3copyimagetobuffer4upu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4copyimagetobuffer-returning-by-manoeuvre3copyimagetobuffer4upu: " + str(response3manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 

    
    #2 

    #3 


    #4 


    return response3manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3copybuffertoimage4upu" usage = "control function for copy3imagetobuffer4upu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3copybuffertoimage4upu(queue3copybuffertoimagemanoeuvre4upu, consumerchannel3copybuffertoimagemanoeuvre4upu, publisherchannel3copybuffertoimagemanoeuvre4upu):
    '''
    manoeuvres copy control and command for copy3buffertoimage4upu
    '''
    global state3copybuffertoimage4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3copybuffertoimagestate4upu                                                                                        # MAX FOR STATE
    global ack3copybuffertoimageinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message3copybuffertoimageget4upu                                 
    global message3copybuffertoimageput4upu
    global message3copybuffertoimagereceive4upu
    global message3copybuffertoimagesend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3copybuffertoimage4upu" 
    method2print = "manoeuvre3copybuffertoimage4upu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3copybuffertoimage4upu-started: " + str(manoeuvre3copybuffertoimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue3copybuffertoimagemanoeuvre4upu-to-manoeuvre3copybuffertoimage4upu: " + str(queue3copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3copybuffertoimagemanoeuvre4upu-to-manoeuvre3copybuffertoimage4upu: " + str(consumerchannel3copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3copybuffertoimagemanoeuvre4upu-to-manoeuvre3copybuffertoimage4upu: " + str(publisherchannel3copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3copybuffertoimage4upu-try-by-manoeuvre3copybuffertoimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3copybuffertoimage4upu is None:
            state3copybuffertoimage4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3copybuffertoimage4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str(state3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3copybuffertoimage4upu-used-by-manoeuvre3copybuffertoimage4upu: " + str(state3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
    except:
        state3copybuffertoimage4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3copybuffertoimage4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str(state3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copybuffertoimageinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3copybuffertoimageinfo4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str(ack3copybuffertoimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copybuffertoimageget4upu     = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3copybuffertoimageget4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copybuffertoimageput4upu     = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3copybuffertoimageput4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagereceive4upu = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3copybuffertoimagereceive4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagesend4upu    = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3copybuffertoimagesend4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copybuffertoimagestate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3copybuffertoimagestate4upu-initialized-by-manoeuvre3copybuffertoimage4upu to: " + str( max3copybuffertoimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3copybuffertoimage4upu = state3copybuffertoimage4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3copybuffertoimage4upu-added-by-manoeuvre3copybuffertoimage4upu: " + str(state3copybuffertoimage4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
     
    if state3copybuffertoimage4upu > max3copybuffertoimagestate4upu:
        state3copybuffertoimage4upu = 0
        message2print = "{(Step 0I/33) set" + "; state3copybuffertoimage4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str(state3copybuffertoimage4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3copybuffertoimageinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack3copybuffertoimageinfo4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str(ack3copybuffertoimageinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3copybuffertoimageget4upu     = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3copybuffertoimageget4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimageget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3copybuffertoimageput4upu     = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3copybuffertoimageput4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimageput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagereceive4upu = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3copybuffertoimagereceive4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimagereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3copybuffertoimagesend4upu    = '<default4xi>default3copybuffertoimage4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3copybuffertoimagesend4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3copybuffertoimagestate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3copybuffertoimagestate4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str( max3copybuffertoimagestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copybuffertoimage4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3copybuffertoimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copybuffertoimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copybuffertoimagereceive4upu = consumerchannel3copybuffertoimagemanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message3copybuffertoimagereceive4upu-received-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copybuffertoimagereceive4upu == '<copy4xi>copy3copybuffertoimage4upu</copy4xi>':
                message3copybuffertoimagesend4upu = '<roger4xi>roger3copybuffertoimage4upu</roger4xi>'
            else:
                message3copybuffertoimagesend4upu = '<negative4xi><negative3copybuffertoimage4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copybuffertoimagemanoeuvre4upu.send(message3copybuffertoimagesend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message3copybuffertoimagesend4upu-sent-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3copybuffertoimagereceive4upu-failed-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copybuffertoimage4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3copybuffertoimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3copybuffertoimagereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3copybuffertoimagereceive4upu = consumerchannel3copybuffertoimagemanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message3copybuffertoimagereceive4upu-received-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3copybuffertoimagereceive4upu == '<ready4xi>ready3copybuffertoimage4upu</ready4xi>':
                message3copybuffertoimagesend4upu = '<ready4xi>ready3copybuffertoimage4upu</ready4xi>'
            else:
                message3copybuffertoimagesend4upu = '<negative4xi><negative3copybuffertoimage4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3copybuffertoimagemanoeuvre4upu.send(message3copybuffertoimagesend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message3copybuffertoimagesend4upu-sent-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3copybuffertoimagereceive4upu-failed-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3copybuffertoimage4upu == 3:    
        info3image4upu = None
        message2print = "{(Step 3A/70) try" + "; info3image4upu-try-by-manoeuvre3copybuffertoimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4upu = queue3copybuffertoimagemanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4upu-got-by-manoeuvre3copybuffertoimage4upu: " + str(info3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4upu is None:
                code3nack  = code3nack + '3Z1-'
                ack3copybuffertoimageinfo4upu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3copybuffertoimageinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3copybuffertoimagemanoeuvre4upu-failed-by-manoeuvre3copybuffertoimage4upu: " + str(queue3copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3copybuffertoimagesend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3copybuffertoimageinfo4upu is True:
                message3copybuffertoimagesend4upu = '<ack4xi>ack3imageinfo4upu</ack4xi>'
            else:
                message3copybuffertoimagesend4upu = '<nack4xi>nack3imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3copybuffertoimagesend4upu-set-by-manoeuvre3copybuffertoimage4upu to: " + str(message3copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copybuffertoimagesend4upu-failed-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3copybuffertoimagemanoeuvre4upu-calling-by-manoeuvre3copybuffertoimage4upu: " + str(publisherchannel3copybuffertoimagemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3copybuffertoimagesend4upu-passing-to-send-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3copybuffertoimagesend4upu-failed-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3copybuffertoimagemanoeuvre4upu.send(message3copybuffertoimagesend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message3copybuffertoimagesend4upu-sent-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #      
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3copybuffertoimagesend4upu-failed-by-manoeuvre3copybuffertoimage4upu: " + str(message3copybuffertoimagesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4copybuffertoimage['ACK3INFO4UPU']        = ack3copybuffertoimageinfo4upu
    response3manouvre4copybuffertoimage['CODE3ACK4UPU']        = code3ack
    response3manouvre4copybuffertoimage['CODE3NACK4UPU']       = code3nack
    response3manouvre4copybuffertoimage['MESSAGE3GET4UPU']     = message3copybuffertoimageget4upu
    response3manouvre4copybuffertoimage['MESSAGE3PUT4UPU']     = message3copybuffertoimageput4upu
    response3manouvre4copybuffertoimage['MESSAGE3RECEIVE4UPU'] = message3copybuffertoimagereceive4upu
    response3manouvre4copybuffertoimage['MESSAGE3SEND4UPU']    = message3copybuffertoimagesend4upu
    response3manouvre4copybuffertoimage['STATE3MANOEUVRE4UPU'] = state3copybuffertoimage4upu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4copybuffertoimage-returning-by-manoeuvre3copybuffertoimage4upu: " + str(response3manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 

    
    #2 

    #3 
 

    #4 

    return response3manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3readimage4upu" usage = "control function for read3image4upu function in this module">
# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3readimage4upu(queue3readmanoeuvre4upu, consumerchannel3readmanoeuvre4upu, publisherchannel3readmanoeuvre4upu):
    '''
    manoeuvres read control and command
    '''
    global state3read4upu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max3readstate4upu                                                                                        # MAX FOR STATE
    global ack3readinfo4upu                                                                                         # ACK FOR IMAGE INFO
    global message3readget4upu                                 
    global message3readput4upu
    global message3readreceive4upu
    global message3readsend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3readimage4upu" 
    method2print = "manoeuvre3readimage4upu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3readimage4upu-started: " + str(manoeuvre3readimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue3readmanoeuvre4upu-to-manoeuvre3readimage4upu: " + str(queue3readmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3readmanoeuvre4upu-to-manoeuvre3readimage4upu: " + str(consumerchannel3readmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3readmanoeuvre4upu-to-manoeuvre3readimage4upu: " + str(publisherchannel3readmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    #   
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3read4upu-try-by-manoeuvre3readimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3read4upu is None:
            state3read4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3read4upu-initialized-by-manoeuvre3readimage4upu to: " + str(state3read4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3read4upu-used-by-manoeuvre3readimage4upu: " + str(state3read4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
    except:
        state3read4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3read4upu-initialized-by-manoeuvre3readimage4upu to: " + str(state3read4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3readinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3readinfo4upu-initialized-by-manoeuvre3readimage4upu to: " + str(ack3readinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3readget4upu     = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3readget4upu-initialized-by-manoeuvre3readimage4upu to: " + str(message3readget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3readput4upu     = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3readput4upu-initialized-by-manoeuvre3readimage4upu to: " + str(message3readput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readreceive4upu = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3readreceive4upu-initialized-by-manoeuvre3readimage4upu to: " + str(message3readreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readsend4upu    = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3readsend4upu-initialized-by-manoeuvre3readimage4upu to: " + str(message3readsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3readstate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3readstate4upu-initialized-by-manoeuvre3readimage4upu to: " + str( max3readstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3read4upu = state3read4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3read4upu-added-by-manoeuvre3readimage4upu: " + str(state3read4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3read4upu > max3readstate4upu:
        state3read4upu = 0
        message2print = "{(Step 0I/33) set" + "; state3read4upu-set-by-manoeuvre3readimage4upu to: " + str(state3read4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3readinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack3readinfo4upu-set-by-manoeuvre3readimage4upu to: " + str(ack3readinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3readget4upu     = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3readget4upu-set-by-manoeuvre3readimage4upu to: " + str(message3readget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3readput4upu     = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3readput4upu-set-by-manoeuvre3readimage4upu to: " + str(message3readput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readreceive4upu = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3readreceive4upu-set-by-manoeuvre3readimage4upu to: " + str(message3readreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3readsend4upu    = '<default4xi>default3read4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3readsend4upu-set-by-manoeuvre3readimage4upu to: " + str(message3readsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3readstate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3readstate4upu-set-by-manoeuvre3readimage4upu to: " + str( max3readstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3read4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3readimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3readreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3readreceive4upu = consumerchannel3readmanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message3readreceive4upu-received-by-manoeuvre3readimage4upu: " + str(message3readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3readreceive4upu == '<copy4xi>copy3read4upu</copy4xi>':
                message3readsend4upu = '<roger4xi>roger3read4upu</roger4xi>'
            else:
                message3readsend4upu = '<negative4xi><negative3read4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3readmanoeuvre4upu.send(message3readsend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message3readsend4upu-sent-by-manoeuvre3readimage4upu: " + str(message3readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3readreceive4upu-failed-by-manoeuvre3readimage4upu: " + str(message3readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3read4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3readimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3readreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3readreceive4upu = consumerchannel3readmanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message3readreceive4upu-received-by-manoeuvre3readimage4upu: " + str(message3readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3readreceive4upu == '<ready4xi>ready3read4upu</ready4xi>':
                message3readsend4upu = '<ready4xi>ready3read4upu</ready4xi>'
            else:
                message3readsend4upu = '<negative4xi><negative3ready4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3readmanoeuvre4upu.send(message3readsend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message3readsend4upu-sent-by-manoeuvre3readimage4upu: " + str(message3readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3readreceive4upu-failed-by-manoeuvre3readimage4upu: " + str(message3readreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3read4upu == 3:    
        info3image4upu = None
        message2print = "{(Step 3A/70) try" + "; info3image4upu-try-by-manoeuvre3readimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4upu = queue3readmanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4upu-got-by-manoeuvre3readimage4upu: " + str(info3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4upu is None:
                code3nack  = code3nack + '3Z1-'
                ack3readinfo4upu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3readinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3readmanoeuvre4upu-failed-by-manoeuvre3readimage4upu: " + str(queue3readmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3readsend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3readinfo4upu is True:
                message3readsend4upu = '<ack4xi>ack3imageinfo4upu</ack4xi>'
            else:
                message3readsend4upu = '<nack4xi>nack3imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3readsend4upu-set-by-manoeuvre3readimage4upu to: " + str(message3readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3readsend4upu-failed-by-manoeuvre3readimage4upu: " + str(message3readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3readmanoeuvre4upu-calling-by-manoeuvre3readimage4upu: " + str(publisherchannel3readmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3readsend4upu-passing-to-send-by-manoeuvre3readimage4upu: " + str(message3readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3readsend4upu-failed-by-manoeuvre3readimage4upu: " + str(message3readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3readmanoeuvre4upu.send(message3readsend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message3readsend4upu-sent-by-manoeuvre3readimage4upu: " + str(message3readsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #         
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3readsend4upu-failed-by-manoeuvre3readimage4upu: " + str(message3readsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4read['ACK3INFO4UPU']        = ack3readinfo4upu
    response3manouvre4read['CODE3ACK4UPU']        = code3ack
    response3manouvre4read['CODE3NACK4UPU']       = code3nack
    response3manouvre4read['MESSAGE3GET4UPU']     = message3readget4upu
    response3manouvre4read['MESSAGE3PUT4UPU']     = message3readput4upu
    response3manouvre4read['MESSAGE3RECEIVE4UPU'] = message3readreceive4upu
    response3manouvre4read['MESSAGE3SEND4UPU']    = message3readsend4upu
    response3manouvre4read['STATE3MANOEUVRE4UPU'] = state3read4upu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4read-returning-by-manoeuvre3readimage4upu: " + str(response3manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 
    # {'CODE3NACK4UPU': ' ',
    #  'STATE3MANOEUVRE4UPU': 1,
    #  'MESSAGE3PUT4UPU': '<default4xi>default3read4upu</default4xi>',
    #  'MESSAGE3SEND4UPU': '<roger4xi>roger3read4upu</roger4xi>',
    #  'MESSAGE3RECEIVE4UPU': '<copy4xi>copy3read4upu</copy4xi>',
    #  'ACK3INFO4UPU': False,
    #  'MESSAGE3GET4UPU': '<default4xi>default3read4upu</default4xi>',
    #  'CODE3ACK4UPU': ' 0D-1C-'}; time: 07:14:55 10/10/11 Pacific Daylight Time}
    
    #2 
    
    #3 
    
    #4 
    
    return response3manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3writeimage4upu" usage = "control function for write3image4upu function in this module">
# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3writeimage4upu(queue3writemanoeuvre4upu, consumerchannel3writemanoeuvre4upu, publisherchannel3writemanoeuvre4upu):
    '''
    manoeuvres control and command
    '''
    global state3write4upu                                                       # STATE FOR COMMAND AND CONTROL
    global max3writestate4upu                                                    # MAX FOR STATE
    global ack3writeinfo4upu                                                     # ACK FOR IMAGE INFO
    global message3writeget4upu                                 
    global message3writeput4upu
    global message3writereceive4upu
    global message3writesend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3writeimage4upu" 
    method2print = "manoeuvre3writeimage4upu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3writeimage4upu-started: " + str(manoeuvre3writeimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue3writemanoeuvre4upu-to-manoeuvre3writeimage4upu: " + str(queue3writemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3writemanoeuvre4upu-to-manoeuvre3writeimage4upu: " + str(consumerchannel3writemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3writemanoeuvre4upu-to-manoeuvre3writeimage4upu: " + str(publisherchannel3writemanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3write4upu-try-by-manoeuvre3writeimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3write4upu is None:
            state3write4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3write4upu-initialized-by-manoeuvre3writeimage4upu to: " + str(state3write4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3write4upu-used-by-manoeuvre3writeimage4upu: " + str(state3write4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state3write4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3write4upu-initialized-by-manoeuvre3writeimage4upu to: " + str(state3write4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3writeinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3writeinfo4upu-initialized-by-manoeuvre3writeimage4upu to: " + str(ack3writeinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3writeget4upu     = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3writeget4upu-initialized-by-manoeuvre3writeimage4upu to: " + str(message3writeget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3writeput4upu     = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3writeput4upu-initialized-by-manoeuvre3writeimage4upu to: " + str(message3writeput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writereceive4upu = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3writereceive4upu-initialized-by-manoeuvre3writeimage4upu to: " + str(message3writereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writesend4upu    = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3writesend4upu-initialized-by-manoeuvre3writeimage4upu to: " + str(message3writesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3writestate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3writestate4upu-initialized-by-manoeuvre3writeimage4upu to: " + str( max3writestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3write4upu = state3write4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3write4upu-added-by-manoeuvre3writeimage4upu: " + str(state3write4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3write4upu > max3writestate4upu:
        state3write4upu = 0
        message2print = "{(Step 0I/33) set" + "; state3write4upu-set-by-manoeuvre3writeimage4upu to: " + str(state3write4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3writeinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack3writeinfo4upu-set-by-manoeuvre3writeimage4upu to: " + str(ack3writeinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3writeget4upu     = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3writeget4upu-set-by-manoeuvre3writeimage4upu to: " + str(message3writeget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3writeput4upu     = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3writeput4upu-set-by-manoeuvre3writeimage4upu to: " + str(message3writeput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writereceive4upu = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3writereceive4upu-set-by-manoeuvre3writeimage4upu to: " + str(message3writereceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3writesend4upu    = '<default4xi>default3write4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3writesend4upu-set-by-manoeuvre3writeimage4upu to: " + str(message3writesend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3writestate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3writestate4upu-set-by-manoeuvre3writeimage4upu to: " + str( max3writestate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3write4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3writeimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3writereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3writereceive4upu = consumerchannel3writemanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message3writereceive4upu-received-by-manoeuvre3writeimage4upu: " + str(message3writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3writereceive4upu == '<copy4xi>copy3write4upu</copy4xi>':
                message3writesend4upu = '<roger4xi>roger3write4upu</roger4xi>'
            else:
                message3writesend4upu = '<negative4xi><negative3write4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3writemanoeuvre4upu.send(message3writesend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message3writesend4upu-sent-by-manoeuvre3writeimage4upu: " + str(message3writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3writereceive4upu-failed-by-manoeuvre3writeimage4upu: " + str(message3writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3write4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3writeimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3writereceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3writereceive4upu = consumerchannel3writemanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message3writereceive4upu-received-by-manoeuvre3writeimage4upu: " + str(message3writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3writereceive4upu == '<ready4xi>ready3write4upu</ready4xi>':
                message3writesend4upu = '<ready4xi>ready3write4upu</ready4xi>'
            else:
                message3writesend4upu = '<negative4xi><negative3ready4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3writemanoeuvre4upu.send(message3writesend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message3writesend4upu-sent-by-manoeuvre3writeimage4upu: " + str(message3writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3writereceive4upu-failed-by-manoeuvre3writeimage4upu: " + str(message3writereceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3write4upu == 3:    
        info3image4upu = None
        message2print = "{(Step 3A/70) try" + "; info3image4upu-try-by-manoeuvre3writeimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4upu = queue3writemanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4upu-got-by-manoeuvre3writeimage4upu: " + str(info3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4upu is None:
                code3nack  = code3nack + '3Z1-'
                ack3writeinfo4upu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3writeinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3writemanoeuvre4upu-failed-by-manoeuvre3writeimage4upu: " + str(queue3writemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3writesend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3writeinfo4upu is True:
                message3writesend4upu = '<ack4xi>ack3imageinfo4upu</ack4xi>'
            else:
                message3writesend4upu = '<nack4xi>nack3imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3writesend4upu-set-by-manoeuvre3writeimage4upu to: " + str(message3writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3writesend4upu-failed-by-manoeuvre3writeimage4upu: " + str(message3writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3writemanoeuvre4upu-calling-by-manoeuvre3writeimage4upu: " + str(publisherchannel3writemanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3writesend4upu-passing-to-send-by-manoeuvre3writeimage4upu: " + str(message3writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3writesend4upu-failed-by-manoeuvre3writeimage4upu: " + str(message3writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3writemanoeuvre4upu.send(message3writesend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message3writesend4upu-sent-by-manoeuvre3writeimage4upu: " + str(message3writesend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #         
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3writesend4upu-failed-by-manoeuvre3writeimage4upu: " + str(message3writesend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4write['ACK3INFO4UPU']        = ack3writeinfo4upu
    response3manouvre4write['CODE3ACK4UPU']        = code3ack
    response3manouvre4write['CODE3NACK4UPU']       = code3nack
    response3manouvre4write['MESSAGE3GET4UPU']     = message3writeget4upu
    response3manouvre4write['MESSAGE3PUT4UPU']     = message3writeput4upu
    response3manouvre4write['MESSAGE3RECEIVE4UPU'] = message3writereceive4upu
    response3manouvre4write['MESSAGE3SEND4UPU']    = message3writesend4upu
    response3manouvre4write['STATE3MANOEUVRE4UPU'] = state3write4upu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4write-returning-by-manoeuvre3writeimage4upu: " + str(response3manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # 
    
    #2
    # 
    
    #3
    # 
    

    return response3manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre3mapimage4upu" usage = "control function for map3image4upu function in this module">
# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre3mapimage4upu(queue3mapmanoeuvre4upu, consumerchannel3mapmanoeuvre4upu, publisherchannel3mapmanoeuvre4upu):
    '''
    manoeuvres control and command
    '''
    global state3map4upu                                                       # STATE FOR COMMAND AND CONTROL
    global max3mapstate4upu                                                    # MAX FOR STATE
    global ack3mapinfo4upu                                                     # ACK FOR IMAGE INFO
    global message3mapget4upu                                 
    global message3mapput4upu
    global message3mapreceive4upu
    global message3mapsend4upu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre3mapimage4upu" 
    method2print = "manoeuvre3mapimage4upu" 

    code3ack  = ' '
    code3nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre3mapimage4upu-started: " + str(manoeuvre3mapimage4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue3mapmanoeuvre4upu-to-manoeuvre3mapimage4upu: " + str(queue3mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel3mapmanoeuvre4upu-to-manoeuvre3mapimage4upu: " + str(consumerchannel3mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel3mapmanoeuvre4upu-to-manoeuvre3mapimage4upu: " + str(publisherchannel3mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code3ack  = code3ack + '0D-' 
    response3manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state3map4upu-try-by-manoeuvre3mapimage4upu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state3map4upu is None:
            state3map4upu = 0
            message2print = "{(Step 0F/33) initialized" + "; state3map4upu-initialized-by-manoeuvre3mapimage4upu to: " + str(state3map4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state3map4upu-used-by-manoeuvre3mapimage4upu: " + str(state3map4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
    except:
        state3map4upu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state3map4upu-initialized-by-manoeuvre3mapimage4upu to: " + str(state3map4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3mapinfo4upu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack3mapinfo4upu-initialized-by-manoeuvre3mapimage4upu to: " + str(ack3mapinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3mapget4upu     = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message3mapget4upu-initialized-by-manoeuvre3mapimage4upu to: " + str(message3mapget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3mapput4upu     = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message3mapput4upu-initialized-by-manoeuvre3mapimage4upu to: " + str(message3mapput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapreceive4upu = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message3mapreceive4upu-initialized-by-manoeuvre3mapimage4upu to: " + str(message3mapreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapsend4upu    = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message3mapsend4upu-initialized-by-manoeuvre3mapimage4upu to: " + str(message3mapsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3mapstate4upu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max3mapstate4upu-initialized-by-manoeuvre3mapimage4upu to: " + str( max3mapstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state3map4upu = state3map4upu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state3map4upu-added-by-manoeuvre3mapimage4upu: " + str(state3map4upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    if state3map4upu > max3mapstate4upu:
        state3map4upu = 0
        message2print = "{(Step 0I/33) set" + "; state3map4upu-set-by-manoeuvre3mapimage4upu to: " + str(state3map4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack3mapinfo4upu        = False
        message2print = "{(Step 0J/33) set" + "; ack3mapinfo4upu-set-by-manoeuvre3mapimage4upu to: " + str(ack3mapinfo4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message3mapget4upu     = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message3mapget4upu-set-by-manoeuvre3mapimage4upu to: " + str(message3mapget4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message3mapput4upu     = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message3mapput4upu-set-by-manoeuvre3mapimage4upu to: " + str(message3mapput4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapreceive4upu = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message3mapreceive4upu-set-by-manoeuvre3mapimage4upu to: " + str(message3mapreceive4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message3mapsend4upu    = '<default4xi>default3map4upu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message3mapsend4upu-set-by-manoeuvre3mapimage4upu to: " + str(message3mapsend4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max3mapstate4upu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max3mapstate4upu-set-by-manoeuvre3mapimage4upu to: " + str( max3mapstate4upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3map4upu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre3mapimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3mapreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3mapreceive4upu = consumerchannel3mapmanoeuvre4upu.recv()
            message2print = "{(Step 1B/33) received" + "; message3mapreceive4upu-received-by-manoeuvre3mapimage4upu: " + str(message3mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3mapreceive4upu == '<copy4xi>copy3mapimage4upu</copy4xi>':
                message3mapsend4upu = '<roger4xi>roger3mapimage4upu</roger4xi>'
            else:
                message3mapsend4upu = '<negative4xi><negative3mapimage4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3mapmanoeuvre4upu.send(message3mapsend4upu)
            
            message2print = "{(Step 1C/33) sent" + "; message3mapsend4upu-sent-by-manoeuvre3mapimage4upu: " + str(message3mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message3mapreceive4upu-failed-by-manoeuvre3mapimage4upu: " + str(message3mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state3map4upu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre3mapimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message3mapreceive4upu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message3mapreceive4upu = consumerchannel3mapmanoeuvre4upu.recv()
            message2print = "{(Step 2B/33) received" + "; message3mapreceive4upu-received-by-manoeuvre3mapimage4upu: " + str(message3mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message3mapreceive4upu == '<ready4xi>ready3mapimage4upu</ready4xi>':
                message3mapsend4upu = '<ready4xi>ready3mapimage4upu</ready4xi>'
            else:
                message3mapsend4upu = '<negative4xi><negative3ready4upu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel3mapmanoeuvre4upu.send(message3mapsend4upu)
            
            message2print = "{(Step 2C/33) sent" + "; message3mapsend4upu-sent-by-manoeuvre3mapimage4upu: " + str(message3mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code3ack  = code3ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message3mapreceive4upu-failed-by-manoeuvre3mapimage4upu: " + str(message3mapreceive4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state3map4upu == 3:    
        info3image4upu = None
        message2print = "{(Step 3A/70) try" + "; info3image4upu-try-by-manoeuvre3mapimage4upu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info3image4upu = queue3mapmanoeuvre4upu.get()
            
            message2print = "{(Step 3B/33) got" + "; info3image4upu-got-by-manoeuvre3mapimage4upu: " + str(info3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info3image4upu is None:
                code3nack  = code3nack + '3Z1-'
                ack3mapinfo4upu = False 
            else:    
                code3ack  = code3ack + '3C-'
                ack3mapinfo4upu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue3mapmanoeuvre4upu-failed-by-manoeuvre3mapimage4upu: " + str(queue3mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '3Z1-'
            
        message3mapsend4upu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack3mapinfo4upu is True:
                message3mapsend4upu = '<ack4xi>ack3imageinfo4upu</ack4xi>'
            else:
                message3mapsend4upu = '<nack4xi>nack3imageinfo4upu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message3mapsend4upu-set-by-manoeuvre3mapimage4upu to: " + str(message3mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3mapsend4upu-failed-by-manoeuvre3mapimage4upu: " + str(message3mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code3nack  = code3nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel3mapmanoeuvre4upu-calling-by-manoeuvre3mapimage4upu: " + str(publisherchannel3mapmanoeuvre4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message3mapsend4upu-passing-to-send-by-manoeuvre3mapimage4upu: " + str(message3mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message3mapsend4upu-failed-by-manoeuvre3mapimage4upu: " + str(message3mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z2-'                
        try:    
            publisherchannel3mapmanoeuvre4upu.send(message3mapsend4upu)
            
            message2print = "{(Step 4E/33) sent" + "; message3mapsend4upu-sent-by-manoeuvre3mapimage4upu: " + str(message3mapsend4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code3ack  = code3ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message3mapsend4upu-failed-by-manoeuvre3mapimage4upu: " + str(message3mapsend4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code3nack  = code3nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response3manouvre4map['ACK3INFO4UPU']        = ack3mapinfo4upu
    response3manouvre4map['CODE3ACK4UPU']        = code3ack
    response3manouvre4map['CODE3NACK4UPU']       = code3nack
    response3manouvre4map['MESSAGE3GET4UPU']     = message3mapget4upu
    response3manouvre4map['MESSAGE3PUT4UPU']     = message3mapput4upu
    response3manouvre4map['MESSAGE3RECEIVE4UPU'] = message3mapreceive4upu
    response3manouvre4map['MESSAGE3SEND4UPU']    = message3mapsend4upu
    response3manouvre4map['STATE3MANOEUVRE4UPU'] = state3map4upu
                          
    message2print = "{(Step 33/33) returning" + "; response3manouvre4map-returning-by-manoeuvre3mapimage4upu: " + str(response3manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 
    
    #2 
    
    #3 
    
    #4 
    
    return response3manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write3image4upu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4UPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4UPU ', time4xi())    
#   respon4write3image = {} 
#   task4write3image = None
#   try:                               
#       response4write3image = write3image4upu(task4write3image)                                                                                                          
#       print('(Step 16B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ3IMAGE4UPU : ', response4write3image)
#       queue3write4upu      = response4write3image['QUEUE']
#       channel3publish4upu = response4write3image['PUBLISHERCHANNEL']
#       channel3consume4upu = response4write3image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ3IMAGE4UPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE3IMAGE4UPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:                               
#       message3writesend4upu = "<copy4xi>copy3write4upu</copy4xi>"
#       channel3publish4upu = response4write3image['PUBLISHERCHANNEL']
#       channel3publish4upu.send(message3writesend4upu)
#       print("(Step 17B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3publish4upu, " MESSAGE3SEND4UPU: " , message3writesend4upu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE3IMAGE4UPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3write4upu, channel3consume4upu, channel3publish4upu)
#       print("(Step 18D/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3readimage4upu)
#   except:
#       print("(Step 18Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:
#       channel3consume4upu = response4write3image['CONSUMERCHANNEL']                              
#       message3writereceive4upu = channel3consume4upu.recv()
#       print("(Step 19B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3consume4upu, " CONTROL & COMMAND: ", message3writereceive4upu)                                                                                                  
#       # <roger4xi>roger3write4upu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4UPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE3IMAGE4UPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF UPU3IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:                               
#       message3writesend4upu = "<write4xi>write3write4upu</write4xi>"
#       channel3publish4upu = response4write3image['PUBLISHERCHANNEL']
#       channel3publish4upu.send(message3writesend4upu)
#       print("(Step 20B/700) OF UPU3IMAGE4XI5TEST - SENT TO CHANNEL3PUBLISH4UPU: ", channel3publish4upu, " MESSAGE3SEND4UPU: " , message3writesend4upu)                                                                                                
#       # <write4xi>write3write4upu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE3IMAGE4UPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3write4upu, channel3consume4upu, channel3publish4upu)
#       print("(Step 21B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3readimage4upu)
#   except:
#       print("(Step 21Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ3IMAGE4UPU ', time4xi())     
#   try:
#       channel3consume4upu = response4write3image['CONSUMERCHANNEL']                              
#       message3writereceive4upu = channel3consume4upu.recv()
#       print("(Step 22B/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3consume4upu, " CONTROL & COMMAND: ", message3writereceive4upu)                                                                                                  
#       # <write4xi>write3write4upu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ3IMAGE4UPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue3write4upu.put(path3upu)             
#       print("(Step 24B/700) OF UPU3IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path3upu) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE3IMAGE4UPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3write4upu, channel3consume4upu, channel3publish4upu)
#       print("(Step 25B/700) OF UPU3IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", manoeuvre3readimage4upu)
#   except:
#       print("(Step 25Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF UPU3IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message3writereceive4upu = channel3consume4upu.recv()
#       print("(Step 26D/700) OF UPU3IMAGE4XI5TEST - RECEIVED FROM CHANNEL3CONSUME4UPU: ", channel3consume4upu, " CONTROL & COMMAND: ", message3writereceive4upu)                                                                                                   
#       # <ack4xi>ack3imageinfo4upu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE3IMAGE4UPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF UPU3IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi()) 
#   try:        
#       response3manoeuvre4upu = manoeuvre3readimage4upu(queue3write4upu, channel3consume4upu, channel3publish4upu)
#       print("(Step 27B/700) OF UPU3IMAGE4XI5TEST - GOT RESPONSE3MANOEUVRE4UPU ", response3manoeuvre4upu)
#   except:
#       print("(Step 27Z/700) OF UPU3IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE3IMAGE4UPU ", time4xi())   

#">
# -------------------------------------------------------------------------------------------------   
def write3image4upu(task4imagewrite3upu=None):

    '''
    writes 3D images for UPU
    '''
    global upu3image4xi                                                     # 3D-IMAGES FOR UPU  
    global control3writeimage4upu                                              # COMMAND AND CONTROL FOR UPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write3image4upu"
    method2print = "write3image4upu"

    message2print = "{(Step 1A/33) started" + "; write3image4upu-started: " + str(write3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write3image4upu at 0x000000000321ACC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite3upu-to-write3image4upu: " + str(task4imagewrite3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 
            
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE UPU3IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                              # INITIALIZE IMAGE FOR UPU 
            message2print = "{(Step 1C/70) created" + "; upu3image4xi-created-by-write3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create3image4upu-skipped-by-write3image4upu: " + str(write3image4upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function write3image4upu at 0x000000000321ACC8>                            
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 1E/70) set" + "; upu3image4xi-set-by-write3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            upu3image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; upu3image4xi-yielded-by-write3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite3upu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite3upu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite3upu-created-by-write3image4upu to: " + str(task4imagewrite3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                                 # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function3callback4upu-created-by-write3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            
            code3ack  = code3ack + '1B1-'
           
            args3callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "write3image4upu"                                                            # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "write3image4upu"                                                            # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args3callback-created-by-write3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # <function trace3image4upu at 0x0000000003214848> 
            
            code3ack  = code3ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite3upu['DATA3CALLBACK'] = args3callback                                                    # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite3upu['FUNCTION3CALLBACK'] = function3callback4upu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite3upu-created-by-write3image4upu to: " + str(task4imagewrite3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write3image4upu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}',
            #  'CLASS': 'write3image4upu',
            #  'METHOD': 'write3image4upu'},
            #  'FUNCTION3CALLBACK': <function trace3image4upu at 0x0000000003214848>}; time: 07:14:55 10/10/11 Pacific Daylight Time}

            
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform3xi-created-by-write3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform3xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform3xi-got-by-write3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:14:55 10/10/11 Pacific Daylight Time}

                code3ack  = code3ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform3xi-failed-by-write3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '2Z-'
                 
            task4imagewrite3upu['PLATFORM'] = platform3xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 3A/70) trying" + "; context3upu-trying-by-write3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context3upu-got-by-write3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}', ...
                
                code3ack  = code3ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context3upu-failed-by-write3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '3Z-'
                
            task4imagewrite3upu ['CONTEXT'] = context3upu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device3upu-trying-by-write3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device3upu-got-by-write3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR' ...
                
                code3ack  = code3ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device3upu-failed-by-write3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '4Z-'
            
            task4imagewrite3upu ['DEVICE'] = device3upu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3upu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo3upu-trying-by-write3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo3upu-got-by-write3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #  {'SIZE2QUEUE': 2147483646, 
                #   'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #   'ORDER': 'CL_RGBA'}, 
                #   'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004406E48>, 
                #   'WIDTH': 1100, 
                #   'DEPTH': 900, 
                #   'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 07:14:55 10/10/11 Pacific Day ...
                
                code3ack  = code3ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo3upu-failed-by-write3image4upu: " + str(imageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '5Z-'          
                      
            task4imagewrite3upu ['IMAGE'] = imageinfo3upu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue3upu-trying-by-write3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue3upu-got-by-write3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004777978> 
                
                code3ack  = code3ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue3upu-failed-by-write3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '6Z-'
                
            task4imagewrite3upu ['QUEUE'] = queue3upu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe3upu-trying-by-write3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe3upu-got-by-write3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 636>, <read-write PipeConnection, handle 640>) 
                
                code3ack  = code3ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe3upu-failed-by-write3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '7Z-'
                
            task4imagewrite3upu ['PIPE'] = pipe3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo3upu-trying-by-write3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo3upu-got-by-write3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 07:14:55 10/10/11 Pa ...
                
                code3ack  = code3ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo3upu-failed-by-write3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '8Z-'
        
            task4imagewrite3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo3upu-trying-by-write3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                               # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo3upu-got-by-write3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 07:14:55 10/10/11 Pacific  ...
                
                code3ack  = code3ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo3upu-failed-by-write3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '9Z-'
        
            task4imagewrite3upu ['BUFFER'] = bufferinfo3upu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo3upu-trying-by-write3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo3upu-got-by-write3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 07:14:55 10/10/11 Pacific Daylight Time}', 'CLASS ...
                
                code3ack  = code3ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo3upu-failed-by-write3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '10Z-'
        
            task4imagewrite3upu ['EVENT'] = eventinfo3upu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist3upu-trying-by-write3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist3upu-set-by-write3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code3ack  = code3ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist3upu-failed-by-write3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '11Z-'
        
            task4imagewrite3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity3upu-trying-by-write3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity3upu-set-by-write3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code3ack  = code3ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-write3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '12Z-'
        
            task4imagewrite3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3upu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking3upu-trying-by-write3image4upu: " + str(blocking3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3upu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking3upu-set-by-write3image4upu to: " + str(blocking3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                
                code3ack  = code3ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking3upu-failed-by-write3image4upu: " + str(blocking3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '13Z-'
        
            task4imagewrite3upu ['BLOCKING'] = blocking3upu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3upu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin3upu-trying-by-write3image4upu: " + str(origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3upu['X'] = 0 
                origin3upu['Y'] = 0  
                origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin3upu-set-by-write3image4upu to: " + str(origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0}
                
                code3ack  = code3ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin3upu-failed-by-write3image4upu: " + str(origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '14Z-'
        
            task4imagewrite3upu ['ORIGIN'] = origin3upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region3upu-trying-by-write3image4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO READ 
                region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO READ 
                region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region3upu-set-by-write3image4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 330, 'X': 330, 'Z': 330}
                
                code3ack  = code3ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region3upu-failed-by-write3image4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '15Z-'
        
            task4imagewrite3upu ['REGION'] = region3upu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height3upu = 800
            width3upu = 800
            rowpitch3upu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch3upu-trying-by-write3image4upu: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch3upu-set-by-write3image4upu to: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                           
                 
                code3ack  = code3ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch3upu-failed-by-write3image4upu: " + str(rowpitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '16Z-'
        
            task4imagewrite3upu ['ROWPITCH'] = rowpitch3upu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3upu = None
            size3upu = int(width3upu * height3upu)                                                                  # CALCULATED MEMORY
            if size3upu == 0:
                size3upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer3upu-trying-by-write3image4upu: " + str(pointer3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3upu = Arena4Xi(size3upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena3upu-got-by-write3image4upu: " + str(arena3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041F4C50>
                
                size3arena = arena3upu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size3arena-got-by-write3buffer4upu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer3arena = arena3upu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer3arena-got-by-write3buffer4upu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000042554A8> 
                
                code3ack  = code3ack + '17D-' 
                
                if size3arena < size3upu:
                    size3upu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size3upu-set-by-write3buffer4upu to: " + str(size3upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '17E-'
                else:
                    pass
                    
                pointer3upu = buffer3arena
                message2print = "{(Step 17F/70) set" + "; pointer3upu-set-by-write3buffer4upu to: " + str(pointer3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000042554A8> 
                
                code3ack  = code3ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer3upu-failed-by-write3image4upu: " + str(pointer3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '17Z-'
        
            task4imagewrite3upu ['POINTER'] = pointer3upu                            
            task4imagewrite3upu ['SIZE']    = size3upu                                                               # AUTOMATIC BY UPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3upu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch3upu-trying-by-write3image4upu: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3upu = int(rowpitch3upu * height3upu)                                                       
                message2print = "{(Step 18B/70) set" + "; slicepitch3upu-set-by-write3image4upu to: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 640000 

                code3ack  = code3ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch3upu-failed-by-write3image4upu: " + str(slicepitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '18Z-'
            
            task4imagewrite3upu ['SLICEPITCH'] = slicepitch3upu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagewrite3upu.keys()
        message2print = "{(Step 20/70) using" + "; keys3task-using-by-write3image4upu: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:14:55 10/10/11 Pacific Daylight Time}

        if (('DATA3CALLBACK' in keys3task) and ('FUNCTION3CALLBACK' in keys3task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function3callback4upu = trace3image4upu                                                             # CALLBACK FUNCTION FOR UPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function3callback4upu-created-by-write3image4upu to: " + str(function3callback4upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code3ack  = code3ack + '21B1-'
           
            args3callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args3callback['CLASS']   = "write3image4upu"                                                        # MODULE OF UPUIZED OPENCL MODULE
            args3callback['METHOD']  = "write3image4upu"                                                        # FUNCTION OF UPUIZED OPENCL MODULE
            args3callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write3image4upu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR UPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args3callback-created-by-write3image4upu to: " + str(args3callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code3ack  = code3ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite3upu['DATA3CALLBACK'] = args3callback                                                # USER DATA PASSED TO UPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite3upu['FUNCTION3CALLBACK'] = function3callback4upu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code3ack  = code3ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite3upu-created-by-write3image4upu to: " + str(task4imagewrite3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys3task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform3xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform3xi-created-by-write3image4upu to: " + str(platform3xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform3xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform3xi-got-by-write3image4upu: " + str(platform3xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code3ack  = code3ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform3xi-failed-by-write3image4upu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code3nack  = code3nack + '22Z-'
                     
            task4imagewrite3upu['PLATFORM'] = platform3xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context3upu = {}
            message2print = "{(Step 23A/70) trying" + "; context3upu-trying-by-write3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context3upu = get2context4upu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context3upu-got-by-write3image4upu: " + str(context3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context3upu-failed-by-write3image4upu: " + str(context3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '23Z-'
                
            task4imagewrite3upu ['CONTEXT'] = context3upu           
        
        if 'DEVICE' in keys3task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device3upu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device3upu-trying-by-write3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device3upu = get2upu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device3upu-got-by-write3image4upu: " + str(device3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device3upu-failed-by-write3image4upu: " + str(device3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '24Z-'
            
            task4imagewrite3upu ['DEVICE'] = device3upu
        
        if 'IMAGE' in keys3task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo3upu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo3upu-trying-by-write3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo3upu = get3imageinfo4upu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo3upu-got-by-write3image4upu: " + str(imageinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code3ack  = code3ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo3upu-failed-by-write3image4upu: " + str(imageinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '25Z-'          
                      
            task4imagewrite3upu ['IMAGE'] = imageinfo3upu         
                    
        if 'QUEUE' in keys3task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY UPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue3upu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue3upu-trying-by-write3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue3upu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue3upu-got-by-write3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue3upu-failed-by-write3image4upu: " + str(queue3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '26Z-'
                
            task4imagewrite3upu ['QUEUE'] = queue3upu

        if 'PIPE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY UPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe3upu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe3upu-trying-by-write3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe3upu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe3upu-got-by-write3image4upu: " + str(pipe3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code3ack  = code3ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe3upu-failed-by-write3image4upu: " + str(pipe3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '27Z-'
                
            task4imagewrite3upu ['PIPE'] = pipe3upu 
            
        if 'COMMANDQUEUE' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo3upu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo3upu-trying-by-write3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo3upu = get2commandqueueinfo4upu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo3upu-got-by-write3image4upu: " + str(commandqueueinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo3upu-failed-by-write3image4upu: " + str(commandqueueinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '28Z-'
        
            task4imagewrite3upu ['COMMANDQUEUE'] = commandqueueinfo3upu
            
            
        if 'BUFFER' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY UPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo3upu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo3upu-trying-by-write3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo3upu = get2bufferinfo4upu()                                                           # SYSTEM CREATES BUFFER FOR UPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo3upu-got-by-write3image4upu: " + str(bufferinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo3upu-failed-by-write3image4upu: " + str(bufferinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '29Z-'
        
            task4imagewrite3upu ['BUFFER'] = bufferinfo3upu
             
        if 'EVENT' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo3upu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo3upu-trying-by-write3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo3upu = get2eventinfo4upu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo3upu-got-by-write3image4upu: " + str(eventinfo3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo3upu-failed-by-write3image4upu: " + str(eventinfo3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '30Z-'
        
            task4imagewrite3upu ['EVENT'] = eventinfo3upu 
        
             
        if 'EVENTWAITLIST' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist3upu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist3upu-trying-by-write3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist3upu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist3upu-set-by-write3image4upu: " + str(eventwaitlist3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist3upu-failed-by-write3image4upu: " + str(eventwaitlist3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '31Z-'
        
            task4imagewrite3upu ['EVENTWAITLIST'] = eventwaitlist3upu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity3upu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity3upu-trying-by-write3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys3event = eventwaitlist3upu.keys()
                for key3event in keys3event:
                    eventwaitlistquantity3upu = eventwaitlistquantity3upu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity3upu-set-by-write3image4upu: " + str(eventwaitlistquantity3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code3ack  = code3ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity3upu-failed-by-write3image4upu: " + str(eventwaitlistquantity3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '32Z-'
        
            task4imagewrite3upu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity3upu       
                                                 
                 
        if 'BLOCKING' in keys3task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking3upu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking3upu-trying-by-write3image4upu: " + str(blocking3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking3upu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking3upu-set-by-write3image4upu to: " + str(blocking3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking3upu-failed-by-write3image4upu: " + str(blocking3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '33Z-'
        
            task4imagewrite3upu ['BLOCKING'] = blocking3upu             
                 
        if 'ORIGIN' in keys3task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin3upu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin3upu-trying-by-write3image4upu: " + str(origin3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin3upu['X'] = 0 
                origin3upu['Y'] = 0  
                origin3upu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin3upu-set-by-write3image4upu to: " + str(origin3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin3upu-failed-by-write3image4upu: " + str(origin3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '34Z-'
        
            task4imagewrite3upu ['ORIGIN'] = origin3upu
                
                 
        if 'REGION' in keys3task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region3upu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region3upu-trying-by-write3image4upu: " + str(region3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region3upu['X'] = 330                                                                               # WIDTH  IN PIXELS TO WRITE 
                region3upu['Y'] = 330                                                                                # HEIGHT IN PIXELS TO WRITE
                region3upu['Z'] = 330                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region3upu-set-by-write3image4upu to: " + str(region3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region3upu-failed-by-write3image4upu: " + str(region3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '35Z-'
        
            task4imagewrite3upu ['REGION'] = region3upu
            
                
                 
        if 'ROWPITCH' in keys3task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch3upu = 0
            size3element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity3layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 3D LAYER 
            quantity3direction = 1                                                                                  # ONE DIRECTIONS IN 3D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch3upu-trying-by-write3image4upu: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch3upu = int(width3upu * size3element * voxelquantity3layer / quantity3direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch3upu-set-by-write3image4upu to: " + str(rowpitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code3ack  = code3ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch3upu-failed-by-write3image4upu: " + str(rowpitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '36Z-'
        
            task4imagewrite3upu ['ROWPITCH'] = rowpitch3upu     
                 
        if 'POINTER' in keys3task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer3upu = None
            size3upu = int(width3upu * height3upu)                                                                  # CALCULATED MEMORY
            if size3upu == 0:
                size3upu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer3upu-trying-by-write3image4upu: " + str(pointer3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena3upu = Arena4Xi(size3upu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena3upu-got-by-write3image4upu: " + str(arena3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size3arena = arena3upu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size3arena-got-by-write3buffer4upu: " + str(size3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer3arena = arena3upu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer3arena-got-by-write3buffer4upu: " + str(buffer3arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code3ack  = code3ack + '37D-' 
                
                if size3arena < size3upu:
                    size3upu = size3arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size3upu-set-by-write3buffer4upu to: " + str(size3upu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code3ack  = code3ack + '37E-'
                else:
                    pass
                    
                pointer3upu = buffer3arena
                message2print = "{(Step 37F/70) set" + "; pointer3upu-set-by-write3buffer4upu to: " + str(pointer3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code3ack  = code3ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer3upu-failed-by-write3image4upu: " + str(pointer3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code3nack  = code3nack + '37Z-'
        
            task4imagewrite3upu ['POINTER'] = pointer3upu                            
            task4imagewrite3upu ['SIZE']    = size3upu                                                               # AUTOMATIC BY UPUXI
        
             
        if 'SLICEPITCH' in keys3task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 3D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch3upu = 0                                                                                   # SLICEPITCH IS ZERO FOR 3D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch3upu-trying-by-write3image4upu: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch3upu = int(rowpitch3upu * height3upu)                                                      
                message2print = "{(Step 38B/70) set" + "; slicepitch3upu-set-by-write3image4upu to: " + str(slicepitch3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code3ack  = code3ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch3upu-failed-by-write3image4upu: " + str(slicepitch3upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code3nack  = code3nack + '38Z-'
            
            task4imagewrite3upu ['SLICEPITCH'] = slicepitch3upu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys3task = task4imagewrite3upu.keys()
        message2print = "{(Step 39/70) set" + "; keys3task-set-by-write3image4upu to: " + str(sorted(keys3task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA3CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION3CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 07:14:55 10/10/11 Pacific Daylight Time}

        
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite3upu-yielded-by-write3image4upu: " + str(task4imagewrite3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code3nack  = code3nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control3writeimage4upu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels3upu = False
    channels3upu = ()
    message2print = "{(Step 40A/70) trying" + "channels3upu-trying-by-write3image4upu: " + str(channels3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel3upu, publisherchannel3upu = task4imagewrite3upu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel3upu-got-by-write3image4upu: " + str(consumerchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 636> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel3upu-got-by-write3image4upu: " + str(publisherchannel3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 640>
                     
        ack4channels3upu = True
        
        control3writeimage4upu['CONSUMERCHANNEL']  = consumerchannel3upu
        
        
        control3writeimage4upu['PUBLISHERCHANNEL'] = publisherchannel3upu
        
        code3ack  = code3ack + '40D-' 
        
    except:
        ack4channels3upu = False
        code3nack  = code3nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue3upu = False
    queue3upu = None
    message2print = "{(Step 41A/70) trying" + "queue3upu-trying-by-write3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue3upu = task4imagewrite3upu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue3upu-got-by-write3image4upu: " + str(queue3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004777978> 
        
              
        ack4queue3upu = True
        control3writeimage4upu['QUEUE']  = queue3upu        
        code3ack  = code3ack + '41C-'
    except:
        ack4queue3upu = False  
        code3nack  = code3nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy3image4upu = None
    # message2print = "{(Step 69A/70) trying" + "dummy3image4upu-trying-by-write3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy3image4upu = DummyProcess4Xi(target=manoeuvre3readimage4upu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue3upu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel3upu, publisherchannel3upu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy3image4upu-created-by-write3image4upu: " + str(dummy3image4upu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code3ack  = code3ack + '69B-'
        
    # except:
        # code3nack  = code3nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code3ack-set-by-write3image4upu: " + str(code3ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code3nack-set-by-write3image4upu: " + str(code3nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control3writeimage4upu-returning-by-write3image4upu: " + str(control3writeimage4upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004777978>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 640>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 636>}; time: 07:14:55 10/10/11 Pacific Daylight Time}

    return control3writeimage4upu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release3image4upu" usage = "
#
#   task4release3image  = {} 
#   task4release3image  ['COUNT3REFERENCE'] = 1 
#   response4release3image = {}
#   
#   print('(Step 12A/33) OF UPU3IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF UPU DEVICE ', time4xi())   
#   try:                               
#       response4release3image = release3image4upu(task4release3image)                                                                                             
#       print('(Step 12B/33) OF UPU3IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release3image)                           
#   except:
#       print('(Step 12Z1/33) OF UPU3IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF UPU DEVICE ', time4xi())  
#   
#                 
#">
# -------------------------------------------------------------------------------------------------   
def release3image4upu(task4release2upu=None):

    '''
    releases upu3image4xi
    '''
    global upu3image4xi  
    global code3ack
    global code3nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release3image4upu"
    method2print = "release3image4upu" 

    message2print = "{(Step 1A/33) started" + "; release3image4upu-started: " + str(release3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release3image4upu at 0x00000000031B7D48> 
    
    message2print = "{(Step 1B/33) passed" + "; task4release2upu-to-release3image4upu: " + str(task4release2upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT3REFERENCE': 1}

             
    code3ack  = ' '
    code3nack = ' '

    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            upu3image4xi = create3image4upu()
        except:
            upu3image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; upu3image4xi-yielded-by-release3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release3image4upu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2upu is None:
        task4release2upu = {}
        task4release2upu['COUNT3REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        upu3image4xi['COUNT3REFERENCE'] = upu3image4xi['COUNT3REFERENCE'] - task4release2upu['COUNT3REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release3image4upu: " + str(upu3image4xi['COUNT3REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if upu3image4xi['COUNT3REFERENCE'] < 0:
            upu3image4xi['COUNT3REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release3image4upu: " + " ** " + str(upu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # 
              
            code3nack  = code3nack + '1B-'
        else:
            code3ack  = code3ack + '1B-'
    except:
        try:
            upu3image4xi['COUNT3REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release3image4upu: " + " ** " + str(upu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # warning; count2reference-warning-by-release3image4upu:  ** 0 **
            
            code3nack  = code3nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; upu3image4xi-yielded-by-release3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    upu3image4xi ['ACK2RETURN']  = code3ack
    upu3image4xi ['NACK2RETURN'] = code3nack  
    
    message2print = "{(Step 70/70) returning" + "; upu3image4xi-returning-by-release3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # ACK2RETURN  :   
    # BUFFER  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4upu-failed ; time: 06:24:23 10/10/ ...
    # CODE3ERROR  :  0
    # COMMANDQUEUE  :  {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 06: ...
    # CONTEXT  :  {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 06:24:23 10/10/11 Pacific Dayl ...
    # COUNT3REFERENCE  :  0
    # DATA3CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create3image4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4upu', 'METHOD': 'create3image4upu'}
    # DEPTH  :  900
    # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHL ...
    # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time ...
    # FLAGS  :  0
    # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # FUNCTION3CALLBACK  :  <function trace3image4upu at 0x00000000031B1848>
    # HEIGHT  :  1000
    # NACK2RETURN  :   8Z1-
    # PIPE  :  (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>)
    # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
    # POINTER  :  <mmap.mmap object at 0x00000000041E03F8>
    # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000041E5EB8>
    # ROWPITCH  :  901
    # SLICEPITCH  :  902
    # WIDTH  :  1100
    
    return upu3image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain3image4upu" usage = "
#
#   task4retain3image  = {}
#   task4retain3image  ['DIMENSION'] = 3 
#   task4retain3image  ['COUNT3REFERENCE'] = 1 
#   response4retain3Dimage = {}
#   print('(Step 13C/700) OF UPU3IMAGE4XI5TEST - TRYING TO RETAIN 3D IMAGE OF UPU DEVICE ', time4xi())  
#   try:                               
#       response4retain3Dimage = retain3image4upu(task4retain3image)                                                                                                 
#       print('(Step 13D/700) OF UPU3IMAGE4XI5TEST - RESPONSE4RETAIN3DIMAGE : ', response4retain3Dimage)                          
#       # 
#   except:
#       print('(Step 13Z2/700) OF UPU3IMAGE4XI5TEST *** FAILED TO RETAIN 3D IMAGE OF UPU DEVICE ', time4xi())      
#
#">
# -------------------------------------------------------------------------------------------------   
def retain3image4upu(task4retain3upu=None):

    '''
    retains upu3image4xi
    '''
    global upu3image4xi 
     
    global code3ack
    global code3nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain3image4upu"
    method2print = "retain3image4upu"

    message2print = "{(Step 1A/33) started" + "; retain3image4upu-started: " + str(retain3image4upu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain3image4upu at 0x00000000031B7DC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4retain3upu-to-retain3image4upu: " + str(task4retain3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT3REFERENCE': 1, 'DIMENSION': 3} 
    
    code3ack  = ' '
    code3nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain3upu is None:
        task4retain3upu = {}
        task4retain3upu['DIMENSION'] = 3
        task4retain3upu['COUNT3REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain3upu-set-by-retain3image4upu to: " + str(task4retain3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain3upu.keys():
        if task4retain3upu['DIMENSION'] == 3:
            pass
        else:
            task4retain3upu['DIMENSION'] = 3 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain3image4upu to: " + str(task4retain3upu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain3upu['DIMENSION'] = 3
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT3REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT3REFERENCE' in task4retain3upu.keys():
        pass
    else:
        task4retain3upu['COUNT3REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain3image4upu to: " + str(task4retain3upu['COUNT3REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain3upu['COUNT3REFERENCE'] < 1:
        task4retain3upu['COUNT3REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain3image4upu to: " + str(task4retain3upu['COUNT3REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 3D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if upu3image4xi is None:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 4A/70) created" + "; upu3image4xi-created-by-retain3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            upu3image4xi = create3image4upu()
            message2print = "{(Step 4B/70) created" + "; upu3image4xi-created-by-retain3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            upu3image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; upu3image4xi-yielded-by-retain3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        upu3image4xi['COUNT3REFERENCE'] = upu3image4xi['COUNT3REFERENCE'] + task4retain3upu['COUNT3REFERENCE']
        message2print = "{(Step 6B/70) calculated" + "; count2reference-calculated-by-retain3image4upu: " + str(upu3image4xi['COUNT3REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 1 

        if upu3image4xi['COUNT3REFERENCE'] <= 0:
            upu3image4xi['COUNT3REFERENCE'] = 1
            message2print = "{(Step 6Z2/70) warning" + "; count2reference-warning-by-retain3image4upu: " + " ** " + str(upu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #   
            code3nack  = code3nack + '6Z2-'
        else:
            code3ack  = code3ack + '6D-'
                                
    except:
        try:
            upu3image4xi['COUNT3REFERENCE'] = 1
        
            message2print = "{(Step 7Z3/70) warning" + "; count2reference-warning-by-retain3image4upu: " + " ** " + str(upu3image4xi['COUNT3REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code3nack  = code3nack + '7Z3-'
        
        except:
            message2print = "{(Step 7Z4/70) yielded" + "; upu3image4xi-yielded-by-retain3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code3nack  = code3nack + '7Z4-'                
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    upu3image4xi ['ACK2RETURN']  = code3ack
    upu3image4xi ['NACK2RETURN'] = code3nack     
    message2print = "{(Step 70B70) returning" + "; upu3image4xi-returning-by-retain3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}, 'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT' ...
    #  'COUNT3REFERENCE': 1, 'EVENT': {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4upu-failed ; time: 06:24:23 10/10/11 ...
    
    return upu3image4xi         
    

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get3imageinfo4upu" usage = "
#
#   task4getinfo3image = {}
#   task4getinfo3image ['DIMENSION'] = 3
#   task4getinfo3image ['WIDTH'] = 0
#   task4getinfo3image ['HEIGHT'] = 0
#   task4getinfo3image ['DEPTH'] = 0
#   response4getinfo3image = {}
#   print('(Step 12C/33) OF UPU3IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF UPU DEVICE ', time4xi())    
#   try:                               
#       response4getinfo3image = get3imageinfo4upu (task4getinfo3image)                                                                                             
#       print('(Step 13D/33) OF UPU3IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo3image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF UPU3IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF UPU DEVICE ', time4xi()) 
#            
#">
# -------------------------------------------------------------------------------------------------   
def get3imageinfo4upu(task4getinfo3image=None):

    '''
    gets upu3image4xi
    '''
    
    global upu3image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get3imageinfo4upu"
    method2print = "get3imageinfo4upu"

    message2print = "{(Step 0A/33) started" + "; get3imageinfo4upu-started: " + str(get3imageinfo4upu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get3imageinfo4upu at 0x000000000318FE48>                                                      
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo3image-passed-to-get3imageinfo4upu: " + str(task4getinfo3image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    
    image3upu = {}
    try: 
        if upu3image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 3D IMAGE FOR UPU           
            # -------------------------------------------------------------------------------------           
            image3upu = create3image4upu()
            message2print = "{(Step 1B3/33) initialized" + "; image3upu-initialized-by-get3imageinfo4upu to: " + str(image3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                           
        else:
            message2print = "{(Step 1B4/33) using" + "; upu3image4xi-using-by-get3imageinfo4upu: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'PIPE': (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>), 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-1 ...
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF UPU          
        # -------------------------------------------------------------------------------------     
        dimension3xi = 0
        
        if task4getinfo3image is None:
            task4getinfo3image = {}
            task4getinfo3image ['DIMENSION'] = 3 
            task4getinfo3image ['DOMAIN'] = 'ALL'
            image3upu = upu3image4xi
            message2print = "{(Step 1C/33) using" + "; image3upu-got-by-get3imageinfo4upu: " + str(image3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
        else:
            pass    

        keys3task4imageinfo = task4getinfo3image.keys()
        if 'DIMENSION' in keys3task4imageinfo:
            dimension3xi = task4getinfo3image['DIMENSION']
            if dimension3xi == 3:
                pass
            else:
                dimension3xi = 3    
        else:
            pass
            
        if 'DOMAIN' in keys3task4imageinfo:
            domain3xi = task4getinfo3image['DOMAIN']
            if domain3xi == 'ALL':
                pass
            else:
                domain3xi = 'ALL'

            keys3domain = upu3image4xi.keys()
            for key3domain in keys3domain:
                task4getinfo3image[key3domain] = upu3image4xi[key3domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo3image-using-by-get3imageinfo4upu: " + str(task4getinfo3image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
        keys3task = task4getinfo3image.keys()
        for key3task in keys3task:
            try:
                message2print = "{(Step 3D2/33) using" + "; key3task-using-by-get2image4upu: " + str(key3task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DEPTH; DIMENSION; HEIGHT 
                image3upu [key3task] = upu3image4xi[key3task]
                
                message2print = "{(Step 3D2/33) set" + "; info-of-image3upu-set-by-get3imageinfo4upu to: " + str(image3upu [key3task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800; 800 
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key3task-yielded-by-get2image4upu: " + str(key3task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION - OK 
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo3image-using-by-get2image4upu: " + str(task4getinfo3image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; upu3image4xi-using-by-get2image4upu: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image3upu = create3image4upu()
            message2print = "{(Step 1F/33) initialized" + "; image3upu-initialized-by-get3imageinfo4upu to: " + str(image3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo3image is None:
                image3upu = upu3image4xi
                message2print = "{(Step 1G/33) got" + "; image3upu-got-by-get3imageinfo4upu: " + str(image3upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys3task = task4getinfo3image.keys()
                for key3task in keys3task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key3task-using-by-get2image4upu: " + str(key3task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image3upu [key3task] = upu3image4xi[key3task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image3upu-set-by-get3imageinfo4upu to: " + str(image3upu [key3task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key3task-yielded-by-get2image4upu: " + str(key3task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image3upu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image3upu-yielded-by-get2image4upu: " + str(image3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image3upu-returning-by-get3imageinfo4upu: " + str(image3upu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    #SOME: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    #ALL:  {'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time ...
    
    return image3upu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get3supportedimageformats4upu" usage = "
#
#    print('(Step 3A/33) OF UPU3IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR UPU', time4xi())  
#    try:
#        imageformats = get3supportedimageformats4upu()                                                                       
#        print('(Step 3B/33) OF UPU3IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR UPU: ', imageformats) 
#    except:
#        print('(Step 3Z/33) OF UPU3IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR UPU', time4xi())
#            
#">
# -------------------------------------------------------------------------------------------------   
def get3supportedimageformats4upu(task4supportedimageformats3upu=None):  

    '''
    gets supported image formats for upu
    '''
    
    global upu3image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get3supportedimageformats4upu"
    method2print = "get3supportedimageformats4upu"

    message2print = "{(Step 0A/33) started" + "; get3supportedimageformats4upu-started: " + str(get3supportedimageformats4upu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get3supportedimageformats4upu at 0x000000000318FEC8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats3upu-passed-to-get3supportedimageformats4upu: " + str(task4supportedimageformats3upu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'DIMENSION': 3} 
    
    
    if task4supportedimageformats3upu is None:
        task4supportedimageformats3upu = {} 
        task4supportedimageformats3upu['DIMENSION'] = 3                                                             # 3D image is the default
    
    dimension3xi = 3
    try:
        keys2supportedimageformats = task4supportedimageformats3upu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension3xi = task4supportedimageformats3upu['DIMENSION']
            if ((dimension3xi <=0) or (dimension3xi > 3)): 
                dimension3xi = 3
        else:
            dimension3xi = 3
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension3xi-using-by-dimension3xi: " + str(dimension3xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 
                            
    supportedimageformats3upu = {} 
    try: 
        if upu3image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image3upu = create3image4upu()                                                                          # create upu3image4xi
            message2print = "{(Step 1C/33) initialized" + "; image3upu-initialized-by-get3supportedimageformats4upu to: " + str(image3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass
                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR UPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats3upu = upu3image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats3upu-got-by-get3supportedimageformats4upu: " + str(supportedimageformats3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}; time: 05:18:11 10/10/11 Pacific Daylight Time}
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image3upu = create3image4upu()
            message2print = "{(Step 1F2/33) initialized" + "; image3upu-initialized-by-get3supportedimageformats4upu to: " + str(image3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR UPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats3upu = upu3image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats3upu-got-by-get3supportedimageformats4upu: " + str(supportedimageformats3upu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                          
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats3upu-yielded-by-get2image4upu: " + str(supportedimageformats3upu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats3upu-returning-by-get3supportedimageformats4upu: " + str(supportedimageformats3upu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}; time: 05:18:11 10/10/11 Pacific Daylight Time}
    
    return supportedimageformats3upu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait3image4upu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF UPU3IMAGE4XI5TEST - TRYING TO WAIT 3D IMAGE FOR UPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2upu = wait3image4upu()
#       keys3response = response4wait2upu.keys()
#       print('(Step 16B/700) OF UPU3IMAGE4XI5TEST - WAITED 3D IMAGE FOR UPU ', time4xi()) 
#       for key3response in sorted(keys3response):
#           print(key3response, ' : ', response4wait2upu[key3response])
#           # 
#   except:
#       print('(Step 16Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO WAIT 3D IMAGE FOR UPU: ', wait2upu)            
#">
# -------------------------------------------------------------------------------------------------   
def wait3image4upu(task4wait3image=None):

    '''
    waits upu3image4xi
    '''
    
    global upu3image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait3image4upu"
    method2print = "wait3image4upu"

    message2print = "{(Step 1A1/33) started" + "; wait3image4upu-started: " + str(wait3image4upu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function wait3image4upu at 0x00000000031B7F48>  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait3image-passed-by-wait3image4upu to: " + str(task4wait3image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # None 
    
    try: 
        if upu3image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE     
            # -------------------------------------------------------------------------------------           
            upu3image4xi = create3image4upu()
            
            message2print = "{(Step 1B/33) created" + "; upu3image4xi-created-by-wait3image4upu: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue3image4upu = None
            try:
                queue3image4upu = upu3image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue3image4upu-got-by-wait3image4upu to: " + str(queue3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; upu2xi-yielded-by-wait3image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size3image4upu = 0
            try:                    
                size3image4upu = queue3image4upu.qsize()
                message2print = "{(Step 1D/33) got" + "; size3image4upu-got-by-wait3image4upu to: " + str(size3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; upu2xi-yielded-by-wait3image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty3image4upu = True       
            try:                    
                empty3image4upu= queue3image4upu.empty()
                message2print = "{(Step 1E/33) got" + "; empty3image4upu-got-by-wait3image4upu to: " + str(empty3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty3image4upu-yielded-by-wait3image4upu: " + str(empty3image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full3image4upu = False       
            try:                                
                full3image4upu = queue3image4upu.full()
                message2print = "{(Step 1F/33) got" + "; full3image4upu-got-by-wait3image4upu to: " + str(full3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full3image4upu-yielded-by-wait3image4upu: " + str(full3image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; upu3image4xi-finished-by-wait3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR UPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; upu3image4xi-used-by-wait3image4upu: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{( ...
            
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue3image4upu = None
            try:
                queue3image4upu = upu3image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue3image4upu-got-by-wait3image4upu to: " + str(queue3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <queue4xi.Queue4Xi object at 0x00000000041E5EB8>
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; upu2xi-yielded-by-wait3image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait3image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait3image-passed-by-wait3image4upu to: " + str(task4wait3image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue3task = {}    
                try:
                    queue3task = task4wait3image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue3task-set-by-wait3image4upu to: " + str(queue3task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue3task-yielded-by-wait3image4upu: " + str(queue3task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue3task == queue3image4upu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue3task-accepted-by-wait3image4upu: " + str(queue3task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue3task-warned-by-wait3image4upu: " + " ** " + str(queue3task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue3image4upu-warned-by-wait3image4upu: " + " ** " + str(queue3image4upu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue3task-yielded-by-wait3image4upu: " + str(queue3task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size3queue4upu = 0        
            try:                    
                size3queue4upu = queue3image4upu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size3queue4upu-got-by-wait3image4upu to: " + str(size3queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 2147483646 
                
                upu3image4xi ['SIZE2QUEUE'] = size3queue4upu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait3image4upu to: " + str(upu3image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 2147483646
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size3queue4upu-yielded-by-wait3image4upu: " + str(size3queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty3queue4upu = False
            try:                    
                empty3queue4upu = queue3image4upu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty3queue4upu-got-by-wait3image4upu to: " + str(empty3queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # True
                
                upu3image4xi ['EMPTY2QUEUE'] = empty3queue4upu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait3image4upu to: " + str(upu3image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # True               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty3queue4upu-yielded-by-wait3image4upu: " + str(empty3queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full3queue4upu = False
            try:                                
                full3queue4upu = queue3image4upu.full()
                message2print = "{(Step 1P1/33) got" + "; full3queue4upu-got-by-wait3image4upu to: " + str(full3queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # False
                 
                upu3image4xi ['FULL2QUEUE'] = full3queue4upu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait3image4upu to: " + str(upu3image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                # False                         
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full3queue4upu-yielded-by-wait3image4upu: " + str(full3queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state3queue4upu = None
            try:                                
                state3queue4upu = queue3image4upu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state3queue4upu-got-by-wait3image4upu to: " + str(state3queue4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #  (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200); time: 06:24:23 10/10/11 Pacific Daylight Time}
                 
                upu3image4xi ['STATE2QUEUE'] = state3queue4upu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait3image4upu to: " + str(upu3image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200); time: 06:24:23 10/10/11 Pacific Daylight Time}
   
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state3queue4upu-yielded-by-wait3image4upu: " + str(state3queue4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; upu3image4xi-finished-by-wait3image4upu: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'SIZE2QUEUE': 2147483646, 'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}, 'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4upu', 'METHOD': 'create2context4upu'}, 'FUNCTION2CALLBACK': <function trace2context4upu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4upu', 'METHOD': 'create2commandqueue4upu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004387E10>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)}, 'STATE2QUEUE': (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200), 'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'EMPTY2QUEUE': True, 'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4apu', 'METHOD': 'create2buffer4apu'}, 'FUNCTION2CALLBACK': <function trace2buffer4apu at 0x0000000004AFFE48>, 'POINTER2BUFFER': <mmap.mmap object at 0x00000000041E0500>, 'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'NACK2RETURN': ' ', 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6A-7A-8A-8E-9A-10A-11B-12B-13B-14B-15B-16B-17B-18B-19B-', 'SIZE': 278528}, 'ACK2RETURN': ' 6D-', 'HEIGHT': 1000, 'FULL2QUEUE': False, 'FLAGS': 0, 'ROWPITCH': 901, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'CODE3ERROR': 0, 'POINTER': <mmap.mmap object at 0x00000000041E03F8>, 'CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'NACK2RETURN': ' ', 'FUNCTION3CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'SLICEPITCH': 902, 'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4apu', 'METHOD': 'create3image4apu'}, 'COUNT3REFERENCE': 1, 'EVENT': {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2event4apu', 'METHOD': 'create2event4apu'}, 'FUNCTION2CALLBACK': <function trace2event4apu at 0x000000000315DCC8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-', 'QUEUE': <queue4xi.Queue4Xi object at 0x000000000465C630>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4apu', 'METHOD': 'create2commandqueue4apu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4apu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C8A0B8>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 488>, <read-write PipeConnection, handle 492>)}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PIPE': (<read-write PipeConnection, handle 456>, <read-write PipeConnection, handle 460>)}}; time: 06:24:23 10/10/11 Pacific Daylight Time}

    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            upu3image4upu = create3image4upu()
            message2print = "{(Step 1R/33) got" + "; upu3image4xi-got-by-wait3image4upu to: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue3image4upu = None
            try:
                queue3image4upu = upu3image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue3image4upu-got-by-wait3image4upu to: " + str(queue3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue3image4upu-yielded-by-wait3image4upu: " + str(queue3image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait3image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue3task = None    
                try:
                    queue3task = task4wait3image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue3task-passed-by-wait3image4upu to: " + str(queue3task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue3task == queue3image4upu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue3task-accepted-by-wait3image4upu: " + str(queue3task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue3task-warned-by-wait3image4upu: " + " ** " + str(queue3task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue3image4upu-warned-by-wait3image4upu: " + " ** " + str(queue3image4upu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue3task-yielded-by-wait3image4upu: " + str(queue3task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size3image4upu = 0                             
            try:                    
                size3image4upu = queue3image4upu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size3image4upu-got-by-wait3image4upu to: " + str(size3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size3image4upu-yielded-by-wait3image4upu: " + str(size3image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty3image4upu = True          
            try:                    
                empty3image4upu = queue3image4upu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty3image4upu-got-by-wait3image4upu to: " + str(empty3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; upu2xi-yielded-by-wait3image4upu: " + str(upu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full3image4upu = False          
            try:                                
                full3image4upu= queue3image4upu.full()
                message2print = "{(Step 1Y4/33) got" + "; full3image4upu-got-by-wait3image4upu to: " + str(full3image4upu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full3image4upu-yielded-by-wait3image4upu: " + str(full3image4upu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; upu3image4xi-finished-by-wait3image4upu: " + str(upu3image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            upu3image4upu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; upu3image4xi-yielded-by-wait3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; upu3image4xi-returning-by-wait3image4upu: " + str(upu3image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}, 'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041E5EB8>, 'WIDTH': 1100, 'DEPTH': 900, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4upu', 'METHOD': 'create2context4upu'}, 'FUNCTION2CALLBACK': <function trace2context4upu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4upu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4upu', 'METHOD': 'create2commandqueue4upu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4upu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004387E10>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)}, 'STATE2QUEUE': (2147483647, <read-only PipeConnection, handle 500>, <write-only PipeConnection, handle 504>, <Lock(owner=None)>, None, <BoundedSemaphore4Xi(value=1, maxvalue=2147483647)>, 2200), 'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'EMPTY2QUEUE': True, 'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4apu', 'METHOD': 'create2buffer4apu'}, 'FUNCTION2CALLBACK': <function trace2buffer4apu at 0x0000000004AFFE48>, 'POINTER2BUFFER': <mmap.mmap object at 0x00000000041E0500>, 'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'NACK2RETURN': ' ', 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6A-7A-8A-8E-9A-10A-11B-12B-13B-14B-15B-16B-17B-18B-19B-', 'SIZE': 278528}, 'ACK2RETURN': ' 6D-', 'HEIGHT': 1000, 'FULL2QUEUE': False, 'FLAGS': 0, 'ROWPITCH': 901, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'CODE3ERROR': 0, 'POINTER': <mmap.mmap object at 0x00000000041E03F8>, 'CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'NACK2RETURN': ' ', 'FUNCTION3CALLBACK': <function trace3image4apu at 0x00000000031B1848>, 'SLICEPITCH': 902, 'DATA3CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create3image4apu', 'METHOD': 'create3image4apu'}, 'COUNT3REFERENCE': 1, 'EVENT': {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2event4apu', 'METHOD': 'create2event4apu'}, 'FUNCTION2CALLBACK': <function trace2event4apu at 0x000000000315DCC8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-', 'QUEUE': <queue4xi.Queue4Xi object at 0x000000000465C630>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2commandqueue4apu', 'METHOD': 'create2commandqueue4apu'}, 'FUNCTION2CALLBACK': <function trace2commandqueue4apu at 0x00000000031517C8>, 'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-', 'INFO2PRIVATE': None, 'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C8A0B8>, 'NACK2RETURN': ' ', 'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4apu-failed ; time: 06:24:23 10/10/11 Pacific Daylight Time}', 'CLASS': 'create2context4apu', 'METHOD': 'create2context4apu'}, 'FUNCTION2CALLBACK': <function trace2context4apu at 0x000000000313E0C8>, 'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 'INFO2PRIVATE': None, 'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ', 'SIZE2PRIVATE': 0, 'PROPERTIES': None}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PROPERTIES': None, 'PIPE': (<read-write PipeConnection, handle 488>, <read-write PipeConnection, handle 492>)}, 'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK', 'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics cl_khr_local_int32_extended_atomics cl_khr_int64_base_atomics cl_khr_int64_extended_atomics cl_khr_byte_addressable_store cl_khr_gl_sharing cl_ext_device_fission cl_amd_device_attribute_query cl_amd_media_ops cl_amd_popcnt cl_amd_printf cl_khr_d3d10_sharing', 'AVAILABLE2COMPILER': 'ACK', 'MAX2DIMENSIONS4WORKITEMS': 3, 'SUPPORT2CORRECTION4ERROR': 0, 'BITS2ADDRESS': 64, 'AVAILABLE': 'ACK', 'SIZE2MEMORY4LOCAL': 32768, 'VENDOR': 'AuthenticAMD', 'SUPPORT2IMAGE': 'NACK', 'MAX2SIZE4KERNELARGUMENT': 4096, 'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-12A-13A-14A-15A-16A-17A-18A-19A-20A-21A-22A-23A-24A-25A-26A-27A-28A-29A-30A-31A-32A-33A-34A-35A-36A-37A-38A-39A-40A-41A-42A-43A-44A-45A-46A-47A-48A-49A-50A-51A-52A-53A-54A-55A-56A-57A-58A-59A-60A-', 'OUT-OF-ORDER2QUEUE': 'NACK', 'ALIGNMENT2ADDRESS4BASE': 1024, 'PROFILE': 'FULL_PROFILE', 'TYPE2MEMORY4LOCAL': 'Global', 'PREFERRED2WIDTHFLOAT4VECTOR': 4, 'PREFERRED2WIDTHLONG4VECTOR': 2, 'MAX2FREQUENCY4CLOCK': 3210, 'PREFERRED2WIDTHSHORT4VECTOR': 8, 'ID': 4033, 'MAX2WORKITEM2': 1024, 'NATIVE2WIDTHDOUBLE4VECTOR': 0, 'ROUND2ZERO': 'ACK', 'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD', 'MAX2WORKITEM1': 1024, 'MAX2WORKITEM0': 1024, 'SIZE2LINE4CACHE': 64, 'MAX2ALLOCATION4MEMORY': 1073741824, 'NATIVE2WIDTHCHAR4VECTOR': 16, 'QUIET2NANS': 'ACK', 'NATIVE2WIDTHSHORT4VECTOR': 8, 'ROUND2NEAREST4EVEN': 'ACK', 'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 'NACK2RETURN': ' ', 'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 'ENDIANESS2DEVICE': 'Little', 'MINIMUM2ALIGNMENT4DATATYPE': 128, 'ROUND2+VE3INFINITY': 'ACK', 'TYPE': 'CL_DEVICE_TYPE_UPU', 'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}, 'PIPE': (<read-write PipeConnection, handle 456>, <read-write PipeConnection, handle 460>)}}; time: 06:24:23 10/10/11 Pacific Daylight Time}
    
    return upu3image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "UPU3Image4Xi" base = "object" child = "None"
#   synopsis = "offers  methods for 3D UPU images" usage = "3D Image4Xi of UPU"
#>
# =================================================================================================
class UPU3Image4Xi(object): 
    ''' 
    UPU3Image4Xi offers  methods for 3D UPU images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable3upu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create3image',                    # ACK 5.31   p76    
            'copy3image',                      # ACK 5.31   p88
            'copy3imagetobuffer',              # ACK 5.3.4  p90
            'copy3buffertoimage',              # ACK 5.3.4  p92
            'read3image',                      # ACK 5.3.3  p84
            'write3image',                     # ACK 5.3.3  p84            
            'map3image',                       # ACK 5.3.5  p94            
            'get3imageinfo',                   # ACK 5.3.6  p97
            'get3supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get3imageinfo you can access any element by key such as 'FORMAT'   
            'retain3image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get3imageinfo
            'release3image',                   # ACK 5.4.1  p99
            'trace3image',                     # ACK modernized 
            'set3imagecallback',               # ACK modernized
            'set3image',                       # ACK modernized nb: with memorymap features you can process 3D images as files!                    
            'wait3image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-UPU3Image4Xi" usage = "
    #   print('(Step 5A/33) OF UPU3IMAGE4XI5TEST - CREATING 3D IMAGE OBJECT FOR UPU ', time4xi())     
    #   try:
    #       imageobject3upu = UPU3Image4Xi()
    #       print('(Step 5B/33) OF UPU3IMAGE4XI5TEST - CREATED 3D IMAGE OBJECT FOR UPU: ', imageobject3upu)                                  
    #   except:
    #       print('(Step 5Z/33) OF UPU3IMAGE4XI5TEST *** FAILED TO CREATE 3D IMAGE OBJECT FOR UPU')
    #">
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate3upu = None):
        '''
        _init__ method initializes UPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "UPU3Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-UPU3Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 

        message2print = "{(Step 1B/33) borning" + "; upu3image3xi-of-UPU3Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-upu3image3xi-of-UPU3Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable3upu-sharing-by-upu3image3xi-of-UPU3Image4Xi: " + str(UPU3Image4Xi.classvariable3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #           
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate3upu-passed-to-init-of-UPU3Image4Xi: " + str(task4imagecreate3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #                       
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate3upu-passing-to-create3image4upu-by-init-of-UPU3Image4Xi: " + str(task4imagecreate3upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        upu3image3xi = create3image4upu(task4imagecreate3upu)
        
        message2print = "{(Step 2B/33) performed" + "; create3image4upu-performed-by-init-of-UPU3Image4Xi: " + str(create3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; upu3image3xi-of-UPU3Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-upu3image3xi-of-UPU3Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable3upu-shared-by-upu3image3xi-of-UPU3Image4Xi: " + str(UPU3Image4Xi.classvariable3upu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #          
       
        message2print = "{(Step 3D/33) returning" + "; init-of-UPU3Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 23A/33) OF UPU3IMAGE4XI5TEST - TRYING TO CREATE UPU IMAGE', time4xi()) 
    #   try:
    #       xi3upu = UPU3Image4Xi()                            
    #       image3upu = xi3upu.create3image()                                  
    #       print('(Step 23B/33) OF UPU3IMAGE4XI5TEST - CREATED UPU IMAGE: ', image3upu)
    #   except:
    #       print('(Step 23Z/33) OF UPU3IMAGE4XI5TEST *** FAILED TO CREATE UPU IMAGE ***', time4xi())                  
    #                     
    #">   
    # ---------------------------------------------------------------------------------------------   
    def create3image(self, task4image2create = None):
        '''
        create3image method creates 3D image objects of UPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "create3image"
                     
        message2print = "{(Step 1A/33) started" + "; create3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-UPU3Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create3image4upu-by-create3image-of-UPU3Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #    
  
        upu3image3xi = create3image4upu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create3image4upu-performed-by-create3image-of-UPU3Image4Xi: " + str(create3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; upu3image3xi-returning-by-create3image-of-UPU3Image4Xi: " + str(upu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # 

        return upu3image3xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU3IMAGE4XI - TRYING TO SET 3D IMAGE FOR UPU')
    #   try:
    #       cc3xi = UPU3Image4Xi()                            
    #       image3upu = cc3xi.set3image()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - SET 3D IMAGE FOR UPU: ', image3upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO SET 3D IMAGE FOR UPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def set3image(self, task4set3image = None):
        '''
        sets 3D image UPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "set3image"
                     
        message2print = "{(Step 1A/33) started" + "; set3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set3image-passed-to-set3image-of-UPU3Image4Xi: " + str(task4set3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set3image-passing-to-set3image4upu-by-set3image-of-UPU3Image4Xi: " + str(task4set3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        upu3image3xi = set3image4upu(task4set3image)

        message2print = "{(Step 2B/33) performed" + "; set3image4upu-performed-by-set3image-of-UPU3Image4Xi: " + str(set3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu3image3xi-returning-by-set3image-of-UPU3Image4Xi: " + str(upu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #  
                
        return upu3image3xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get3imageinfo-of-UPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU3IMAGE4XI - TRYING TO GET UPU 3D IMAGE')
    #   try:
    #       cc3xi = UPU3Image4Xi()                            
    #       image3upu = cc3xi.get3imageinfo()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - GOT UPU 3D IMAGE: ', image3upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO GET UPU 3D IMAGE ***')   
    # ">
    # ---------------------------------------------------------------------------------------------   
    def get3imageinfo(self, task4getinfo3image=None):
        '''
        get3imageinfo method gets 3D objects of UPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "get3imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get3imageinfo-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        upu3image3xi = get3imageinfo4upu(task4getinfo3image)
        
        message2print = "{(Step 2A/33) returning" + "; upu3image3xi-returning-by-get3imageinfo-of-UPU3Image4Xi: " + str(upu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        return upu3image3xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get3supportedimageformats-of-UPU3Image4Xi" usage = "
    #   print('(Step 6B1/33) OF UPU3IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR UPU')
    #   try:
    #       cc3xi = UPU3Image4Xi()                            
    #       supportedimageformats3upu = cc3xi.get3supportedimageformats()                                  
    #       print('(Step 6B2/33) OF UPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR UPU: ', supportedimageformats3upu)
    #                                                   
    #   except:
    #       print('(Step 2Z/33) OF UPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR UPU***')     
    # ">
    # ---------------------------------------------------------------------------------------------   
    def get3supportedimageformats(self):
        '''
        get3supportedimageformats method gets supported imageformats by objects of UPU3Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "get3supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get3supportedimageformats-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats3upu = get3supportedimageformats4upu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats3upu-returning-by-get3imageinfo-of-UPU3Image4Xi: " + str(supportedimageformats3upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        return supportedimageformats3upu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read3upu = {}
    #   try:
    #       response4read3upu = xi3upu.read3image()
    #       print('(Step 127B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2UPU: ', response4read3upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read3image4upu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def read3image(self, task4read3image = None):
        '''
        reads UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "read3image"
                     
        message2print = "{(Step 1A/33) started" + "; read3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4read3image-passed-to-read3image-of-UPU3Image4Xi: " + str(task4read3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read3image-passing-to-read3image4upu-by-read3image-of-UPU3Image4Xi: " + str(task4read3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4readimage3upu = read3image4upu(task4read3image)

        message2print = "{(Step 2B/33) performed" + "; read3image4upu-performed-by-read3image-of-UPU3Image4Xi: " + str(read3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage3upu-returning-by-read3image-of-UPU3Image4Xi: " + str(response4readimage3upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
  
        return response4readimage3upu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   response4map3upu = {}
    #   try:
    #       response4map3upu = xi3upu.map3image()
    #       print('(Step 127B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2UPU: ', response4map3upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map3image4upu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def map3image(self, task4map3image = None):
        '''
        maps UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "map3image"
                     
        message2print = "{(Step 1A/33) started" + "; map3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4map3image-passed-to-map3image-of-UPU3Image4Xi: " + str(task4map3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map3image-passing-to-map3image4upu-by-map3image-of-UPU3Image4Xi: " + str(task4map3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4mapimage3upu = map3image4upu(task4map3image)

        message2print = "{(Step 2B/33) performed" + "; map3image4upu-performed-by-map3image-of-UPU3Image4Xi: " + str(map3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage3upu-returning-by-map3image-of-UPU3Image4Xi: " + str(response4mapimage3upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        
        return response4mapimage3upu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU3IMAGE4XI - TRYING TO RELEASE 3D IMAGE FOR UPU')
    #   try:
    #       cc3xi = UPU3Image4Xi()                            
    #       image3upu = cc3xi.release3image()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - RELEASED 3D IMAGE FOR UPU: ', image3upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO RELEASE 3D IMAGE FOR UPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def release3image(self, task4release3image = None):
        '''
        releases UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "release3image"
                     
        message2print = "{(Step 1A/33) started" + "; release3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release3image-passed-to-release3image-of-UPU3Image4Xi: " + str(task4release3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release3image-passing-to-release3image4upu-by-release3image-of-UPU3Image4Xi: " + str(task4release3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        upu3image3xi = release3image4upu(task4release3image)

        message2print = "{(Step 2B/33) performed" + "; release3image4upu-performed-by-release3image-of-UPU3Image4Xi: " + str(release3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu3image3xi-returning-by-release3image-of-UPU3Image4Xi: " + str(upu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return upu3image3xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU3IMAGE4XI - TRYING TO RETAIN 3D IMAGE FOR UPU')
    #   try:
    #       cc3xi = UPU3Image4Xi()                            
    #       image3upu = cc3xi.retain3image()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - RETAINED 3D IMAGE FOR UPU: ', image3upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO RETAIN 3D IMAGE FOR UPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def retain3image(self, task4retain3image = None):
        '''
        retains UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "retain3image"
                     
        message2print = "{(Step 1A/33) started" + "; retain3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain3image-of-UPU3Image4Xi: " + str(task4retain3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain3image4upu-by-retain-of-UPU3Image4Xi: " + str(task4retain3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        upu3image3xi = retain3image4upu(task4retain3image)

        message2print = "{(Step 2B/33) performed" + "; retain3image4upu-performed-by-retain3image-of-UPU3Image4Xi: " + str(retain3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu3image3xi-returning-by-retain3image-of-UPU3Image4Xi: " + str(upu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
              
                
        return upu3image3xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU3IMAGE4XI - TRYING TO WAIT UPU 3D IMAGE')
    #   try:
    #       cc3xi = UPU3Image4Xi()                            
    #       image3upu = cc3xi.wait3image()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - WAITED UPU 3D IMAGE: ', image3upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO WAIT UPU 3D IMAGE ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def wait3image(self, task4wait3image = None):
        '''
        waits UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "wait3image"
                     
        message2print = "{(Step 1A/33) started" + "; wait3image-of-UPU3Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait3image-passed-to-wait3image-of-UPU3Image4Xi: " + str(task4wait3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait3image4upu-by-wait3image-of-UPU3Image4Xi: " + str(task4wait3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        upu3image3xi = wait3image4upu(task4wait3image)

        message2print = "{(Step 2B/33) performed" + "; wait3image4upu-performed-by-wait3image-of-UPU3Image4Xi: " + str(wait3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; upu3image3xi-returning-by-wait3image-of-UPU3Image4Xi: " + str(upu3image3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
                
        return upu3image3xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write3upu = {}
    #   try:
    #       response4write3upu = xi3upu.write3image()
    #       print('(Step 127B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2UPU: ', response4write3upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   ... continue as with write3image4upu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def write3image(self, task4write3image = None):
        '''
        writes UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "write3image"
                     
        message2print = "{(Step 1A/33) started" + "; write3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4write3image-passed-to-write3image-of-UPU3Image4Xi: " + str(task4write3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write3image-passing-to-write3image4upu-by-write3image-of-UPU3Image4Xi: " + str(task4write3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4writeimage3upu = write3image4upu(task4write3image)

        message2print = "{(Step 2B/33) performed" + "; write3image4upu-performed-by-write3image-of-UPU3Image4Xi: " + str(write3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage3upu-returning-by-write3image-of-UPU3Image4Xi: " + str(response4writeimage3upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #    
                
        return response4writeimage3upu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy3image-of-UPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy3upu = {}
    #   try:
    #       response4copy3upu = xi3upu.copy3image()
    #       print('(Step 127B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2UPU: ', response4copy3upu) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   ... continue as with copy3image4upu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def copy3image(self, task4copy3image = None):
        '''
        copies UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "copy3image"
                     
        message2print = "{(Step 1A/33) started" + "; copy3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy3image-passed-to-copy3image-of-UPU3Image4Xi: " + str(task4copy3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #  
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy3image-passing-to-copy3image4upu-by-copy3image-of-UPU3Image4Xi: " + str(task4copy3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4copyimage3upu = copy3image4upu(task4copy3image)

        message2print = "{(Step 2B/33) performed" + "; copy3image4upu-performed-by-copy3image-of-UPU3Image4Xi: " + str(copy3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage3upu-returning-by-copy3image-of-UPU3Image4Xi: " + str(response4copyimage3upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
        
        return response4copyimage3upu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy3imagetobuffer-of-UPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi3upu.copy3imagetobuffer()
    #       print('(Step 127B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2UPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #   ... continue as with copy3imagetobuffer4upu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def copy3imagetobuffer(self, task4copy3imagetobuffer = None):
        '''
        copies 3D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "copy3imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy3imagetobuffer-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy3imagetobuffer-passed-to-copy3imagetobuffer-of-UPU3Image4Xi: " + str(task4copy3imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy3imagetobuffer-passing-to-copy3imagetobuffer4upu-by-copy3imagetobuffer-of-UPU3Image4Xi: " + str(task4copy3imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4copyimage3upu = copy3imagetobuffer4upu(task4copy3imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy3imagetobuffer4upu-performed-by-copy3imagetobuffer-of-UPU3Image4Xi: " + str(copy3imagetobuffer4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage3upu-returning-by-copy3imagetobuffer-of-UPU3Image4Xi: " + str(response4copyimage3upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
          
        return response4copyimage3upu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy3buffertoimage-of-UPU3Image4Xi" usage = "
    #   print('(Step 127A/700) OF UPU3IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi3upu.copy3buffertoimage()
    #       print('(Step 127B/700) OF UPU3IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2UPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF UPU3IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #   ... continue as with copy3buffertoimage4upu function
    #">
    # ---------------------------------------------------------------------------------------------   
    def copy3buffertoimage(self, task4copy3buffertoimage = None):
        '''
        copies 3D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "copy3buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy3buffertoimage-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy3buffertoimage-passed-to-copy3buffertoimage-of-UPU3Image4Xi: " + str(task4copy3buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy3buffertoimage-passing-to-copy3buffertoimage4upu-by-copy3buffertoimage-of-UPU3Image4Xi: " + str(task4copy3buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        response4copyimage3upu = copy3buffertoimage4upu(task4copy3buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy3buffertoimage4upu-performed-by-copy3buffertoimage-of-UPU3Image4Xi: " + str(copy3buffertoimage4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage3upu-returning-by-copy3buffertoimage-of-UPU3Image4Xi: " + str(response4copyimage3upu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 

                
        return response4copyimage3upu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace3image-of-UPU3Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF UPU3IMAGE4XI5TEST - TRYING TO TRACE UPU: ', time4xi())      
    #   print('=' *33)      
    #   trace3upu = None
    #   try:
    #       trace3upu = trace3image4upu() 
    #       print('(Step 15B/25) OF UPU3IMAGE4XI5TEST - TRACE3UPU: ', trace3upu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace3image4upu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace3image4upu', 'METHOD': 'trace3image4upu'}
    #    except:
    #       print('(Step 15Z1/25) OF UPU3IMAGE4XI5TEST *** FAILED TO TRACE UPU: ', trace3upu)     
    #">
    # ---------------------------------------------------------------------------------------------   
    def trace3image(self, task4trace3image = None):
        '''
        traces UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "trace3image"
                     
        message2print = "{(Step 1A/33) started" + "; trace3image-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace3image-passed-to-trace-of-UPU3Image4Xi: " + str(task4trace3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #  
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace3image-passing-to-trace3image4upu-by-trace3image-of-UPU3Image4Xi: " + str(task4trace3image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        trace3xi = trace3image4upu(task4trace3image)

        message2print = "{(Step 2B/33) performed" + "; trace3image4upu-performed-by-trace3image-of-UPU3Image4Xi: " + str(trace3image4upu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace3xi-returning-by-trace3image-of-UPU3Image4Xi: " + str(trace3xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                
        return trace3xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set3imagecallback-of-UPU3Image4Xi" usage = "
    #   print('(Step 1A/33) OF UPU3IMAGE4XI - TRYING TO SET 3D IMAGE FOR UPU')
    #   try:
    #       cc3xi = UPU3Image4Xi()                            
    #       image3upu = cc3xi.set3imagecallback()                                  
    #       print('(Step 1B/33) OF UPU4XI5TEST - SET 3D IMAGE FOR UPU: ', image3upu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF UPU4XI5TEST *** FAILED TO SET 3D IMAGE FOR UPU ***')  
    #">
    # ---------------------------------------------------------------------------------------------   
    def set3imagecallback(self, task4set3imagecallback = None): 
        '''
        set3imagecallback of UPU3Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "UPU3Image4Xi"
        method2print = "set3imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set3imagecallback-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4set3imagecallback-passed-to-set3imagecallback-of-UPU3Image4Xi: " + str(task4set3imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        #   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set3imagecallback-passing-to-set3imagecallback -by-set3imagecallback-of-UPU3Image4Xi: " + str(task4set3imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
        
        callback3upu4image = {} 
        try:
            callback3upu4image = set3imagecallback4upu (task4set3imagecallback)

            message2print = "{(Step 2B/33) callback3upu4image" + "; callback3upu4image-set-by-set3imagecallback-of-UPU3Image4Xi to: " + str(callback3upu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # 
            
        except:
            message2print = "{(Step 4A/33) failed" + "; callback3upu4image-failed-by-set3imagecallback-of-UPU3Image4Xi: " + str(callback3upu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback3upu4image-returning-by-set3imagecallback-of-UPU3Image4Xi: " + str(callback3upu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 
                                             
        return callback3upu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-UPU3Image4Xi" usage = "
    #   try:
    #       print(xi3upu.sense())
    #       print('(Step 24B/33) OF UPU3IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF UPU3IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses UPU3Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'UPU3Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; UPU3Image4Xi-sensing-by-sense-of-UPU3Image4Xi: " + str(UPU3Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #    
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-UPU3Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable3upu-of-UPU3Image4Xi'] = UPU3Image4Xi.classvariable3upu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-UPU3Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; UPU3Image4Xi-sensed-by-sense-of-UPU3Image4Xi: " + str(UPU3Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-UPU3Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-UPU3Image4Xi" usage = "
    #   print('(Step 25A/33) OF UPU3IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi3upu.reason())
    #       print('(Step 25B/33) OF UPU3IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF UPU3IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    # ">
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons UPU3Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'UPU3Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; UPU3Image4Xi-reasoning-by-reason-of-UPU3Image4Xi: " + str(UPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-UPU3Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-UPU3Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-UPU3Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; UPU3Image4Xi-reasoned-by-reason-of-UPU3Image4Xi: " + str(UPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-UPU3Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        #               
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-UPU3Image4Xi" usage = "
    #   print('(Step 26A/33) OF UPU3IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi3upu.ace())
    #       print('(Step 26B/33) OF UPU3IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF UPU3IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    # ">
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces UPU3Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'UPU3Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-UPU3Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; UPU3Image4Xi-acing-by-ace-of-UPU3Image4Xi: " + str(UPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #       
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-UPU3Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-UPU3Image4Xi'] = UPU3Image4Xi.classvariable3upu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-UPU3Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; UPU3Image4Xi-aced-by-ace-of-UPU3Image4Xi: " + str(UPU3Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-UPU3Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   upu3image4xi5test.txt
#   sense-upu3image4xi5test.xml
#   reason-upu3image4xi5test.xml
#   ace-upu3image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of upu3image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
